@@ -4,11 +4,14 @@
 from weakref import WeakSet
 
 __all__ = [
+    'aio',
     'auth',
     'crypto',
     'http',
     'io',
     'mqtt',
+    'mqtt5',
+    'mqtt_request_response',
     's3',
     'websocket',
 ]
