@@ -4,27 +4,22 @@ set -ex
 
 /opt/python/cp39-cp39/bin/python ./continuous-delivery/update-version.py
 
-/opt/python/cp38-cp38/bin/python setup.py sdist bdist_wheel
+/opt/python/cp38-cp38/bin/python -m build
 auditwheel repair --plat musllinux_1_1_x86_64 dist/awscrt-*cp38*.whl
 
-/opt/python/cp39-cp39/bin/python setup.py sdist bdist_wheel
+/opt/python/cp39-cp39/bin/python -m build
 auditwheel repair --plat musllinux_1_1_x86_64 dist/awscrt-*cp39*.whl
 
-/opt/python/cp310-cp310/bin/python setup.py sdist bdist_wheel
+/opt/python/cp310-cp310/bin/python -m build
 auditwheel repair --plat musllinux_1_1_x86_64 dist/awscrt-*cp310*.whl
 
-/opt/python/cp311-cp311/bin/python setup.py sdist bdist_wheel
+/opt/python/cp311-cp311/bin/python -m build
 auditwheel repair --plat musllinux_1_1_x86_64 dist/awscrt-*cp311*.whl
 
-# Don't need to build wheels for Python 3.12.
-# The 3.11 wheel uses the stable ABI, so it works with newer versions too.
+# Don't need to build wheels for Python 3.12, it works with the 3.11 stable ABI wheel
 
 # We are using the Python 3.13 stable ABI from Python 3.13 onwards because of deprecated functions.
-# Manylinux images don't contain setuptools from Python 3.13, so we need to install it.
-# Install in a custom location due to access issues.
-/opt/python/cp313-cp313/bin/python -m pip install --target ./local -r requirements-dev.txt
-export PYTHONPATH=./local:$PYTHONPATH
-/opt/python/cp313-cp313/bin/python setup.py sdist bdist_wheel
+/opt/python/cp313-cp313/bin/python -m build
 auditwheel repair --plat musllinux_1_1_x86_64 dist/awscrt-*cp313*.whl
 
 rm dist/*.whl
