@@ -527,9 +527,7 @@ static void s_on_stream_complete_fn(struct aws_http_stream *stream, int error_co
     struct sts_web_identity_user_data *user_data = data;
 
     struct aws_credentials_provider_sts_web_identity_impl *impl = user_data->sts_web_identity_provider->impl;
-    struct aws_http_connection *connection = impl->function_table->aws_http_stream_get_connection(stream);
     impl->function_table->aws_http_stream_release(stream);
-    impl->function_table->aws_http_connection_manager_release_connection(impl->connection_manager, connection);
 
     /*
      * On anything other than a 200, if we can retry the request based on
@@ -886,6 +884,7 @@ on_error:
 }
 
 AWS_STATIC_STRING_FROM_LITERAL(s_sts_service_name, "sts");
+AWS_STATIC_STRING_FROM_LITERAL(s_sts_service_env_name, "STS");
 
 static int s_generate_uuid_to_buf(struct aws_allocator *allocator, struct aws_byte_buf *dst) {
 
@@ -1034,8 +1033,15 @@ static struct sts_web_identity_parameters *s_parameters_new(
     }
 
     /* determine endpoint */
-    if (aws_credentials_provider_construct_regional_endpoint(
-            allocator, &parameters->endpoint, region, s_sts_service_name)) {
+    if (aws_credentials_provider_construct_endpoint(
+            allocator,
+            &parameters->endpoint,
+            region,
+            s_sts_service_name,
+            s_sts_service_env_name,
+            s_sts_service_name,
+            config_profile,
+            profile)) {
         AWS_LOGF_ERROR(
             AWS_LS_AUTH_CREDENTIALS_PROVIDER, "Failed to construct sts endpoint with, probably region is missing.");
         goto on_finish;
