@@ -386,15 +386,111 @@ static struct aws_byte_cursor s_aws_isoe_dns_suffix = AWS_BYTE_CUR_INIT_FROM_STR
 static struct aws_byte_cursor s_isof_region_prefix = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("us-isof-");
 static struct aws_byte_cursor s_aws_isof_dns_suffix = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("csp.hci.ic.gov");
 
-int aws_credentials_provider_construct_regional_endpoint(
+AWS_STATIC_STRING_FROM_LITERAL(s_endpoint_url_env, "AWS_ENDPOINT_URL");
+AWS_STATIC_STRING_FROM_LITERAL(s_endpoint_url_property, "endpoint_url");
+AWS_STATIC_STRING_FROM_LITERAL(s_services_property, "services");
+
+/*
+ * Resolve any endpoint overrides for the service
+ * See: https://docs.aws.amazon.com/sdkref/latest/guide/feature-ss-endpoints.html
+ *
+ * The order of resolution for configured endpoint is as follows:
+ * 1. The value provided by a service-specific environment variable, `AWS_ENDPOINT_URL_<SERVICE>`.
+ * 2. The value provided by the global endpoint environment variable, `AWS_ENDPOINT_URL`.
+ * 3. The value provided by a service-specific parameter from a services definition section referenced in a profile.
+ * 4. The value provided by the global parameter from a profile in the shared configuration file.
+ *
+ * Returns NULL if there are no endpoint overrides configured
+ */
+struct aws_string *s_get_override_endpoint(
+    struct aws_allocator *allocator,
+    const struct aws_string *override_service_name_env,
+    const struct aws_string *override_service_name_property,
+    const struct aws_profile_collection *profile_collection,
+    const struct aws_profile *profile) {
+
+    struct aws_byte_cursor url_env_cursor = aws_byte_cursor_from_string(s_endpoint_url_env);
+    struct aws_byte_cursor underscore_cursor = aws_byte_cursor_from_c_str("_");
+    struct aws_byte_cursor service_name_env_cursor = aws_byte_cursor_from_string(override_service_name_env);
+
+    struct aws_string *out_endpoint = NULL;
+    struct aws_string *service_endpoint_str = NULL;
+
+    struct aws_byte_buf service_endpoint_buf;
+    AWS_ZERO_STRUCT(service_endpoint_buf);
+    /* Check AWS_ENDPOINT_URL_SERVICENAME variable */
+    aws_byte_buf_init(&service_endpoint_buf, allocator, 10);
+    if (aws_byte_buf_append_dynamic(&service_endpoint_buf, &url_env_cursor) ||
+        aws_byte_buf_append_dynamic(&service_endpoint_buf, &underscore_cursor) ||
+        aws_byte_buf_append_dynamic(&service_endpoint_buf, &service_name_env_cursor)) {
+        goto on_finish;
+    }
+    service_endpoint_str = aws_string_new_from_buf(allocator, &service_endpoint_buf);
+    out_endpoint = aws_get_env_nonempty(allocator, aws_string_c_str(service_endpoint_str));
+    if (out_endpoint) {
+        goto on_finish;
+    }
+
+    /* Check AWS_ENDPOINT_URL variable */
+    out_endpoint = aws_get_env_nonempty(allocator, aws_string_c_str(s_endpoint_url_env));
+    if (out_endpoint) {
+        goto on_finish;
+    }
+
+    /* parse endpoint override from config file */
+    if (profile_collection == NULL || profile == NULL) {
+        goto on_finish;
+    }
+
+    /* check services specific endpoint property */
+    const struct aws_profile_property *property = aws_profile_get_property(profile, s_services_property);
+    if (property) {
+        const struct aws_profile *services = aws_profile_collection_get_section(
+            profile_collection, AWS_PROFILE_SECTION_TYPE_SERVICES, aws_profile_property_get_value(property));
+        if (services) {
+            property = aws_profile_get_property(services, override_service_name_property);
+            if (property) {
+                out_endpoint = aws_string_new_from_string(
+                    allocator, aws_profile_property_get_sub_property(property, s_endpoint_url_property));
+                goto on_finish;
+            }
+        }
+        goto on_finish;
+    }
+
+    /* check aws_endpoint_url property */
+    property = aws_profile_get_property(profile, s_endpoint_url_property);
+    if (property) {
+        out_endpoint = aws_string_new_from_string(allocator, aws_profile_property_get_value(property));
+
+        goto on_finish;
+    }
+
+on_finish:
+    aws_byte_buf_clean_up(&service_endpoint_buf);
+    aws_string_destroy(service_endpoint_str);
+    return out_endpoint;
+}
+
+int aws_credentials_provider_construct_endpoint(
     struct aws_allocator *allocator,
     struct aws_string **out_endpoint,
     const struct aws_string *region,
-    const struct aws_string *service_name) {
+    const struct aws_string *service_name_host,
+    const struct aws_string *service_name_env,
+    const struct aws_string *service_name_property,
+    const struct aws_profile_collection *profile_collection,
+    const struct aws_profile *profile) {
+
+    *out_endpoint =
+        s_get_override_endpoint(allocator, service_name_env, service_name_property, profile_collection, profile);
+    if (*out_endpoint) {
+        return AWS_OP_SUCCESS;
+    }
 
     AWS_PRECONDITION(allocator);
     AWS_PRECONDITION(out_endpoint);
-    if (!region || !service_name) {
+    if (!region || !service_name_host) {
         return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
     }
     int result = AWS_OP_ERR;
@@ -402,7 +498,7 @@ int aws_credentials_provider_construct_regional_endpoint(
     struct aws_byte_buf endpoint;
     AWS_ZERO_STRUCT(endpoint);
     aws_byte_buf_init(&endpoint, allocator, 10);
-    struct aws_byte_cursor service_cursor = aws_byte_cursor_from_string(service_name);
+    struct aws_byte_cursor service_cursor = aws_byte_cursor_from_string(service_name_host);
     struct aws_byte_cursor region_cursor = aws_byte_cursor_from_string(region);
 
     if (aws_byte_buf_append_dynamic(&endpoint, &service_cursor) ||
@@ -455,16 +551,14 @@ AWS_STATIC_STRING_FROM_LITERAL(s_region_env, "AWS_REGION");
 AWS_STATIC_STRING_FROM_LITERAL(s_default_region_env, "AWS_DEFAULT_REGION");
 
 struct aws_string *aws_credentials_provider_resolve_region_from_env(struct aws_allocator *allocator) {
-    struct aws_string *region = NULL;
-
     /* check AWS_REGION environment variable first */
-    aws_get_environment_value(allocator, s_region_env, &region);
-    if (region != NULL && region->len > 0) {
+    struct aws_string *region = aws_get_env_nonempty(allocator, aws_string_c_str(s_region_env));
+    if (region != NULL) {
         return region;
     }
 
-    aws_get_environment_value(allocator, s_default_region_env, &region);
-    return region;
+    /* check AWS_DEFAULT_REGION environment variable first */
+    return aws_get_env_nonempty(allocator, aws_string_c_str(s_default_region_env));
 }
 
 struct aws_byte_cursor aws_parse_account_id_from_arn(struct aws_byte_cursor arn) {
