@@ -28,6 +28,9 @@
 AWS_STATIC_STRING_FROM_LITERAL(s_access_key_id_test_value, "My Access Key");
 AWS_STATIC_STRING_FROM_LITERAL(s_secret_access_key_test_value, "SekritKey");
 AWS_STATIC_STRING_FROM_LITERAL(s_session_token_test_value, "Some Session Token");
+AWS_STATIC_STRING_FROM_LITERAL(s_account_id_test_value, "Some Account Value");
+
+AWS_STATIC_STRING_FROM_LITERAL(s_account_id_env_var, "AWS_ACCOUNT_ID");
 
 static int s_credentials_create_destroy_test(struct aws_allocator *allocator, void *ctx) {
     (void)ctx;
@@ -115,7 +118,8 @@ static int s_do_basic_provider_test(
     int expected_calls,
     const struct aws_string *expected_access_key_id,
     const struct aws_string *expected_secret_access_key,
-    const struct aws_string *expected_session_token) {
+    const struct aws_string *expected_session_token,
+    const struct aws_string *expected_account_id) {
 
     struct aws_get_credentials_test_callback_result callback_results;
     aws_get_credentials_test_callback_result_init(&callback_results, expected_calls);
@@ -141,6 +145,13 @@ static int s_do_basic_provider_test(
         } else {
             ASSERT_TRUE(aws_credentials_get_session_token(callback_results.credentials).len == 0);
         }
+
+        if (expected_account_id != NULL) {
+            ASSERT_CURSOR_VALUE_STRING_EQUALS(
+                aws_credentials_get_account_id(callback_results.credentials), expected_account_id);
+        } else {
+            ASSERT_TRUE(aws_credentials_get_account_id(callback_results.credentials).len == 0);
+        }
     } else {
         ASSERT_TRUE(expected_access_key_id == NULL);
         ASSERT_TRUE(expected_secret_access_key == NULL);
@@ -159,6 +170,7 @@ static int s_static_credentials_provider_basic_test(struct aws_allocator *alloca
         .access_key_id = aws_byte_cursor_from_string(s_access_key_id_test_value),
         .secret_access_key = aws_byte_cursor_from_string(s_secret_access_key_test_value),
         .session_token = aws_byte_cursor_from_string(s_session_token_test_value),
+        .account_id = aws_byte_cursor_from_string(s_account_id_test_value),
         .shutdown_options =
             {
                 .shutdown_callback = s_on_shutdown_complete,
@@ -172,8 +184,12 @@ static int s_static_credentials_provider_basic_test(struct aws_allocator *alloca
 
     ASSERT_TRUE(
         s_do_basic_provider_test(
-            provider, 1, s_access_key_id_test_value, s_secret_access_key_test_value, s_session_token_test_value) ==
-        AWS_OP_SUCCESS);
+            provider,
+            1,
+            s_access_key_id_test_value,
+            s_secret_access_key_test_value,
+            s_session_token_test_value,
+            s_account_id_test_value) == AWS_OP_SUCCESS);
 
     aws_credentials_provider_release(provider);
 
@@ -198,7 +214,7 @@ static int s_anonymous_credentials_provider_basic_test(struct aws_allocator *all
 
     struct aws_credentials_provider *provider = aws_credentials_provider_new_anonymous(allocator, &shutdown_options);
 
-    ASSERT_TRUE(s_do_basic_provider_test(provider, 1, NULL, NULL, NULL) == AWS_OP_SUCCESS);
+    ASSERT_TRUE(s_do_basic_provider_test(provider, 1, NULL, NULL, NULL, NULL) == AWS_OP_SUCCESS);
 
     aws_credentials_provider_release(provider);
 
@@ -236,8 +252,23 @@ static int s_environment_credentials_provider_basic_test(struct aws_allocator *a
 
     ASSERT_TRUE(
         s_do_basic_provider_test(
-            provider, 1, s_access_key_id_test_value, s_secret_access_key_test_value, s_session_token_test_value) ==
-        AWS_OP_SUCCESS);
+            provider,
+            1,
+            s_access_key_id_test_value,
+            s_secret_access_key_test_value,
+            s_session_token_test_value,
+            NULL) == AWS_OP_SUCCESS);
+
+    aws_set_environment_value(s_account_id_env_var, s_account_id_test_value);
+
+    ASSERT_TRUE(
+        s_do_basic_provider_test(
+            provider,
+            1,
+            s_access_key_id_test_value,
+            s_secret_access_key_test_value,
+            s_session_token_test_value,
+            s_account_id_test_value) == AWS_OP_SUCCESS);
 
     aws_credentials_provider_release(provider);
 
@@ -270,7 +301,7 @@ static int s_environment_credentials_provider_empty_env_test(struct aws_allocato
 
     struct aws_credentials_provider *provider = aws_credentials_provider_new_environment(allocator, &options);
     /* Instead of getting an empty credentials, should just fail to fetch credentials */
-    ASSERT_TRUE(s_do_basic_provider_test(provider, 1, NULL, NULL, NULL) == AWS_OP_SUCCESS);
+    ASSERT_TRUE(s_do_basic_provider_test(provider, 1, NULL, NULL, NULL, NULL) == AWS_OP_SUCCESS);
 
     aws_credentials_provider_release(provider);
 
@@ -300,7 +331,7 @@ static int s_do_environment_credentials_provider_failure(struct aws_allocator *a
 
     struct aws_credentials_provider *provider = aws_credentials_provider_new_environment(allocator, &options);
 
-    ASSERT_TRUE(s_do_basic_provider_test(provider, 1, NULL, NULL, NULL) == AWS_OP_SUCCESS);
+    ASSERT_TRUE(s_do_basic_provider_test(provider, 1, NULL, NULL, NULL, NULL) == AWS_OP_SUCCESS);
 
     aws_credentials_provider_release(provider);
 
@@ -883,6 +914,11 @@ AWS_STATIC_STRING_FROM_LITERAL(
 AWS_STATIC_STRING_FROM_LITERAL(
     s_config_contents2,
     "[profile default]\naws_access_key_id=fake_access_key2\naws_secret_access_key=fake_secret_key2\n");
+AWS_STATIC_STRING_FROM_LITERAL(
+    s_account_id_config_contents,
+    "[profile "
+    "default]\naws_access_key_id=fake_access_key\naws_secret_access_key=fake_secret_key\naws_account_id=fake_account_"
+    "id");
 
 AWS_STATIC_STRING_FROM_LITERAL(
     s_credentials_contents,
@@ -893,6 +929,7 @@ AWS_STATIC_STRING_FROM_LITERAL(
 
 AWS_STATIC_STRING_FROM_LITERAL(s_fake_access, "fake_access_key");
 AWS_STATIC_STRING_FROM_LITERAL(s_fake_secret, "fake_secret_key");
+AWS_STATIC_STRING_FROM_LITERAL(s_fake_account_id, "fake_account_id");
 
 int s_verify_default_credentials_callback(struct aws_get_credentials_test_callback_result *callback_results) {
     ASSERT_TRUE(callback_results->count == 1);
@@ -939,6 +976,53 @@ static int s_profile_credentials_provider_default_test(struct aws_allocator *all
 
 AWS_TEST_CASE(profile_credentials_provider_default_test, s_profile_credentials_provider_default_test);
 
+int s_verify_account_id_credentials_callback(struct aws_get_credentials_test_callback_result *callback_results) {
+    ASSERT_TRUE(callback_results->count == 1);
+    ASSERT_TRUE(callback_results->credentials != NULL);
+    ASSERT_CURSOR_VALUE_STRING_EQUALS(aws_credentials_get_access_key_id(callback_results->credentials), s_fake_access);
+    ASSERT_CURSOR_VALUE_STRING_EQUALS(
+        aws_credentials_get_secret_access_key(callback_results->credentials), s_fake_secret);
+    ASSERT_CURSOR_VALUE_STRING_EQUALS(aws_credentials_get_account_id(callback_results->credentials), s_fake_account_id);
+    ASSERT_TRUE(aws_credentials_get_session_token(callback_results->credentials).len == 0);
+
+    return AWS_OP_SUCCESS;
+}
+
+static int s_profile_credentials_provider_account_id_test(struct aws_allocator *allocator, void *ctx) {
+    (void)ctx;
+    (void)s_account_id_config_contents;
+
+    struct aws_string *config_file_str = aws_create_process_unique_file_name(allocator);
+    struct aws_string *creds_file_str = aws_create_process_unique_file_name(allocator);
+
+    struct aws_credentials_provider_profile_options options = {
+        .config_file_name_override = aws_byte_cursor_from_string(config_file_str),
+        .credentials_file_name_override = aws_byte_cursor_from_string(creds_file_str),
+        .shutdown_options =
+            {
+                .shutdown_callback = s_on_shutdown_complete,
+                .shutdown_user_data = NULL,
+            },
+    };
+
+    ASSERT_SUCCESS(s_do_credentials_provider_profile_test(
+        allocator,
+        config_file_str,
+        s_account_id_config_contents,
+        creds_file_str,
+        s_credentials_contents,
+        &options,
+        s_verify_account_id_credentials_callback,
+        true));
+
+    aws_string_destroy(config_file_str);
+    aws_string_destroy(creds_file_str);
+
+    return AWS_OP_SUCCESS;
+}
+
+AWS_TEST_CASE(profile_credentials_provider_account_id_test, s_profile_credentials_provider_account_id_test);
+
 static int s_profile_credentials_provider_cached_test(struct aws_allocator *allocator, void *ctx) {
     (void)ctx;
 
@@ -1338,8 +1422,12 @@ static int s_credentials_provider_default_test(struct aws_allocator *allocator,
 
     ASSERT_TRUE(
         s_do_basic_provider_test(
-            provider, 1, s_access_key_id_test_value, s_secret_access_key_test_value, s_session_token_test_value) ==
-        AWS_OP_SUCCESS);
+            provider,
+            1,
+            s_access_key_id_test_value,
+            s_secret_access_key_test_value,
+            s_session_token_test_value,
+            NULL) == AWS_OP_SUCCESS);
 
     /*
      * Verify that there's some caching before the environment by modifying the environment and requerying
@@ -1350,8 +1438,12 @@ static int s_credentials_provider_default_test(struct aws_allocator *allocator,
 
     ASSERT_TRUE(
         s_do_basic_provider_test(
-            provider, 1, s_access_key_id_test_value, s_secret_access_key_test_value, s_session_token_test_value) ==
-        AWS_OP_SUCCESS);
+            provider,
+            1,
+            s_access_key_id_test_value,
+            s_secret_access_key_test_value,
+            s_session_token_test_value,
+            NULL) == AWS_OP_SUCCESS);
 
     aws_credentials_provider_release(provider);
 
