@@ -26,7 +26,7 @@ struct aws_der_decoder {
     int tlv_idx;                  /* index to elements after parsing */
     struct aws_byte_cursor input; /* input buffer */
     uint32_t depth;               /* recursion depth when expanding containers */
-    struct der_tlv *container;    /* currently expanding container */
+    uint64_t container_index;     /* currently expanding container index in the list `tlvs` */
 };
 
 struct der_tlv {
@@ -80,16 +80,22 @@ static int s_der_read_tlv(struct aws_byte_cursor *cur, struct der_tlv *tlv) {
     if (len_bytes & 0x80) {
         len_bytes &= 0x7f;
         switch (len_bytes) {
-            case 1:
-                if (!aws_byte_cursor_read_u8(cur, (uint8_t *)&len)) {
+            case 1: {
+                uint8_t len_u8;
+                if (!aws_byte_cursor_read_u8(cur, &len_u8)) {
                     return aws_raise_error(AWS_ERROR_CAL_MALFORMED_ASN1_ENCOUNTERED);
                 }
+                len = len_u8;
                 break;
-            case 2:
-                if (!aws_byte_cursor_read_be16(cur, (uint16_t *)&len)) {
+            }
+            case 2: {
+                uint16_t len_u16;
+                if (!aws_byte_cursor_read_be16(cur, &len_u16)) {
                     return aws_raise_error(AWS_ERROR_CAL_MALFORMED_ASN1_ENCOUNTERED);
                 }
+                len = len_u16;
                 break;
+            }
             case 4:
                 if (!aws_byte_cursor_read_be32(cur, &len)) {
                     return aws_raise_error(AWS_ERROR_CAL_MALFORMED_ASN1_ENCOUNTERED);
@@ -204,7 +210,14 @@ static int s_der_write_tlv(struct der_tlv *tlv, struct aws_byte_buf *buf) {
         case AWS_DER_NULL:
             /* No value bytes */
             break;
-        default:
+        default: {
+            if (tlv->tag & AWS_DER_CLASS_CONTEXT) {
+                if (!aws_byte_buf_write(buf, tlv->value, tlv->length)) {
+                    return aws_raise_error(AWS_ERROR_INVALID_BUFFER_SIZE);
+                }
+                break;
+            }
+        }
             return aws_raise_error(AWS_ERROR_CAL_MISMATCHED_DER_TYPE);
     }
 
@@ -291,6 +304,17 @@ int aws_der_encoder_write_octet_string(struct aws_der_encoder *encoder, struct a
     return s_der_write_tlv(&tlv, encoder->buffer);
 }
 
+int aws_der_encoder_write_object_identifier(struct aws_der_encoder *encoder, struct aws_byte_cursor bytes) {
+    AWS_FATAL_ASSERT(bytes.len <= UINT32_MAX);
+    struct der_tlv tlv = {
+        .tag = AWS_DER_OBJECT_IDENTIFIER,
+        .length = (uint32_t)bytes.len,
+        .value = bytes.ptr,
+    };
+
+    return s_der_write_tlv(&tlv, encoder->buffer);
+}
+
 static int s_der_encoder_begin_container(struct aws_der_encoder *encoder, enum aws_der_type type) {
     struct aws_byte_buf *seq_buf = aws_mem_acquire(encoder->allocator, sizeof(struct aws_byte_buf));
     AWS_FATAL_ASSERT(seq_buf);
@@ -344,6 +368,25 @@ int aws_der_encoder_end_sequence(struct aws_der_encoder *encoder) {
     return s_der_encoder_end_container(encoder);
 }
 
+AWS_CAL_API int aws_der_encoder_begin_context_aware_tag(
+    struct aws_der_encoder *encoder,
+    bool is_constructed,
+    uint64_t tag_value) {
+    static uint64_t tag_mask = 0x1f; /* 5bit mask */
+    if (tag_value >= tag_mask) {
+        return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+    }
+
+    enum aws_der_type constructed_type =
+        AWS_DER_CLASS_CONTEXT | (is_constructed ? AWS_DER_FORM_CONSTRUCTED : 0) | (tag_value & tag_mask);
+
+    return s_der_encoder_begin_container(encoder, constructed_type);
+}
+
+AWS_CAL_API int aws_der_encoder_end_context_aware_tag(struct aws_der_encoder *encoder) {
+    return s_der_encoder_end_container(encoder);
+}
+
 int aws_der_encoder_begin_set(struct aws_der_encoder *encoder) {
     return s_der_encoder_begin_container(encoder, AWS_DER_SET);
 }
@@ -377,7 +420,7 @@ struct aws_der_decoder *aws_der_decoder_new(struct aws_allocator *allocator, str
     decoder->input = input;
     decoder->tlv_idx = -1;
     decoder->depth = 0;
-    decoder->container = NULL;
+    decoder->container_index = UINT64_MAX; /* no container is being expanded */
     if (aws_array_list_init_dynamic(&decoder->tlvs, decoder->allocator, 16, sizeof(struct der_tlv))) {
         goto error;
     }
@@ -394,6 +437,15 @@ error:
     return NULL;
 }
 
+struct aws_der_decoder *aws_der_decoder_nested_tlv_decoder(struct aws_der_decoder *decoder) {
+    struct aws_byte_cursor cursor;
+    AWS_ZERO_STRUCT(cursor);
+    if (aws_der_decoder_tlv_string(decoder, &cursor)) {
+        return NULL;
+    }
+    return aws_der_decoder_new(decoder->allocator, cursor);
+}
+
 void aws_der_decoder_destroy(struct aws_der_decoder *decoder) {
     if (!decoder) {
         return;
@@ -421,15 +473,20 @@ int s_parse_cursor(struct aws_der_decoder *decoder, struct aws_byte_cursor cur)
         if (aws_array_list_push_back(&decoder->tlvs, &tlv)) {
             return aws_raise_error(AWS_ERROR_INVALID_STATE);
         }
-        if (decoder->container) {
-            decoder->container->count++;
+        if (decoder->container_index != UINT64_MAX) {
+            struct der_tlv *container = NULL;
+            aws_array_list_get_at_ptr(&decoder->tlvs, (void **)&container, (size_t)decoder->container_index);
+            if (!container) {
+                return aws_raise_error(AWS_ERROR_INVALID_STATE);
+            }
+            container->count++;
         }
         /* if the last element was a container, expand it recursively to maintain order */
         if (tlv.tag & AWS_DER_FORM_CONSTRUCTED) {
-            struct der_tlv *outer_container = decoder->container;
+            uint64_t outer_container_index = decoder->container_index;
             struct der_tlv *container = NULL;
             aws_array_list_get_at_ptr(&decoder->tlvs, (void **)&container, decoder->tlvs.length - 1);
-            decoder->container = container;
+            decoder->container_index = decoder->tlvs.length - 1;
 
             if (!container) {
                 return aws_raise_error(AWS_ERROR_INVALID_STATE);
@@ -439,7 +496,7 @@ int s_parse_cursor(struct aws_der_decoder *decoder, struct aws_byte_cursor cur)
             if (s_parse_cursor(decoder, container_cur)) {
                 return aws_raise_error(AWS_ERROR_CAL_MALFORMED_ASN1_ENCOUNTERED);
             }
-            decoder->container = outer_container; /* restore the container stack */
+            decoder->container_index = outer_container_index; /* restore the container stack */
         }
     }
 
@@ -456,6 +513,10 @@ bool aws_der_decoder_next(struct aws_der_decoder *decoder) {
     return (++decoder->tlv_idx < (int)decoder->tlvs.length);
 }
 
+void aws_der_decoder_reset(struct aws_der_decoder *decoder) {
+    decoder->tlv_idx = -1;
+}
+
 static struct der_tlv s_decoder_tlv(struct aws_der_decoder *decoder) {
     AWS_FATAL_ASSERT(decoder->tlv_idx < (int)decoder->tlvs.length);
     struct der_tlv tlv = {0};
