@@ -11,7 +11,7 @@
 #include <aws/io/uri.h>
 #include <aws/testing/aws_test_harness.h>
 
-#if _MSC_VER
+#ifdef _MSC_VER
 #    pragma warning(disable : 4204) /* non-constant aggregate initializer */
 #endif
 
