@@ -17,7 +17,7 @@
 #include "preamble.h"
 
 static void s_cancel_task(struct aws_event_loop *event_loop, struct aws_task *task) {
-    AWS_LOGF_TRACE(AWS_LS_IO_EVENT_LOOP, "id=%p: cancelling task %p", (void *)event_loop, (void *)task);
+    AWS_LOGF_TRACE(AWS_LS_IO_EVENT_LOOP, "id=%p: cancelling %s task %p", (void *)event_loop, task->type_tag, (void *)task);
     struct epoll_loop *epoll_loop = event_loop->impl_data;
     aws_task_scheduler_cancel_task(&epoll_loop->scheduler, task);
 }
