@@ -184,6 +184,7 @@ struct aws_event_loop *aws_event_loop_new_with_epoll(
 
     loop->impl_data = epoll_loop;
     loop->vtable = &s_vtable;
+    loop->base_elg = options->parent_elg;
 
     _(wrap(&epoll_loop->task_pre_queue.head))
     _(wrap(&epoll_loop->task_pre_queue.tail))
@@ -249,11 +250,17 @@ clean_up_loop:
     return NULL;
 }
 
+static void s_start_destroy(struct aws_event_loop *event_loop
+    _(ghost \claim(c_event_loop)) _(ghost \claim(c_mutex))
+) {
+    (void)event_loop;
+}
+
 /* Fake-up call to s_stop since this is just a vtable lookup */
 #define aws_event_loop_stop(event_loop) \
     s_stop(event_loop _(ghost c_event_loop) _(ghost c_mutex));
 
-static void s_destroy(struct aws_event_loop *event_loop
+static void s_complete_destroy(struct aws_event_loop *event_loop
     _(ghost \claim(c_event_loop)) _(ghost \claim(c_mutex))
 ) {
     AWS_LOGF_INFO(AWS_LS_IO_EVENT_LOOP, "id=%p: Destroying event_loop", (void *)event_loop);
