@@ -228,19 +228,18 @@ static void s_write_correlation_token_string(struct aws_byte_cursor scratch_spac
 }
 
 static void s_on_get_shadow_complete(
-    const struct aws_byte_cursor *response_topic,
-    const struct aws_byte_cursor *payload,
+    const struct aws_mqtt_rr_incoming_publish_event *publish_event,
     int error_code,
     void *user_data) {
 
     struct aws_string *correlation_token = user_data;
 
-    if (payload != NULL) {
+    if (publish_event != NULL) {
         printf(
             "GetNamedShadow request '%s' response received on topic '" PRInSTR "' with body:\n  " PRInSTR "\n\n",
             correlation_token->bytes,
-            AWS_BYTE_CURSOR_PRI(*response_topic),
-            AWS_BYTE_CURSOR_PRI(*payload));
+            AWS_BYTE_CURSOR_PRI(publish_event->topic),
+            AWS_BYTE_CURSOR_PRI(publish_event->payload));
     } else {
         printf(
             "GetNamedShadow request '%s' failed with error code %d(%s)\n\n",
@@ -350,19 +349,18 @@ static void s_handle_get(struct app_ctx *context, struct aws_allocator *allocato
 }
 
 static void s_on_update_shadow_complete(
-    const struct aws_byte_cursor *response_topic,
-    const struct aws_byte_cursor *payload,
+    const struct aws_mqtt_rr_incoming_publish_event *publish_event,
     int error_code,
     void *user_data) {
 
     struct aws_string *correlation_token = user_data;
 
-    if (payload != NULL) {
+    if (publish_event != NULL) {
         printf(
             "UpdateNamedShadow request '%s' response received on topic '" PRInSTR "' with body:\n  " PRInSTR "\n\n",
             correlation_token->bytes,
-            AWS_BYTE_CURSOR_PRI(*response_topic),
-            AWS_BYTE_CURSOR_PRI(*payload));
+            AWS_BYTE_CURSOR_PRI(publish_event->topic),
+            AWS_BYTE_CURSOR_PRI(publish_event->payload));
     } else {
         printf(
             "UpdateNamedShadow request '%s' failed with error code %d(%s)\n\n",
@@ -553,19 +551,18 @@ static void s_handle_update_reported(
 }
 
 static void s_on_delete_shadow_complete(
-    const struct aws_byte_cursor *response_topic,
-    const struct aws_byte_cursor *payload,
+    const struct aws_mqtt_rr_incoming_publish_event *publish_event,
     int error_code,
     void *user_data) {
 
     struct aws_string *correlation_token = user_data;
 
-    if (payload != NULL) {
+    if (publish_event != NULL) {
         printf(
             "DeleteNamedShadow request '%s' response received on topic '" PRInSTR "' with body:\n  " PRInSTR "\n\n",
             correlation_token->bytes,
-            AWS_BYTE_CURSOR_PRI(*response_topic),
-            AWS_BYTE_CURSOR_PRI(*payload));
+            AWS_BYTE_CURSOR_PRI(publish_event->topic),
+            AWS_BYTE_CURSOR_PRI(publish_event->payload));
     } else {
         printf(
             "DeleteNamedShadow request '%s' failed with error code %d(%s)\n\n",
@@ -750,7 +747,9 @@ static void s_stream_subscription_status_fn(
         aws_error_debug_str(error_code));
 }
 
-static void s_stream_incoming_publish_fn(struct aws_byte_cursor payload, void *user_data) {
+static void s_stream_incoming_publish_fn(
+    const struct aws_mqtt_rr_incoming_publish_event *publish_event,
+    void *user_data) {
     struct aws_shadow_streaming_operation *operation = user_data;
 
     struct aws_byte_cursor thing_cursor = aws_byte_cursor_from_buf(&operation->thing);
@@ -764,7 +763,7 @@ static void s_stream_incoming_publish_fn(struct aws_byte_cursor payload, void *u
         AWS_BYTE_CURSOR_PRI(thing_cursor),
         AWS_BYTE_CURSOR_PRI(shadow_cursor),
         AWS_BYTE_CURSOR_PRI(topic_filter_cursor));
-    printf("  " PRInSTR "\n\n", AWS_BYTE_CURSOR_PRI(payload));
+    printf("  " PRInSTR "\n\n", AWS_BYTE_CURSOR_PRI(publish_event->payload));
 }
 
 static void s_stream_terminated_fn(void *user_data) {
