@@ -30,7 +30,10 @@ struct mqtt_mock_server_handler {
         size_t pubacks_received;
         size_t ping_received;
         size_t connacks_avail;
+        bool session_present;
+        bool reflect_publishes;
         bool auto_ack;
+        uint8_t suback_reason_code;
 
         /* last ID used when sending PUBLISH (QoS1+) to client */
         uint16_t last_packet_id;
@@ -77,12 +80,14 @@ static int s_mqtt_mock_server_handler_process_packet(
     switch (packet_type) {
         case AWS_MQTT_PACKET_CONNECT: {
             size_t connacks_available = 0;
+            bool session_present = false;
             aws_mutex_lock(&server->synced.lock);
             AWS_LOGF_DEBUG(
                 MOCK_LOG_SUBJECT,
                 "server, CONNECT received, %llu available connacks.",
                 (long long unsigned)server->synced.connacks_avail);
             connacks_available = server->synced.connacks_avail > 0 ? server->synced.connacks_avail-- : 0;
+            session_present = server->synced.session_present;
             aws_mutex_unlock(&server->synced.lock);
 
             if (connacks_available) {
@@ -90,7 +95,7 @@ static int s_mqtt_mock_server_handler_process_packet(
                     aws_channel_acquire_message_from_pool(server->slot->channel, AWS_IO_MESSAGE_APPLICATION_DATA, 256);
 
                 struct aws_mqtt_packet_connack conn_ack;
-                err |= aws_mqtt_packet_connack_init(&conn_ack, false, AWS_MQTT_CONNECT_ACCEPTED);
+                err |= aws_mqtt_packet_connack_init(&conn_ack, session_present, AWS_MQTT_CONNECT_ACCEPTED);
                 err |= aws_mqtt_packet_connack_encode(&connack_msg->message_data, &conn_ack);
                 if (aws_channel_slot_send_message(server->slot, connack_msg, AWS_CHANNEL_DIR_WRITE)) {
                     err |= 1;
@@ -135,6 +140,7 @@ static int s_mqtt_mock_server_handler_process_packet(
 
             aws_mutex_lock(&server->synced.lock);
             bool auto_ack = server->synced.auto_ack;
+            uint8_t reason_code = server->synced.suback_reason_code;
             aws_mutex_unlock(&server->synced.lock);
 
             if (auto_ack) {
@@ -144,7 +150,7 @@ static int s_mqtt_mock_server_handler_process_packet(
                 err |= aws_mqtt_packet_suback_init(&suback, server->handler.alloc, subscribe_packet.packet_identifier);
                 const size_t num_filters = aws_array_list_length(&subscribe_packet.topic_filters);
                 for (size_t i = 0; i < num_filters; ++i) {
-                    err |= aws_mqtt_packet_suback_add_return_code(&suback, AWS_MQTT_QOS_EXACTLY_ONCE);
+                    err |= aws_mqtt_packet_suback_add_return_code(&suback, reason_code);
                 }
                 err |= aws_mqtt_packet_suback_encode(&suback_msg->message_data, &suback);
                 err |= aws_channel_slot_send_message(server->slot, suback_msg, AWS_CHANNEL_DIR_WRITE);
@@ -185,6 +191,7 @@ static int s_mqtt_mock_server_handler_process_packet(
 
             aws_mutex_lock(&server->synced.lock);
             bool auto_ack = server->synced.auto_ack;
+            bool reflect_publishes = server->synced.reflect_publishes;
             aws_mutex_unlock(&server->synced.lock);
 
             uint8_t qos = (publish_packet.fixed_header.flags >> 1) & 0x3;
@@ -197,6 +204,23 @@ static int s_mqtt_mock_server_handler_process_packet(
                 err |= aws_mqtt_packet_ack_encode(&puback_msg->message_data, &puback);
                 err |= aws_channel_slot_send_message(server->slot, puback_msg, AWS_CHANNEL_DIR_WRITE);
             }
+
+            if (reflect_publishes) {
+                struct aws_io_message *publish_msg =
+                    aws_channel_acquire_message_from_pool(server->slot->channel, AWS_IO_MESSAGE_APPLICATION_DATA, 1024);
+                struct aws_mqtt_packet_publish publish;
+                // reusing the packet identifier here is weird but reasonably safe, they're separate id spaces
+                err |= aws_mqtt_packet_publish_init(
+                    &publish,
+                    false,
+                    qos,
+                    false,
+                    publish_packet.topic_name,
+                    publish_packet.packet_identifier,
+                    publish_packet.payload);
+                err |= aws_mqtt_packet_publish_encode(&publish_msg->message_data, &publish);
+                err |= aws_channel_slot_send_message(server->slot, publish_msg, AWS_CHANNEL_DIR_WRITE);
+            }
             break;
         }
 
@@ -438,6 +462,7 @@ struct aws_channel_handler *new_mqtt_mock_server(struct aws_allocator *allocator
     server->synced.ping_resp_avail = SIZE_MAX;
     server->synced.connacks_avail = SIZE_MAX;
     server->synced.auto_ack = true;
+    server->synced.suback_reason_code = AWS_MQTT_QOS_EXACTLY_ONCE;
     aws_mutex_init(&server->synced.lock);
     aws_condition_variable_init(&server->synced.cvar);
 
@@ -466,6 +491,22 @@ void destroy_mqtt_mock_server(struct aws_channel_handler *handler) {
     aws_mem_release(handler->alloc, server);
 }
 
+void mqtt_mock_server_set_session_present(struct aws_channel_handler *handler, bool session_present) {
+    struct mqtt_mock_server_handler *server = handler->impl;
+
+    aws_mutex_lock(&server->synced.lock);
+    server->synced.session_present = session_present;
+    aws_mutex_unlock(&server->synced.lock);
+}
+
+void mqtt_mock_server_set_publish_reflection(struct aws_channel_handler *handler, bool reflect_publishes) {
+    struct mqtt_mock_server_handler *server = handler->impl;
+
+    aws_mutex_lock(&server->synced.lock);
+    server->synced.reflect_publishes = reflect_publishes;
+    aws_mutex_unlock(&server->synced.lock);
+}
+
 void mqtt_mock_server_set_max_ping_resp(struct aws_channel_handler *handler, size_t max_ping) {
     struct mqtt_mock_server_handler *server = handler->impl;
 
@@ -498,6 +539,14 @@ void mqtt_mock_server_enable_auto_ack(struct aws_channel_handler *handler) {
     aws_mutex_unlock(&server->synced.lock);
 }
 
+void mqtt_mock_server_suback_reason_code(struct aws_channel_handler *handler, uint8_t reason_code) {
+    struct mqtt_mock_server_handler *server = handler->impl;
+
+    aws_mutex_lock(&server->synced.lock);
+    server->synced.suback_reason_code = reason_code;
+    aws_mutex_unlock(&server->synced.lock);
+}
+
 struct mqtt_mock_server_ack_args {
     struct aws_channel_task task;
     struct aws_mqtt_packet_ack ack;
