@@ -26,6 +26,7 @@ struct aws_http_connection;
 struct aws_http_connection_manager;
 struct aws_host_resolver;
 struct aws_s3_endpoint;
+struct aws_priority_queue;
 
 enum aws_s3_connection_finish_code {
     AWS_S3_CONNECTION_FINISH_CODE_SUCCESS,
@@ -167,12 +168,17 @@ struct aws_s3_client_vtable {
 
     void (*finish_destroy)(struct aws_s3_client *client);
 
-    struct aws_parallel_input_stream *(
-        *parallel_input_stream_new_from_file)(struct aws_allocator *allocator, struct aws_byte_cursor file_name);
+    struct aws_parallel_input_stream *(*parallel_input_stream_new_from_file)(
+        struct aws_allocator *allocator,
+        struct aws_byte_cursor file_name,
+        struct aws_event_loop_group *reading_elg,
+        bool direct_io_read);
 
     struct aws_http_stream *(*http_connection_make_request)(
         struct aws_http_connection *client_connection,
         const struct aws_http_make_request_options *options);
+
+    void (*after_prepare_upload_part_finish)(struct aws_s3_request *request, struct aws_http_message *message);
 };
 
 struct aws_s3_upload_part_timeout_stats {
@@ -185,6 +191,8 @@ struct aws_s3_upload_part_timeout_stats {
     struct {
         uint64_t sum_ns;
         uint64_t num_samples;
+        bool collecting_p90;
+        struct aws_priority_queue p90_samples;
     } initial_request_time;
 
     /* Track the timeout rate. */
@@ -230,6 +238,10 @@ struct aws_s3_client {
      * to meta requests for use. */
     const uint64_t max_part_size;
 
+    /* File I/O options. */
+    bool fio_options_set;
+    struct aws_s3_file_io_options fio_opts;
+
     /* The size threshold in bytes for when to use multipart uploads for a AWS_S3_META_REQUEST_TYPE_PUT_OBJECT meta
      * request. Uploads over this size will automatically use a multipart upload strategy, while uploads smaller or
      * equal to this threshold will use a single request to upload the whole object. If not set, `part_size` will be
@@ -346,6 +358,9 @@ struct aws_s3_client {
 
         /* Number of requests currently scheduled to be streamed the response body or are actively being streamed. */
         struct aws_atomic_var num_requests_streaming_response;
+
+        /* Number of overall requests currently streaming the request body instead of buffering. */
+        struct aws_atomic_var num_requests_streaming_request_body;
     } stats;
 
     struct {
