@@ -12,7 +12,6 @@
 #include <aws/common/thread.h>
 #include <aws/s3/s3.h>
 
-#include <aws/s3/private/s3_buffer_pool.h>
 #include <aws/s3/private/s3_checksums.h>
 
 struct aws_http_message;
@@ -21,9 +20,8 @@ struct aws_s3_meta_request;
 
 enum aws_s3_request_flags {
     AWS_S3_REQUEST_FLAG_RECORD_RESPONSE_HEADERS = 0x00000001,
-    AWS_S3_REQUEST_FLAG_PART_SIZE_RESPONSE_BODY = 0x00000002,
-    AWS_S3_REQUEST_FLAG_ALWAYS_SEND = 0x00000004,
-    AWS_S3_REQUEST_FLAG_PART_SIZE_REQUEST_BODY = 0x00000008,
+    AWS_S3_REQUEST_FLAG_ALWAYS_SEND = 0x00000002,
+    AWS_S3_REQUEST_FLAG_ALLOCATE_BUFFER_FROM_POOL = 0x00000004,
 };
 
 /**
@@ -72,6 +70,26 @@ struct aws_s3_request_metrics {
         /* The time duration for the request from start signing to finish signing (sign_end_timestamp_ns -
          * sign_start_timestamp_ns). When sign_end_timestamp_ns is 0, means data not available. */
         int64_t signing_duration_ns;
+
+        /* The time stamp when the request started to acquire memory. -1 means data not
+         * available. Timestamp are from `aws_high_res_clock_get_ticks` */
+        int64_t mem_acquire_start_timestamp_ns;
+        /* The time stamp when the response to acquire memory. -1 means data not
+         * available. Timestamp are from `aws_high_res_clock_get_ticks` */
+        int64_t mem_acquire_end_timestamp_ns;
+        /* The time duration for the request from start mem acquire to finish mem acquire (mem_acquire_end_timestamp_ns
+         * - mem_acquire_start_timestamp_ns). When sign_end_timestamp_ns is 0, means data not available. */
+        int64_t mem_acquire_duration_ns;
+
+        /* The time stamp when the request started to be delivered (i.e. on body callback is invoked or write to file
+         * initiated). -1 means data not available. Timestamp are from `aws_high_res_clock_get_ticks` */
+        int64_t deliver_start_timestamp_ns;
+        /* The time stamp when the response finished to be delivered. -1 means data not
+         * available. Timestamp are from `aws_high_res_clock_get_ticks` */
+        int64_t deliver_end_timestamp_ns;
+        /* The time duration for the request from start of delivery to finish of delivery (deliver_end_timestamp_ns -
+         * deliver_start_timestamp_ns). When deliver_duration_ns is 0, means data not available. */
+        int64_t deliver_duration_ns;
     } time_metrics;
 
     struct {
@@ -102,6 +120,8 @@ struct aws_s3_request_metrics {
         uint32_t stream_id;
         /* CRT error code when the aws_s3_request finishes. */
         int error_code;
+        /* Retry attempt. */
+        uint32_t retry_attempt;
     } crt_info_metrics;
 
     struct aws_ref_count ref_count;
@@ -116,10 +136,16 @@ struct aws_s3_request {
     /* Linked list node used for tracking the request is active from HTTP level. */
     struct aws_linked_list_node cancellable_http_streams_list_node;
 
+    /* Linked list node used for tracking buffer acquire futures. */
+    struct aws_linked_list_node pending_buffer_future_list_node;
+
     /* The meta request lock must be held to access the data */
     struct {
         /* The underlying http stream, only valid when the request is active from HTTP level */
         struct aws_http_stream *cancellable_http_stream;
+
+        /* Buffer future. */
+        struct aws_future_s3_buffer_ticket *buffer_future;
     } synced_data;
 
     /* TODO Ref count on the request is no longer needed--only one part of code should ever be holding onto a request,
@@ -131,11 +157,23 @@ struct aws_s3_request {
     /* Owning meta request. */
     struct aws_s3_meta_request *meta_request;
 
+    /* The buffer size to be allocated for the request, defaults the part size. */
+    size_t buffer_size;
     /* Request body to use when sending the request. The contents of this body will be re-used if a request is
      * retried.*/
     struct aws_byte_buf request_body;
 
-    struct aws_s3_buffer_pool_ticket *ticket;
+    /* Set when the request will be streaming from file directly instead of the request_body. */
+    bool fio_streaming;
+    /* If file I/O options configure streaming. The request body will be streaming from this. If a request is retried,
+     * this stream will be recreated. */
+    struct aws_input_stream *request_body_stream;
+    uint64_t content_length;
+
+    /**
+     * Ticket to acquire the buffer.
+     */
+    struct aws_s3_buffer_ticket *ticket;
 
     /* Beginning range of this part. */
     /* TODO currently only used by auto_range_get, could be hooked up to auto_range_put as well. */
@@ -210,6 +248,11 @@ struct aws_s3_request {
         /* Returned response status of this request. */
         int response_status;
 
+        /* Returned request ID of this request. */
+        struct aws_string *request_id;
+        /* Returned amz ID 2 of this request. */
+        struct aws_string *amz_id_2;
+
         /* The metrics for the request telemetry */
         struct aws_s3_request_metrics *metrics;
 
@@ -220,11 +263,8 @@ struct aws_s3_request {
     /* When true, response headers from the request will be stored in the request's response_headers variable. */
     uint32_t record_response_headers : 1;
 
-    /* When true, the response body buffer will be allocated in the size of a part. */
-    uint32_t has_part_size_response_body : 1;
-
-    /* When true, the request body buffer will be allocated in the size of a part. */
-    uint32_t has_part_size_request_body : 1;
+    /* Indicates whether buffer should be allocated for the request from the pool. */
+    uint32_t should_allocate_buffer_from_pool : 1;
 
     /* When true, this request is being tracked by the client for limiting the amount of in-flight-requests/stats. */
     uint32_t tracked_by_client : 1;
@@ -277,10 +317,7 @@ AWS_S3_API
 struct aws_s3_request *aws_s3_request_release(struct aws_s3_request *request);
 
 AWS_S3_API
-struct aws_s3_request_metrics *aws_s3_request_metrics_new(
-    struct aws_allocator *allocator,
-    const struct aws_s3_request *request,
-    const struct aws_http_message *message);
+struct aws_s3_request_metrics *aws_s3_request_metrics_new(struct aws_allocator *allocator);
 
 AWS_EXTERN_C_END
 
