@@ -44,12 +44,12 @@ static struct aws_http_connection_manager *s_s3_endpoint_create_http_connection_
     const struct proxy_env_var_settings *proxy_ev_settings,
     uint32_t connect_timeout_ms,
     const struct aws_s3_tcp_keep_alive_options *tcp_keep_alive_options,
-    const struct aws_http_connection_monitoring_options *monitoring_options);
+    const struct aws_http_connection_monitoring_options *monitoring_options,
+    const struct aws_byte_cursor *network_interface_names_array,
+    size_t num_network_interface_names);
 
 static void s_s3_endpoint_http_connection_manager_shutdown_callback(void *user_data);
 
-static void s_s3_endpoint_ref_count_zero(struct aws_s3_endpoint *endpoint);
-
 static void s_s3_endpoint_acquire(struct aws_s3_endpoint *endpoint, bool already_holding_lock);
 
 static void s_s3_endpoint_release(struct aws_s3_endpoint *endpoint);
@@ -111,7 +111,9 @@ struct aws_s3_endpoint *aws_s3_endpoint_new(
         options->proxy_ev_settings,
         options->connect_timeout_ms,
         options->tcp_keep_alive_options,
-        options->monitoring_options);
+        options->monitoring_options,
+        options->network_interface_names_array,
+        options->num_network_interface_names);
 
     if (endpoint->http_connection_manager == NULL) {
         goto error_cleanup;
@@ -123,8 +125,6 @@ struct aws_s3_endpoint *aws_s3_endpoint_new(
 
 error_cleanup:
 
-    aws_string_destroy(options->host_name);
-
     aws_mem_release(allocator, endpoint);
 
     return NULL;
@@ -141,7 +141,9 @@ static struct aws_http_connection_manager *s_s3_endpoint_create_http_connection_
     const struct proxy_env_var_settings *proxy_ev_settings,
     uint32_t connect_timeout_ms,
     const struct aws_s3_tcp_keep_alive_options *tcp_keep_alive_options,
-    const struct aws_http_connection_monitoring_options *monitoring_options) {
+    const struct aws_http_connection_monitoring_options *monitoring_options,
+    const struct aws_byte_cursor *network_interface_names_array,
+    size_t num_network_interface_names) {
 
     AWS_PRECONDITION(endpoint);
     AWS_PRECONDITION(client_bootstrap);
@@ -179,6 +181,8 @@ static struct aws_http_connection_manager *s_s3_endpoint_create_http_connection_
     manager_options.shutdown_complete_callback = s_s3_endpoint_http_connection_manager_shutdown_callback;
     manager_options.shutdown_complete_user_data = endpoint;
     manager_options.proxy_ev_settings = proxy_ev_settings;
+    manager_options.network_interface_names_array = network_interface_names_array;
+    manager_options.num_network_interface_names = num_network_interface_names;
     if (monitoring_options != NULL) {
         manager_options.monitoring_options = monitoring_options;
     }
@@ -246,7 +250,6 @@ static void s_s3_endpoint_acquire(struct aws_s3_endpoint *endpoint, bool already
         aws_s3_client_lock_synced_data(endpoint->client);
     }
 
-    AWS_ASSERT(endpoint->client_synced_data.ref_count > 0);
     ++endpoint->client_synced_data.ref_count;
 
     if (!already_holding_lock) {
@@ -267,29 +270,32 @@ static void s_s3_endpoint_release(struct aws_s3_endpoint *endpoint) {
     /* BEGIN CRITICAL SECTION */
     aws_s3_client_lock_synced_data(endpoint->client);
 
-    bool should_destroy = (endpoint->client_synced_data.ref_count == 1);
+    bool should_destroy = endpoint->client_synced_data.ref_count == 1 && !endpoint->client->synced_data.active;
     if (should_destroy) {
         aws_hash_table_remove(&endpoint->client->synced_data.endpoints, endpoint->host_name, NULL, NULL);
-    } else {
-        --endpoint->client_synced_data.ref_count;
     }
+    --endpoint->client_synced_data.ref_count;
 
     aws_s3_client_unlock_synced_data(endpoint->client);
     /* END CRITICAL SECTION */
 
     if (should_destroy) {
-        /* The endpoint may have async cleanup to do (connection manager).
+        /* Do a sync cleanup since client is getting destroyed to avoid any cleanup delay.
+         * The endpoint may have async cleanup to do (connection manager).
          * When that's all done we'll invoke a completion callback.
          * Since it's a crime to hold a lock while invoking a callback,
-         * we make sure that we've released the client's lock before proceeding... */
-        s_s3_endpoint_ref_count_zero(endpoint);
+         * we make sure that we've released the client's lock before proceeding...
+         */
+        aws_s3_endpoint_destroy(endpoint);
     }
 }
 
-static void s_s3_endpoint_ref_count_zero(struct aws_s3_endpoint *endpoint) {
+void aws_s3_endpoint_destroy(struct aws_s3_endpoint *endpoint) {
     AWS_PRECONDITION(endpoint);
     AWS_PRECONDITION(endpoint->http_connection_manager);
 
+    AWS_FATAL_ASSERT(endpoint->client_synced_data.ref_count == 0);
+
     struct aws_http_connection_manager *http_connection_manager = endpoint->http_connection_manager;
     endpoint->http_connection_manager = NULL;
 
