@@ -29,6 +29,7 @@ const struct aws_byte_cursor g_host_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_
 const struct aws_byte_cursor g_range_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Range");
 const struct aws_byte_cursor g_if_match_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("If-Match");
 const struct aws_byte_cursor g_request_id_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-request-id");
+const struct aws_byte_cursor g_amz_id_2_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-id-2");
 const struct aws_byte_cursor g_etag_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("ETag");
 const struct aws_byte_cursor g_content_range_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Content-Range");
 const struct aws_byte_cursor g_content_type_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Content-Type");
@@ -64,6 +65,31 @@ const struct aws_byte_cursor g_user_agent_header_unknown = AWS_BYTE_CUR_INIT_FRO
 const uint32_t g_s3_max_num_upload_parts = 10000;
 const size_t g_s3_min_upload_part_size = MB_TO_BYTES(5);
 
+const double g_default_throughput_target_gbps = 10.0;
+
+/**
+ * Streaming buffer size selection based on experimental results on EBS:
+ *
+ * - Too small buffer sizes (e.g., 16KiB) impact disk read performance,
+ *   achieving only 6.73 Gbps throughput from EBS.
+ * - Too large buffer sizes cause network connections to starve more easily
+ *   when disk reads cannot provide data fast enough.
+ * - 1MiB buffer size provides optimal balance: sufficient disk read throughput
+ *   while maintaining reasonable retry rates due to connection starvation.
+ */
+const size_t g_streaming_buffer_size = MB_TO_BYTES(1);
+
+/**
+ * The streaming approach reduces memory consumption without introducing unexpected errors
+ * or performance degradation.
+ *
+ * We start streaming for objects larger than 1TiB, with plans to lower this threshold in future iterations.
+ *
+ * The 1TiB threshold was chosen to minimize the blast radius of this behavioral change
+ * while still providing meaningful memory usage improvements for large objects.
+ */
+const uint64_t g_streaming_object_size_threshold = TB_TO_BYTES(1);
+
 void copy_http_headers(const struct aws_http_headers *src, struct aws_http_headers *dest) {
     AWS_PRECONDITION(src);
     AWS_PRECONDITION(dest);
@@ -682,11 +708,6 @@ int aws_s3_crt_error_code_from_recoverable_server_error_code_string(struct aws_b
         return AWS_ERROR_S3_REQUEST_TIMEOUT;
     }
 
-    if (aws_byte_cursor_eq_c_str_ignore_case(&error_code_string, "ExpiredToken") ||
-        aws_byte_cursor_eq_c_str_ignore_case(&error_code_string, "TokenRefreshRequired")) {
-        return AWS_ERROR_S3_TOKEN_EXPIRED;
-    }
-
     return AWS_ERROR_UNKNOWN;
 }
 
@@ -746,8 +767,7 @@ int aws_s3_check_headers_for_checksum(
             /* Found the checksum header, keep the header value and initialize the running checksum */
             size_t encoded_len = 0;
             aws_base64_compute_encoded_len(aws_get_digest_size_from_checksum_algorithm(algorithm), &encoded_len);
-            if (checksum_value.len == encoded_len - 1) {
-                /* encoded_len includes the nullptr length. -1 is the expected length. */
+            if (checksum_value.len == encoded_len) {
                 aws_byte_buf_init_copy_from_cursor(out_checksum_buffer, meta_request->allocator, checksum_value);
                 *out_checksum = aws_checksum_new(meta_request->allocator, algorithm);
                 if (!*out_checksum) {
