@@ -20,8 +20,6 @@ TIMEOUT = 120  # this must be higher than any response's "delay" setting
 
 VERBOSE = False
 
-# Flags to keep between requests
-SHOULD_THROTTLE = True
 RETRY_REQUEST_COUNT = 0
 
 
@@ -51,16 +49,32 @@ class Response:
 @dataclass
 class ResponseConfig:
     path: str
-    disconnect_after_headers = False
+    request: Optional[object] = None  # Add request as a field
+    disconnect_after_headers: bool = False
     generate_body_size: Optional[int] = None
-    json_path: str = None
-    throttle: bool = False
+    json_path: Optional[str] = None
+    forced_throttle: bool = False
     force_retry: bool = False
+    should_skip_wait: bool = False
     request_headers: Optional[List[Tuple[bytes, bytes]]] = None
 
+    def __post_init__(self):
+        """Called automatically after the dataclass __init__"""
+        if self.request is not None:
+            self.request_headers = self.request.headers
+            if get_request_header_value(self.request, "before_finish") is not None:
+                self.should_skip_wait = True
+            if get_request_header_value(self.request, "force_throttle") is not None:
+                self.forced_throttle = True
+
     def _resolve_file_path(self, wrapper, request_type):
-        global SHOULD_THROTTLE
         if self.json_path is None:
+            if self.forced_throttle:
+                # force the throttle to happen, for triggering retries.
+                response_file = os.path.join(
+                    base_dir, request_type.name, f"throttle.json")
+                self.json_path = response_file
+                return
             response_file = os.path.join(
                 base_dir, request_type.name, f"{self.path[1:]}.json")
             if os.path.exists(response_file) == False:
@@ -68,16 +82,6 @@ class ResponseConfig:
                     response_file, "not exist, using the default response")
                 response_file = os.path.join(
                     base_dir, request_type.name, f"default.json")
-            if "throttle" in response_file:
-                # We throttle the request half the time to make sure it succeeds after a retry
-                if SHOULD_THROTTLE is False:
-                    wrapper.info("Skipping throttling")
-                    response_file = os.path.join(
-                        base_dir, request_type.name, f"default.json")
-                else:
-                    wrapper.info("Throttling")
-                # Flip the flag
-                SHOULD_THROTTLE = not SHOULD_THROTTLE
             self.json_path = response_file
 
     def resolve_response(self, wrapper, request_type, chunked=False, head_request=False):
@@ -90,12 +94,12 @@ class ResponseConfig:
 
         # If request_headers is present, validate that the request contains all required headers
         if 'request_headers' in data:
-                for header in data['request_headers']:
-                    header_bytes = header.encode('utf-8')
-                    if not any(header_bytes == h[0] for h in self.request_headers):
-                        response =  Response(status_code=500, delay=0, headers=headers,
-                                             data=json.dumps({'error': f"Missing required header: {header}"}), chunked=chunked, head_request=head_request)
-                        return response
+            for header in data['request_headers']:
+                header_bytes = header.encode('utf-8')
+                if not any(header_bytes == h[0] for h in self.request_headers):
+                    response = Response(status_code=500, delay=0, headers=headers,
+                                        data=json.dumps({'error': f"Missing required header: {header}"}), chunked=chunked, head_request=head_request)
+                    return response
 
         # if response has delay, then sleep before sending it
         delay = data.get('delay', 0)
@@ -364,38 +368,9 @@ async def send_mock_s3_response(wrapper, request_type, path, generate_body=False
         wrapper.info(response_file, "not exist, using the default response")
         response_file = os.path.join(
             base_dir, request_type.name, f"default.json")
-    if "throttle" in response_file:
-        # We throttle the request half the time to make sure it succeeds after a retry
-        if wrapper.should_throttle is False:
-            wrapper.info("Skipping throttling")
-            response_file = os.path.join(
-                base_dir, request_type.name, f"default.json")
-        else:
-            wrapper.info("Throttling")
-        # Flip the flag
-        wrapper.should_throttle = not wrapper.should_throttle
     await send_response_from_json(wrapper, response_file, generate_body=generate_body, generate_body_size=generate_body_size, head_request=head_request)
 
 
-async def maybe_send_error_response(wrapper, exc):
-    if wrapper.conn.our_state not in {h11.IDLE, h11.SEND_RESPONSE}:
-        wrapper.info("...but I can't, because our state is",
-                     wrapper.conn.our_state)
-        return
-    try:
-        await wrapper.send(res)
-    except Exception as e:
-        print(e)
-
-    if not response.head_request:
-        if response.chunked:
-            await wrapper.send(h11.Data(data=b"%X\r\n%s\r\n" % (len(response.data), response.data.encode())))
-        else:
-            await wrapper.send(h11.Data(data=response.data.encode()))
-
-    await wrapper.send(h11.EndOfMessage())
-
-
 def get_request_header_value(request, header_name):
     for header in request.headers:
         if header[0].decode("utf-8").lower() == header_name.lower():
@@ -411,7 +386,6 @@ def handle_get_object_modified(start_range, end_range, request):
     else:
         # Check the request header to make sure "If-Match" is set
         etag = get_request_header_value(request, "if-match")
-        print(etag)
         # fetch Etag from the first_part response file
         response_file = os.path.join(
             base_dir, S3Opts.GetObject.name, f"get_object_modified_first_part.json")
@@ -436,7 +410,7 @@ def handle_get_object(wrapper, request, parsed_path, head_request=False):
 
     if (parsed_path.path == "/get_object_invalid_response_missing_content_range" or
         parsed_path.path == "/get_object_invalid_response_missing_etags" or
-        parsed_path.path == "/get_object_long_error"):
+            parsed_path.path == "/get_object_long_error"):
         # Don't generate the body for those requests
         return response_config
 
@@ -501,19 +475,20 @@ async def handle_mock_s3_request(wrapper, request):
         wrapper.info("unsupported request:", request)
         request_type = S3Opts.CreateMultipartUpload
 
-    while True:
-        event = await wrapper.next_event()
-        if type(event) is h11.EndOfMessage:
-            break
-        assert type(event) is h11.Data
-
     if response_config is None:
-        response_config = ResponseConfig(parsed_path.path)
-    response_config.request_headers = request.headers
+        response_config = ResponseConfig(parsed_path.path, request=request)
+
+    if not response_config.should_skip_wait:
+        while True:
+            event = await wrapper.next_event()
+            if type(event) is h11.EndOfMessage:
+                break
+            assert type(event) is h11.Data
+    else:
+        print("Skipping waiting for request body")
 
     response = response_config.resolve_response(
         wrapper, request_type, head_request=method == "HEAD")
-
     await send_response(wrapper, response)
 
 
