@@ -3,6 +3,7 @@
  * SPDX-License-Identifier: Apache-2.0.
  */
 
+#include "aws/s3/private/s3_checksum_context.h"
 #include "aws/s3/private/s3_checksums.h"
 #include "s3_tester.h"
 #include <aws/common/byte_buf.h>
@@ -27,8 +28,7 @@ static int compare_checksum_stream(struct aws_allocator *allocator, struct aws_b
         struct aws_byte_cursor checksum_result_cursor = aws_byte_cursor_from_buf(&compute_checksum_output);
         aws_base64_encode(&checksum_result_cursor, &compute_encoded_checksum_output);
         struct aws_input_stream *cursor_stream = aws_input_stream_new_from_cursor(allocator, input);
-        struct aws_input_stream *stream =
-            aws_checksum_stream_new(allocator, cursor_stream, algorithm, &stream_checksum_output);
+        struct aws_input_stream *stream = aws_checksum_stream_new(allocator, cursor_stream, algorithm);
         aws_input_stream_release(cursor_stream);
         struct aws_stream_status status;
         AWS_ZERO_STRUCT(status);
@@ -37,11 +37,12 @@ static int compare_checksum_stream(struct aws_allocator *allocator, struct aws_b
             read_buf.len = 0;
             ASSERT_TRUE(aws_input_stream_get_status(stream, &status) == 0);
         }
-        aws_input_stream_release(stream);
+        aws_checksum_stream_finalize_checksum(stream, &stream_checksum_output);
         ASSERT_TRUE(aws_byte_buf_eq(&compute_encoded_checksum_output, &stream_checksum_output));
         aws_byte_buf_clean_up(&compute_checksum_output);
         aws_byte_buf_clean_up(&stream_checksum_output);
         aws_byte_buf_clean_up(&compute_encoded_checksum_output);
+        aws_input_stream_release(stream);
     }
     aws_byte_buf_clean_up(&read_buf);
     return AWS_OP_SUCCESS;
@@ -111,9 +112,27 @@ static int s_stream_chunk(
     struct aws_byte_buf *output,
     enum aws_s3_checksum_algorithm algorithm,
     struct aws_byte_buf *checksum_result) {
+
+    /* Create checksum config for the context */
+    struct aws_s3_meta_request_checksum_config_storage config = {
+        .allocator = allocator,
+        .checksum_algorithm = algorithm,
+        .location = AWS_SCL_TRAILER,
+        .has_full_object_checksum = false,
+    };
+    AWS_ZERO_STRUCT(config.full_object_checksum);
+
+    /* Create checksum context */
+    struct aws_s3_upload_request_checksum_context *context =
+        aws_s3_upload_request_checksum_context_new(allocator, &config);
+    if (!context) {
+        return AWS_OP_ERR;
+    }
+
     struct aws_input_stream *cursor_stream = aws_input_stream_new_from_cursor(allocator, input);
-    struct aws_input_stream *stream = aws_chunk_stream_new(allocator, cursor_stream, algorithm, checksum_result);
+    struct aws_input_stream *stream = aws_chunk_stream_new(allocator, cursor_stream, context);
     aws_input_stream_release(cursor_stream);
+
     struct aws_stream_status status;
     AWS_ZERO_STRUCT(status);
     while (!status.is_end_of_stream) {
@@ -123,7 +142,19 @@ static int s_stream_chunk(
         read_buf->len = 0;
         ASSERT_TRUE(aws_input_stream_get_status(stream, &status) == 0);
     }
+
+    /* Copy checksum result if requested */
+    if (checksum_result) {
+        struct aws_byte_cursor checksum_cursor = aws_s3_upload_request_checksum_context_get_checksum_cursor(context);
+        if (aws_byte_buf_init_copy_from_cursor(checksum_result, allocator, checksum_cursor) != AWS_OP_SUCCESS) {
+            aws_input_stream_release(stream);
+            aws_s3_upload_request_checksum_context_release(context);
+            return AWS_OP_ERR;
+        }
+    }
+
     aws_input_stream_release(stream);
+    aws_s3_upload_request_checksum_context_release(context);
     return AWS_OP_SUCCESS;
 }
 
@@ -143,6 +174,7 @@ static int compare_chunk_stream(
     struct aws_byte_buf read_buf;
     aws_byte_buf_init(&read_buf, allocator, buffer_size);
     for (size_t i = 0; i < AWS_ARRAY_SIZE(s_checksum_algo_priority_list); i++) {
+        AWS_ZERO_STRUCT(streamed_encoded_checksum);
         enum aws_s3_checksum_algorithm algorithm = s_checksum_algo_priority_list[i];
         aws_base64_compute_encoded_len(aws_get_digest_size_from_checksum_algorithm(algorithm), &encoded_len);
         size_t total_len =
