@@ -123,14 +123,11 @@ struct s3_fail_prepare_test_data {
     uint32_t num_requests_being_prepared_is_correct : 1;
 };
 
-static int s_s3_fail_prepare_request(struct aws_s3_meta_request *meta_request, struct aws_s3_request *request) {
-    (void)meta_request;
-    (void)request;
-    AWS_ASSERT(meta_request != NULL);
+static struct aws_future_void *s_s3_fail_prepare_request(struct aws_s3_request *request) {
     AWS_ASSERT(request != NULL);
-
-    aws_raise_error(AWS_ERROR_UNKNOWN);
-    return AWS_OP_ERR;
+    struct aws_future_void *future = aws_future_void_new(request->allocator);
+    aws_future_void_set_error(future, AWS_ERROR_UNKNOWN);
+    return future;
 }
 
 static struct aws_s3_meta_request *s_meta_request_factory_patch_prepare_request(
@@ -280,9 +277,18 @@ static int s_test_s3_meta_request_sign_request_fail(struct aws_allocator *alloca
     return 0;
 }
 
-static int s_s3_meta_request_prepare_request_fail_first(
-    struct aws_s3_meta_request *meta_request,
-    struct aws_s3_request *request) {
+struct s3_meta_request_prepare_request_fail_first_job {
+    struct aws_allocator *allocator;
+    struct aws_s3_request *request;
+    struct aws_future_void *original_future; /* original future that we're intercepting and patching */
+    struct aws_future_void *patched_future;  /* patched future to set when this job completes */
+};
+
+static void s_s3_meta_request_prepare_request_fail_first_on_original_done(void *user_data);
+
+static struct aws_future_void *s_s3_meta_request_prepare_request_fail_first(struct aws_s3_request *request) {
+
+    struct aws_s3_meta_request *meta_request = request->meta_request;
     AWS_ASSERT(meta_request);
 
     struct aws_s3_client *client = meta_request->client;
@@ -291,13 +297,35 @@ static int s_s3_meta_request_prepare_request_fail_first(
     struct aws_s3_tester *tester = client->shutdown_callback_user_data;
     AWS_ASSERT(tester != NULL);
 
+    struct aws_future_void *patched_future = aws_future_void_new(request->allocator);
+
+    struct s3_meta_request_prepare_request_fail_first_job *patched_prep =
+        aws_mem_calloc(request->allocator, 1, sizeof(struct s3_meta_request_prepare_request_fail_first_job));
+
+    patched_prep->allocator = request->allocator;
+    patched_prep->patched_future = aws_future_void_acquire(patched_future);
+    patched_prep->request = request;
+
     struct aws_s3_meta_request_vtable *original_meta_request_vtable =
         aws_s3_tester_get_meta_request_vtable_patch(tester, 0)->original_vtable;
 
-    int result = original_meta_request_vtable->prepare_request(meta_request, request);
+    patched_prep->original_future = original_meta_request_vtable->prepare_request(request);
+    aws_future_void_register_callback(
+        patched_prep->original_future, s_s3_meta_request_prepare_request_fail_first_on_original_done, patched_prep);
 
-    if (result != AWS_OP_SUCCESS) {
-        return result;
+    return patched_future;
+}
+
+static void s_s3_meta_request_prepare_request_fail_first_on_original_done(void *user_data) {
+
+    struct s3_meta_request_prepare_request_fail_first_job *patched_prep = user_data;
+    struct aws_s3_request *request = patched_prep->request;
+    struct aws_s3_tester *tester = request->meta_request->client->shutdown_callback_user_data;
+
+    int error_code = aws_future_void_get_error(patched_prep->original_future);
+    if (error_code != AWS_ERROR_SUCCESS) {
+        aws_future_void_set_error(patched_prep->patched_future, error_code);
+        goto finish;
     }
 
     if (aws_s3_tester_inc_counter1(tester) == 1) {
@@ -310,7 +338,11 @@ static int s_s3_meta_request_prepare_request_fail_first(
         (void)set_request_path_result;
     }
 
-    return AWS_OP_SUCCESS;
+    aws_future_void_set_result(patched_prep->patched_future);
+finish:
+    aws_future_void_release(patched_prep->original_future);
+    aws_future_void_release(patched_prep->patched_future);
+    aws_mem_release(patched_prep->allocator, patched_prep);
 }
 
 static void s_s3_meta_request_send_request_finish_fail_first(
@@ -327,7 +359,7 @@ static void s_s3_meta_request_send_request_finish_fail_first(
     if (aws_s3_tester_inc_counter2(tester) == 1) {
         AWS_ASSERT(connection->request->send_data.response_status == 404);
 
-        connection->request->send_data.response_status = AWS_S3_RESPONSE_STATUS_INTERNAL_ERROR;
+        connection->request->send_data.response_status = AWS_HTTP_STATUS_CODE_500_INTERNAL_SERVER_ERROR;
     }
 
     struct aws_s3_meta_request_vtable *original_meta_request_vtable =
