@@ -9,6 +9,8 @@
 #include <aws/common/byte_buf.h>
 #include <aws/sdkutils/sdkutils.h>
 
+AWS_PUSH_SANE_WARNING_LEVEL
+
 struct aws_partitions_config;
 
 AWS_EXTERN_C_BEGIN
@@ -34,5 +36,6 @@ AWS_SDKUTILS_API struct aws_partitions_config *aws_partitions_config_acquire(str
 AWS_SDKUTILS_API struct aws_partitions_config *aws_partitions_config_release(struct aws_partitions_config *partitions);
 
 AWS_EXTERN_C_END
+AWS_POP_SANE_WARNING_LEVEL
 
 #endif /* AWS_SDKUTILS_PARTITIONS_H */
