@@ -15,16 +15,15 @@
 #
 set -eu
 
-export S2N_LIBCRYPTO=openssl-3.4
 export CTEST_OUTPUT_ON_FAILURE=1
-BREWINSTLLPATH=$(brew --prefix openssl@3)
-OPENSSL_3_INSTALL_DIR="${BREWINSTLLPATH:-"/opt/homebrew/Cellar/openssl@3"}"
+BREWINSTLLPATH=$(brew --prefix openssl)
+OPENSSL_INSTALL_DIR="${BREWINSTLLPATH:-"/opt/homebrew/Cellar/openssl"}"
 
-echo "Using OpenSSL at $OPENSSL_3_INSTALL_DIR"
-# Build with debug symbols and a specific OpenSSL version
+echo "Using OpenSSL at $OPENSSL_INSTALL_DIR"
+# Build with debug symbols
 cmake . -Bbuild -GNinja \
 -DCMAKE_BUILD_TYPE=Debug \
--DCMAKE_PREFIX_PATH=${OPENSSL_3_INSTALL_DIR} ..
+-DCMAKE_PREFIX_PATH=${OPENSSL_INSTALL_DIR} ..
 
 cmake --build ./build -j $(nproc)
 time CTEST_PARALLEL_LEVEL=$(nproc) ninja -C build test
@@ -32,7 +31,7 @@ time CTEST_PARALLEL_LEVEL=$(nproc) ninja -C build test
 # Build shared library
 cmake . -Bbuild -GNinja \
 -DCMAKE_BUILD_TYPE=Debug \
--DCMAKE_PREFIX_PATH=${OPENSSL_3_INSTALL_DIR} .. \
+-DCMAKE_PREFIX_PATH=${OPENSSL_INSTALL_DIR} .. \
 -DBUILD_SHARED_LIBS=ON
 
 cmake --build ./build -j $(nproc)
