@@ -35,6 +35,9 @@ cp -r \
 
 # generate the bindings modules from the copied sources
 pushd generate
+# Behavior change from https://github.com/rust-lang/rustup/pull/3985
+rustc --version || rustup toolchain install
+rustup component add rustfmt
 cargo run -- ../s2n-tls-sys
 popd
 
@@ -53,6 +56,7 @@ cargo publish --dry-run --allow-dirty --all-features
 popd
 
 pushd ../standard/integration
+rustc --version || rustup toolchain install
 cargo run
 popd
 
