@@ -3,6 +3,8 @@
 
 #![allow(clippy::missing_safety_doc)] // TODO add safety docs
 
+#[cfg(feature = "unstable-cert_authorities")]
+use crate::cert_authorities::CertRequestState;
 #[cfg(feature = "unstable-renegotiate")]
 use crate::renegotiate::RenegotiateState;
 use crate::{
@@ -72,8 +74,8 @@ impl fmt::Debug for Connection {
         if let Ok(version) = self.actual_protocol_version() {
             debug.field("actual_protocol_version", &version);
         }
-        if let Ok(curve) = self.selected_curve() {
-            debug.field("selected_curve", &curve);
+        if let Some(group_name) = self.selected_key_exchange_group() {
+            debug.field("selected_key_exchange_group", &group_name);
         }
         debug.finish_non_exhaustive()
     }
@@ -157,6 +159,18 @@ impl Connection {
         self.connection.as_ptr()
     }
 
+    /// Returns the underlying `s2n_tls_sys::s2n_connection` pointer associated with the
+    /// `Connection`.
+    ///
+    /// #### Warning:
+    /// This API is unstable, and may be removed in a future s2n-tls release. Applications should
+    /// use the higher level s2n-tls bindings rather than calling the low-level `s2n_tls_sys` APIs
+    /// directly.
+    #[cfg(s2n_tls_external_build)]
+    pub fn unstable_as_ptr(&mut self) -> *mut s2n_connection {
+        self.as_ptr()
+    }
+
     /// # Safety
     ///
     /// Caller must ensure s2n_connection is a valid reference to a [`s2n_connection`] object
@@ -923,7 +937,7 @@ impl Connection {
 
     /// Check if client auth was used for a connection.
     ///
-    /// This is only relevant if [`ClientAuthType::Optional] was used.
+    /// This is especially useful when the server has [`ClientAuthType::Optional`] configured.
     ///
     /// Corresponds to [s2n_connection_client_cert_used].
     pub fn client_cert_used(&self) -> bool {
@@ -1112,6 +1126,7 @@ impl Connection {
     }
 
     /// Corresponds to [s2n_connection_get_curve].
+    #[deprecated = "Use selected_key_exchange_group instead"]
     pub fn selected_curve(&self) -> Result<&str, Error> {
         let curve = unsafe { s2n_connection_get_curve(self.connection.as_ptr()).into_result()? };
         unsafe {
@@ -1123,6 +1138,25 @@ impl Connection {
         }
     }
 
+    /// Corresponds to [s2n_connection_get_key_exchange_group].
+    pub fn selected_key_exchange_group(&self) -> Option<&str> {
+        let mut group_name = core::ptr::null();
+        unsafe {
+            s2n_connection_get_key_exchange_group(self.connection.as_ptr(), &mut group_name)
+                .into_result()
+                .ok()
+        }?;
+
+        unsafe {
+            // SAFETY: The data is null terminated because it is declared as a C
+            //         string literal.
+            // SAFETY: group_name has a static lifetime because it lives on either
+            //         s2n_ecc_named_curve or s2n_kem, both of which are static
+            //         const structs.
+            const_str!(group_name).ok()
+        }
+    }
+
     /// Corresponds to [s2n_connection_get_selected_signature_algorithm].
     pub fn selected_signature_algorithm(&self) -> Result<SignatureAlgorithm, Error> {
         let mut sig_alg = s2n_tls_signature_algorithm::ANONYMOUS;
@@ -1143,6 +1177,16 @@ impl Connection {
         hash_alg.try_into()
     }
 
+    /// Corresponds to [s2n_connection_get_certificate_match].
+    pub fn certificate_match(&self) -> Result<CertSNIMatch, Error> {
+        let mut cert_match = s2n_cert_sni_match::SNI_NO_MATCH;
+        unsafe {
+            s2n_connection_get_certificate_match(self.connection.as_ptr(), &mut cert_match)
+                .into_result()?;
+        }
+        cert_match.try_into()
+    }
+
     /// Corresponds to [s2n_connection_get_selected_client_cert_signature_algorithm].
     pub fn selected_client_signature_algorithm(&self) -> Result<Option<SignatureAlgorithm>, Error> {
         let mut sig_alg = s2n_tls_signature_algorithm::ANONYMOUS;
@@ -1409,6 +1453,11 @@ impl Connection {
         }
     }
 
+    #[cfg(feature = "unstable-cert_authorities")]
+    pub(crate) fn cert_request_state(&mut self) -> &mut CertRequestState {
+        &mut self.context_mut().cert_request_state
+    }
+
     #[cfg(feature = "unstable-renegotiate")]
     pub(crate) fn renegotiate_state_mut(&mut self) -> &mut RenegotiateState {
         &mut self.context_mut().renegotiate_state
@@ -1429,6 +1478,8 @@ struct Context {
     app_context: Option<Box<dyn Any + Send + Sync>>,
     #[cfg(feature = "unstable-renegotiate")]
     pub(crate) renegotiate_state: RenegotiateState,
+    #[cfg(feature = "unstable-cert_authorities")]
+    pub(crate) cert_request_state: CertRequestState,
 }
 
 impl Context {
@@ -1442,6 +1493,8 @@ impl Context {
             app_context: None,
             #[cfg(feature = "unstable-renegotiate")]
             renegotiate_state: RenegotiateState::default(),
+            #[cfg(feature = "unstable-cert_authorities")]
+            cert_request_state: CertRequestState::default(),
         }
     }
 }
@@ -1548,6 +1601,7 @@ impl Drop for Connection {
 #[cfg(test)]
 mod tests {
     use super::*;
+    use crate::testing::{build_config, SniTestCerts, TestPair};
 
     // ensure the connection context is send
     #[test]
@@ -1629,4 +1683,73 @@ mod tests {
         // Retrieving the correct type succeeds.
         assert!(connection.application_context::<u32>().is_some());
     }
+
+    /// Test that the `certificate_match` Rust wrapper returns expected enum variant
+    /// for different SNI scenarios (None, NoMatch, ExactMatch)
+    #[test]
+    fn test_certificate_match_variants() -> Result<(), Box<dyn std::error::Error>> {
+        let scenarios = vec![
+            (None, CertSNIMatch::NoSNI),
+            (Some("nonmatching_sni"), CertSNIMatch::NoMatch),
+            (Some("127.0.0.1"), CertSNIMatch::ExactMatch),
+        ];
+
+        for (sni_opt, expected) in scenarios {
+            let config = build_config(&security::DEFAULT_TLS13)?;
+            let mut pair = TestPair::from_config(&config);
+
+            if let Some(sni) = sni_opt {
+                pair.client.set_server_name(sni)?;
+            }
+
+            pair.handshake()?;
+            let cert_match = pair.server.certificate_match()?;
+
+            assert_eq!(cert_match, expected,);
+        }
+
+        Ok(())
+    }
+
+    /// Test that the `certificate_match` Rust wrapper returns WildcardMatch enum
+    #[test]
+    fn test_certificate_match_returns_wildcard_match() -> Result<(), Box<dyn std::error::Error>> {
+        let wildcard_cert = SniTestCerts::WildcardInsectRsa.get();
+
+        let mut builder = crate::config::Builder::new();
+        builder.load_pem(wildcard_cert.cert(), wildcard_cert.key())?;
+        let server_config = builder.build()?;
+
+        let mut client_builder = crate::config::Builder::new();
+        client_builder.trust_pem(wildcard_cert.cert())?;
+        let client_config = client_builder.build()?;
+
+        let mut pair = TestPair::from_configs(&client_config, &server_config);
+
+        pair.client.set_server_name("anything.insect.hexapod")?;
+        pair.handshake()?;
+
+        let cert_match = pair.server.certificate_match()?;
+        assert_eq!(cert_match, CertSNIMatch::WildcardMatch);
+
+        Ok(())
+    }
+
+    /// Test that `unstable_as_ptr()` can be used to call an s2n_tls_sys API.
+    #[cfg(s2n_tls_external_build)]
+    #[test]
+    fn test_unstable_as_ptr() -> Result<(), Error> {
+        let mut connection = Connection::new_client();
+
+        let test_server_name = "test-server-name";
+        connection.set_server_name(test_server_name)?;
+
+        let server_name = unsafe {
+            let server_name = s2n_get_server_name(connection.unstable_as_ptr());
+            CStr::from_ptr(server_name).to_str().unwrap()
+        };
+
+        assert_eq!(server_name, test_server_name);
+        Ok(())
+    }
 }
