@@ -21,12 +21,12 @@ case "${S2N_BUILD_PRESET-default}" in
         : "${S2N_LIBCRYPTO:=awslc}"
         : "${GCC_VERSION:=9}"
         ;;
-    "awslc-fips_gcc4-8")
-        : "${S2N_LIBCRYPTO:=awslc-fips}"
+    "awslc-fips-2022_gcc4-8")
+        : "${S2N_LIBCRYPTO:=awslc-fips-2022}"
         : "${GCC_VERSION:=4.8}"
         ;;
-    "awslc-fips_gcc9")
-        : "${S2N_LIBCRYPTO:=awslc-fips}"
+    "awslc-fips-2022_gcc9")
+        : "${S2N_LIBCRYPTO:=awslc-fips-2022}"
         : "${GCC_VERSION:=9}"
         ;;
     "awslc-fips-2022_gcc6")
@@ -51,7 +51,7 @@ case "${S2N_BUILD_PRESET-default}" in
         ;;
     "openssl-1.0.2-fips")
         : "${S2N_LIBCRYPTO:=openssl-1.0.2-fips}"
-        : "${GCC_VERSION:=6}"
+        : "${GCC_VERSION:=4.8}"
         ;;
     "openssl-1.1.1_gcc4-8")
         : "${S2N_LIBCRYPTO:=openssl-1.1.1}"
@@ -75,5 +75,9 @@ case "${S2N_BUILD_PRESET-default}" in
         : "${S2N_LIBCRYPTO:=openssl-3.0}"
         : "${GCC_VERSION:=9}"
         ;;
+    "openssl-3.0-fips")
+        : "${S2N_LIBCRYPTO:=openssl-3.0-fips}"
+        : "${GCC_VERSION:=9}"
+        ;;
 esac
 
