@@ -59,19 +59,14 @@ int s2n_fips_init(void)
 
     /* When using Openssl, ONLY 3.0 currently supports FIPS.
      * openssl-1.0.2-fips is no longer supported.
-     * openssl > 3.0 will likely have a FIPS 140-3 certificate instead of a
+     * openssl >= 3.5 will likely have a FIPS 140-3 certificate instead of a
      * FIPS 140-2 certificate, which will require additional review in order
      * to properly integrate.
      */
-#if defined(OPENSSL_FIPS) || S2N_OPENSSL_VERSION_AT_LEAST(3, 1, 0)
+#if defined(OPENSSL_FIPS) || S2N_OPENSSL_VERSION_AT_LEAST(3, 5, 0)
     POSIX_ENSURE(!s2n_fips_mode_enabled, S2N_ERR_FIPS_MODE_UNSUPPORTED);
 #endif
 
-    /* For now, openssl is only supported for testing */
-    if (s2n_libcrypto_is_openssl_fips()) {
-        POSIX_ENSURE(s2n_in_unit_test(), S2N_ERR_FIPS_MODE_UNSUPPORTED);
-    }
-
     return S2N_SUCCESS;
 }
 
