@@ -16,20 +16,14 @@
 #pragma once
 
 #include <openssl/bn.h>
+#include <stdbool.h>
 #include <stdint.h>
 
-#include "api/s2n.h"
 #include "crypto/s2n_openssl.h"
-#include "crypto/s2n_rsa.h"
-#include "crypto/s2n_rsa_signing.h"
 
 #define RSA_PSS_SIGN_VERIFY_RANDOM_BLOB_SIZE 32
 #define RSA_PSS_SIGN_VERIFY_SIGNATURE_SIZE   256
 
-#ifndef EVP_PKEY_RSA_PSS
-    #define EVP_PKEY_RSA_PSS EVP_PKEY_NONE
-#endif
-
 /* OpenSSL 1.1.1d  10 Sep 2019 is broken, so disable on that version. For further info see: crypto/evp/p_lib.c:469
  *
  * This feature requires this Openssl commit for Openssl 1.1.x versions: openssl/openssl@4088b92
@@ -40,7 +34,5 @@
     #define RSA_PSS_CERTS_SUPPORTED 0
 #endif
 
-int s2n_is_rsa_pss_certs_supported();
-S2N_RESULT s2n_rsa_pss_pkey_init(struct s2n_pkey *pkey);
-S2N_RESULT s2n_evp_pkey_to_rsa_pss_public_key(struct s2n_rsa_key *rsa_key, EVP_PKEY *pkey);
-S2N_RESULT s2n_evp_pkey_to_rsa_pss_private_key(struct s2n_rsa_key *rsa_key, EVP_PKEY *pkey);
+bool s2n_is_rsa_pss_certs_supported();
+bool s2n_is_rsa_pss_signing_supported();
