@@ -32,7 +32,6 @@ void s2n_hash_is_available_harness()
     switch (alg) {
         case S2N_HASH_MD5:
         case S2N_HASH_MD5_SHA1:
-            assert(is_available == !s2n_is_in_fips_mode()); break;
         case S2N_HASH_NONE:
         case S2N_HASH_SHA1:
         case S2N_HASH_SHA224:
@@ -40,8 +39,6 @@ void s2n_hash_is_available_harness()
         case S2N_HASH_SHA384:
         case S2N_HASH_SHA512:
             assert(is_available); break;
-        case S2N_HASH_SENTINEL:
-            assert(!is_available); break;
         default:
             __CPROVER_assert(!is_available, "Unsupported algorithm.");
     }
