@@ -19,8 +19,6 @@
 
 #include <assert.h>
 
-int __CPROVER_file_local_s2n_hash_c_s2n_hash_set_impl(struct s2n_hash_state *);
-
 void s2n_hmac_digest_two_compression_rounds_harness()
 {
     /* Non-deterministic inputs. */
@@ -30,9 +28,6 @@ void s2n_hmac_digest_two_compression_rounds_harness()
 
     /* Assumptions. */
     __CPROVER_assume(s2n_result_is_ok(s2n_hmac_state_validate(state)));
-    __CPROVER_file_local_s2n_hash_c_s2n_hash_set_impl(&state->inner);
-    __CPROVER_file_local_s2n_hash_c_s2n_hash_set_impl(&state->outer);
-    __CPROVER_file_local_s2n_hash_c_s2n_hash_set_impl(&state->outer_just_key);
 
     /* Operation under verification. */
     if (s2n_hmac_digest_two_compression_rounds(state, out, size) == S2N_SUCCESS)
