@@ -15,11 +15,9 @@
 
 #include "utils/s2n_safety.h"
 
-void* s2n_ensure_memcpy_trace(void *restrict to, const void *restrict from, size_t size, const char *debug_str)
+void* s2n_ensure_memmove_trace(void *to, const void *from, size_t size)
 {
     if (to == NULL || from == NULL) {
-        s2n_errno = S2N_ERR_NULL;
-        s2n_debug_str = debug_str;
         return NULL;
     }
 
