@@ -17,18 +17,18 @@
 
 #include "crypto/s2n_fips.h"
 
-static int flag          = 0;
-static int s2n_fips_mode = 0;
+static bool s2n_fips_mode_flag = 0;
+static bool s2n_fips_mode_enabled = 0;
 
 /**
  * Return 1 if FIPS mode is set, 0 otherwise,
  * where FIPS mode is set nondeterministically on first call.
  */
-int s2n_is_in_fips_mode()
+bool s2n_is_in_fips_mode()
 {
-    if (flag == 0) {
-        s2n_fips_mode = nondet_bool() ? 1 : 0;
-        flag          = 1;
+    if (s2n_fips_mode_flag == 0) {
+        s2n_fips_mode_enabled = nondet_bool() ? 1 : 0;
+        s2n_fips_mode_flag = 1;
     }
-    return s2n_fips_mode;
+    return s2n_fips_mode_enabled;
 }
