@@ -22,10 +22,12 @@ int main()
     if (ctx == NULL) {
         return 1;
     }
-    if (!EVP_PKEY_CTX_kem_set_params(ctx, NID_MLKEM768)) {
+    if (!EVP_PKEY_CTX_kem_set_params(ctx, NID_MLKEM768)
+            || !EVP_PKEY_CTX_kem_set_params(ctx, NID_MLKEM1024)) {
         EVP_PKEY_CTX_free(ctx);
         return 1;
     }
+
     EVP_PKEY_CTX_free(ctx);
     return 0;
 }
