@@ -146,7 +146,7 @@ int s2n_fuzz_test(const uint8_t *buf, size_t len)
     struct s2n_connection *server_conn = s2n_connection_new(S2N_SERVER);
     POSIX_ENSURE_REF(server_conn);
     POSIX_GUARD(s2n_stuffer_write_bytes(&server_conn->handshake.io, buf, len));
-    server_conn->handshake_params.client_public_key.key.rsa_key.rsa = public_key.key.rsa_key.rsa;
+    server_conn->handshake_params.client_public_key.pkey = public_key.pkey;
 
     /* Pull a byte off the libfuzzer input and use it to set parameters */
     uint8_t randval = 0;
@@ -160,7 +160,7 @@ int s2n_fuzz_test(const uint8_t *buf, size_t len)
 
     /* Set the client_rsa_public_key so that it is not free'd during s2n_connection_free since it will be reused in
      * later fuzz tests  */
-    server_conn->handshake_params.client_public_key.key.rsa_key.rsa = NULL;
+    server_conn->handshake_params.client_public_key.pkey = NULL;
 
     /* Cleanup */
     POSIX_GUARD(s2n_connection_free(server_conn));
