@@ -2,26 +2,47 @@
 # SPDX-License-Identifier: Apache-2.0
 import copy
 import os
-import platform
 import pytest
 
-from configuration import available_ports, ALL_TEST_CIPHERS, ALL_TEST_CURVES, ALL_TEST_CERTS, PROTOCOLS, TLS13_CIPHERS
+from configuration import (
+    available_ports,
+    ALL_TEST_CIPHERS,
+    ALL_TEST_CURVES,
+    MINIMAL_TEST_CERTS,
+    PROTOCOLS,
+    TLS13_CIPHERS,
+)
 from common import ProviderOptions, Protocols, data_bytes
-from fixtures import managed_process  # lgtm [py/unused-import]
+from fixtures import managed_process  # noqa: F401
 from providers import Provider, S2N, OpenSSL
-from utils import invalid_test_parameters, get_parameter_name, get_expected_s2n_version, to_bytes
+from utils import (
+    invalid_test_parameters,
+    get_parameter_name,
+    get_expected_s2n_version,
+    to_bytes,
+)
 
 
 @pytest.mark.uncollect_if(func=invalid_test_parameters)
 @pytest.mark.parametrize("cipher", ALL_TEST_CIPHERS, ids=get_parameter_name)
 @pytest.mark.parametrize("curve", ALL_TEST_CURVES, ids=get_parameter_name)
-@pytest.mark.parametrize("certificate", ALL_TEST_CERTS, ids=get_parameter_name)
-@pytest.mark.parametrize("protocol", [p for p in PROTOCOLS if p != Protocols.TLS13], ids=get_parameter_name)
+@pytest.mark.parametrize("certificate", MINIMAL_TEST_CERTS, ids=get_parameter_name)
+@pytest.mark.parametrize(
+    "protocol", [p for p in PROTOCOLS if p != Protocols.TLS13], ids=get_parameter_name
+)
 @pytest.mark.parametrize("provider", [OpenSSL], ids=get_parameter_name)
 @pytest.mark.parametrize("other_provider", [S2N], ids=get_parameter_name)
 @pytest.mark.parametrize("use_ticket", [True, False])
-def test_session_resumption_s2n_server(managed_process, cipher, curve, certificate, protocol, provider, other_provider,
-                                       use_ticket):
+def test_session_resumption_s2n_server(
+    managed_process,  # noqa: F811
+    cipher,
+    curve,
+    certificate,
+    protocol,
+    provider,
+    other_provider,
+    use_ticket,
+):
     port = next(available_ports)
 
     client_options = ProviderOptions(
@@ -31,12 +52,13 @@ def test_session_resumption_s2n_server(managed_process, cipher, curve, certifica
         curve=curve,
         insecure=True,
         reconnect=True,
-        protocol=protocol)
+        protocol=protocol,
+    )
 
     server_options = copy.copy(client_options)
     server_options.reconnects_before_exit = 6
     server_options.mode = Provider.ServerMode
-    server_options.use_session_ticket = use_ticket,
+    server_options.use_session_ticket = (use_ticket,)
     server_options.key = certificate.key
     server_options.cert = certificate.cert
 
@@ -55,20 +77,34 @@ def test_session_resumption_s2n_server(managed_process, cipher, curve, certifica
     # S2N should indicate the procotol version in a successful connection.
     for results in server.get_results():
         results.assert_success()
-        assert results.stdout.count(
-            to_bytes("Actual protocol version: {}".format(expected_version))) == 6
+        assert (
+            results.stdout.count(
+                to_bytes("Actual protocol version: {}".format(expected_version))
+            )
+            == 6
+        )
 
 
 @pytest.mark.uncollect_if(func=invalid_test_parameters)
 @pytest.mark.parametrize("cipher", ALL_TEST_CIPHERS, ids=get_parameter_name)
 @pytest.mark.parametrize("curve", ALL_TEST_CURVES, ids=get_parameter_name)
-@pytest.mark.parametrize("certificate", ALL_TEST_CERTS, ids=get_parameter_name)
-@pytest.mark.parametrize("protocol", [p for p in PROTOCOLS if p != Protocols.TLS13], ids=get_parameter_name)
+@pytest.mark.parametrize("certificate", MINIMAL_TEST_CERTS, ids=get_parameter_name)
+@pytest.mark.parametrize(
+    "protocol", [p for p in PROTOCOLS if p != Protocols.TLS13], ids=get_parameter_name
+)
 @pytest.mark.parametrize("provider", [OpenSSL], ids=get_parameter_name)
 @pytest.mark.parametrize("other_provider", [S2N], ids=get_parameter_name)
 @pytest.mark.parametrize("use_ticket", [True, False])
-def test_session_resumption_s2n_client(managed_process, cipher, curve, protocol, provider, other_provider, certificate,
-                                       use_ticket):
+def test_session_resumption_s2n_client(
+    managed_process,  # noqa: F811
+    cipher,
+    curve,
+    protocol,
+    provider,
+    other_provider,
+    certificate,
+    use_ticket,
+):
     port = next(available_ports)
 
     client_options = ProviderOptions(
@@ -79,7 +115,8 @@ def test_session_resumption_s2n_client(managed_process, cipher, curve, protocol,
         insecure=True,
         reconnect=True,
         use_session_ticket=use_ticket,
-        protocol=protocol)
+        protocol=protocol,
+    )
 
     server_options = copy.copy(client_options)
     server_options.reconnects_before_exit = 6
@@ -96,8 +133,12 @@ def test_session_resumption_s2n_client(managed_process, cipher, curve, protocol,
     expected_version = get_expected_s2n_version(protocol, OpenSSL)
     for results in client.get_results():
         results.assert_success()
-        assert results.stdout.count(
-            to_bytes("Actual protocol version: {}".format(expected_version))) == 6
+        assert (
+            results.stdout.count(
+                to_bytes("Actual protocol version: {}".format(expected_version))
+            )
+            == 6
+        )
 
     for results in server.get_results():
         results.assert_success()
@@ -107,16 +148,24 @@ def test_session_resumption_s2n_client(managed_process, cipher, curve, protocol,
 @pytest.mark.uncollect_if(func=invalid_test_parameters)
 @pytest.mark.parametrize("cipher", TLS13_CIPHERS, ids=get_parameter_name)
 @pytest.mark.parametrize("curve", ALL_TEST_CURVES, ids=get_parameter_name)
-@pytest.mark.parametrize("certificate", ALL_TEST_CERTS, ids=get_parameter_name)
+@pytest.mark.parametrize("certificate", MINIMAL_TEST_CERTS, ids=get_parameter_name)
 @pytest.mark.parametrize("protocol", [Protocols.TLS13], ids=get_parameter_name)
 @pytest.mark.parametrize("provider", [OpenSSL], ids=get_parameter_name)
 @pytest.mark.parametrize("other_provider", [S2N], ids=get_parameter_name)
-def test_tls13_session_resumption_s2n_server(managed_process, tmp_path, cipher, curve, certificate, protocol, provider,
-                                             other_provider):
+def test_tls13_session_resumption_s2n_server(
+    managed_process,  # noqa: F811
+    tmp_path,
+    cipher,
+    curve,
+    certificate,
+    protocol,
+    provider,
+    other_provider,
+):
     port = str(next(available_ports))
 
     # Use temp directory to store session tickets
-    p = tmp_path / 'ticket.pem'
+    p = tmp_path / "ticket.pem"
     path_to_ticket = str(p)
 
     close_marker_bytes = data_bytes(10)
@@ -128,8 +177,9 @@ def test_tls13_session_resumption_s2n_server(managed_process, tmp_path, cipher,
         curve=curve,
         insecure=True,
         reconnect=False,
-        extra_flags=['-sess_out', path_to_ticket],
-        protocol=protocol)
+        extra_flags=["-sess_out", path_to_ticket],
+        protocol=protocol,
+    )
 
     server_options = copy.copy(client_options)
     server_options.mode = Provider.ServerMode
@@ -140,58 +190,73 @@ def test_tls13_session_resumption_s2n_server(managed_process, tmp_path, cipher,
     server_options.data_to_send = close_marker_bytes
 
     server = managed_process(
-        S2N, server_options, timeout=5, send_marker=S2N.get_send_marker())
-    client = managed_process(provider, client_options,
-                             timeout=5, close_marker=str(close_marker_bytes))
+        S2N, server_options, timeout=5, send_marker=S2N.get_send_marker()
+    )
+    client = managed_process(
+        provider, client_options, timeout=5, close_marker=str(close_marker_bytes)
+    )
 
     # The client should have received a session ticket
     for results in client.get_results():
         results.assert_success()
-        assert b'Post-Handshake New Session Ticket arrived:' in results.stdout
+        assert b"Post-Handshake New Session Ticket arrived:" in results.stdout
 
     for results in server.get_results():
         results.assert_success()
         # The first connection is a full handshake
-        assert b'Resumed session' not in results.stdout
+        assert b"Resumed session" not in results.stdout
 
     # Client inputs received session ticket to resume a session
     assert os.path.exists(path_to_ticket)
-    client_options.extra_flags = ['-sess_in', path_to_ticket]
+    client_options.extra_flags = ["-sess_in", path_to_ticket]
 
     port = str(next(available_ports))
     client_options.port = port
     server_options.port = port
 
     server = managed_process(
-        S2N, server_options, timeout=5, send_marker=S2N.get_send_marker())
-    client = managed_process(provider, client_options,
-                             timeout=5, close_marker=str(close_marker_bytes))
+        S2N, server_options, timeout=5, send_marker=S2N.get_send_marker()
+    )
+    client = managed_process(
+        provider, client_options, timeout=5, close_marker=str(close_marker_bytes)
+    )
 
     s2n_version = get_expected_s2n_version(protocol, provider)
 
     # Client has not read server certificate message as this is a resumed session
     for results in client.get_results():
         results.assert_success()
-        assert to_bytes(
-            "SSL_connect:SSLv3/TLS read server certificate") not in results.stderr
+        assert (
+            to_bytes("SSL_connect:SSLv3/TLS read server certificate")
+            not in results.stderr
+        )
 
     # The server should indicate a session has been resumed
     for results in server.get_results():
         results.assert_success()
-        assert b'Resumed session' in results.stdout
-        assert to_bytes("Actual protocol version: {}".format(
-            s2n_version)) in results.stdout
+        assert b"Resumed session" in results.stdout
+        assert (
+            to_bytes("Actual protocol version: {}".format(s2n_version))
+            in results.stdout
+        )
 
 
 @pytest.mark.uncollect_if(func=invalid_test_parameters)
 @pytest.mark.parametrize("cipher", TLS13_CIPHERS, ids=get_parameter_name)
 @pytest.mark.parametrize("curve", ALL_TEST_CURVES, ids=get_parameter_name)
-@pytest.mark.parametrize("certificate", ALL_TEST_CERTS, ids=get_parameter_name)
+@pytest.mark.parametrize("certificate", MINIMAL_TEST_CERTS, ids=get_parameter_name)
 @pytest.mark.parametrize("protocol", [Protocols.TLS13], ids=get_parameter_name)
 @pytest.mark.parametrize("provider", [OpenSSL, S2N], ids=get_parameter_name)
 @pytest.mark.parametrize("other_provider", [S2N], ids=get_parameter_name)
-def test_tls13_session_resumption_s2n_client(managed_process, cipher, curve, certificate, protocol, provider,
-                                             other_provider):
+def test_tls13_session_resumption_s2n_client(
+    managed_process,  # noqa: F811
+    cipher,
+    curve,
+    certificate,
+    protocol,
+    provider,
+    other_provider,
+):
     port = str(next(available_ports))
 
     # The reconnect option for s2nc allows the client to reconnect automatically
@@ -208,13 +273,16 @@ def test_tls13_session_resumption_s2n_client(managed_process, cipher, curve, cer
         insecure=True,
         use_session_ticket=True,
         reconnect=True,
-        protocol=protocol)
+        protocol=protocol,
+    )
 
     server_options = copy.copy(client_options)
     server_options.mode = Provider.ServerMode
     server_options.key = certificate.key
     server_options.cert = certificate.cert
-    server_options.reconnects_before_exit = num_resumed_connections + num_full_connections
+    server_options.reconnects_before_exit = (
+        num_resumed_connections + num_full_connections
+    )
 
     server = managed_process(provider, server_options, timeout=5)
     client = managed_process(S2N, client_options, timeout=5)
@@ -224,42 +292,55 @@ def test_tls13_session_resumption_s2n_client(managed_process, cipher, curve, cer
     # s2nc indicates the number of resumed connections in its output
     for results in client.get_results():
         results.assert_success()
-        assert results.stdout.count(
-            b'Resumed session') == num_resumed_connections
-        assert to_bytes("Actual protocol version: {}".format(
-            s2n_version)) in results.stdout
+        assert results.stdout.count(b"Resumed session") == num_resumed_connections
+        assert (
+            to_bytes("Actual protocol version: {}".format(s2n_version))
+            in results.stdout
+        )
 
-    server_accepts_str = str(
-        num_resumed_connections + num_full_connections) + " server accepts that finished"
+    server_accepts_str = (
+        str(num_resumed_connections + num_full_connections)
+        + " server accepts that finished"
+    )
 
     for results in server.get_results():
         results.assert_success()
         if provider is S2N:
-            assert results.stdout.count(
-                b'Resumed session') == num_resumed_connections
-            assert to_bytes("Actual protocol version: {}".format(
-                s2n_version)) in results.stdout
+            assert results.stdout.count(b"Resumed session") == num_resumed_connections
+            assert (
+                to_bytes("Actual protocol version: {}".format(s2n_version))
+                in results.stdout
+            )
         else:
             assert to_bytes(server_accepts_str) in results.stdout
             # s_server only writes one certificate message in all of the connections
-            assert results.stderr.count(
-                b'SSL_accept:SSLv3/TLS write certificate') == num_full_connections
+            assert (
+                results.stderr.count(b"SSL_accept:SSLv3/TLS write certificate")
+                == num_full_connections
+            )
 
 
-@pytest.mark.flaky(reruns=7, reruns_delay=2, condition=platform.machine().startswith("aarch"))
 @pytest.mark.uncollect_if(func=invalid_test_parameters)
 @pytest.mark.parametrize("cipher", TLS13_CIPHERS, ids=get_parameter_name)
 @pytest.mark.parametrize("curve", ALL_TEST_CURVES, ids=get_parameter_name)
-@pytest.mark.parametrize("certificate", ALL_TEST_CERTS, ids=get_parameter_name)
+@pytest.mark.parametrize("certificate", MINIMAL_TEST_CERTS, ids=get_parameter_name)
 @pytest.mark.parametrize("protocol", [Protocols.TLS13], ids=get_parameter_name)
 @pytest.mark.parametrize("provider", [OpenSSL], ids=get_parameter_name)
 @pytest.mark.parametrize("other_provider", [S2N], ids=get_parameter_name)
-def test_s2nd_falls_back_to_full_connection(managed_process, tmp_path, cipher, curve, certificate, protocol, provider,
-                                            other_provider):
+def test_s2nd_falls_back_to_full_connection(
+    managed_process,  # noqa: F811
+    tmp_path,
+    cipher,
+    curve,
+    certificate,
+    protocol,
+    provider,
+    other_provider,
+):
     port = str(next(available_ports))
 
     # Use temp directory to store session tickets
-    p = tmp_path / 'ticket.pem'
+    p = tmp_path / "ticket.pem"
     path_to_ticket = str(p)
 
     """
@@ -275,9 +356,10 @@ def test_s2nd_falls_back_to_full_connection(managed_process, tmp_path, cipher, c
         curve=curve,
         insecure=True,
         reconnect=False,
-        extra_flags=['-sess_out', path_to_ticket],
+        extra_flags=["-sess_out", path_to_ticket],
         data_to_send=data_bytes(4069),
-        protocol=protocol)
+        protocol=protocol,
+    )
 
     server_options = copy.copy(client_options)
     server_options.mode = Provider.ServerMode
@@ -285,22 +367,28 @@ def test_s2nd_falls_back_to_full_connection(managed_process, tmp_path, cipher, c
     server_options.cert = certificate.cert
     server_options.extra_flags = None
 
-    server = managed_process(provider, server_options, timeout=5)
-    client = managed_process(provider, client_options, timeout=5)
+    server = managed_process(provider, server_options)
+
+    # The send_marker prevents us from closing stdin until the client receives
+    # the session ticket. If we close stdin too early, then the client will exit
+    # the echo loop and stop listening for new messages.
+    client = managed_process(
+        provider, client_options, send_marker="TLS read server session ticket"
+    )
 
     # The client should have received a session ticket
     for results in client.get_results():
         results.assert_success()
-        assert b'Post-Handshake New Session Ticket arrived:' in results.stdout
+        assert b"Post-Handshake New Session Ticket arrived:" in results.stdout
 
     for results in server.get_results():
         results.assert_success()
         # Server should have sent certificate message as this is a full connection
-        assert b'SSL_accept:SSLv3/TLS write certificate' in results.stderr
+        assert b"SSL_accept:SSLv3/TLS write certificate" in results.stderr
 
     # Client inputs received session ticket to resume a session
     assert os.path.exists(path_to_ticket)
-    client_options.extra_flags = ['-sess_in', path_to_ticket]
+    client_options.extra_flags = ["-sess_in", path_to_ticket]
 
     port = str(next(available_ports))
     client_options.port = port
@@ -315,25 +403,38 @@ def test_s2nd_falls_back_to_full_connection(managed_process, tmp_path, cipher, c
     # Client has read server certificate because this is a full connection
     for results in client.get_results():
         results.assert_success()
-        assert to_bytes(
-            "SSL_connect:SSLv3/TLS read server certificate") in results.stderr
+        assert (
+            to_bytes("SSL_connect:SSLv3/TLS read server certificate") in results.stderr
+        )
 
     # The server should indicate a session has not been resumed
     for results in server.get_results():
         results.assert_success()
-        assert b'Resumed session' not in results.stdout
-        assert to_bytes("Actual protocol version: {}".format(
-            s2n_version)) in results.stdout
+        assert b"Resumed session" not in results.stdout
+        assert (
+            to_bytes("Actual protocol version: {}".format(s2n_version))
+            in results.stdout
+        )
 
 
 @pytest.mark.uncollect_if(func=invalid_test_parameters)
 @pytest.mark.parametrize("cipher", ALL_TEST_CIPHERS, ids=get_parameter_name)
 @pytest.mark.parametrize("curve", ALL_TEST_CURVES, ids=get_parameter_name)
-@pytest.mark.parametrize("certificate", ALL_TEST_CERTS, ids=get_parameter_name)
-@pytest.mark.parametrize("protocol", [p for p in PROTOCOLS if p < Protocols.TLS13], ids=get_parameter_name)
+@pytest.mark.parametrize("certificate", MINIMAL_TEST_CERTS, ids=get_parameter_name)
+@pytest.mark.parametrize(
+    "protocol", [p for p in PROTOCOLS if p < Protocols.TLS13], ids=get_parameter_name
+)
 @pytest.mark.parametrize("provider", [OpenSSL, S2N], ids=get_parameter_name)
 @pytest.mark.parametrize("other_provider", [S2N], ids=get_parameter_name)
-def test_session_resumption_s2n_client_tls13_server_not_tls13(managed_process, cipher, curve, protocol, provider, other_provider, certificate):
+def test_session_resumption_s2n_client_tls13_server_not_tls13(
+    managed_process,  # noqa: F811
+    cipher,
+    curve,
+    protocol,
+    provider,
+    other_provider,
+    certificate,
+):
     port = next(available_ports)
 
     # This test verifies that an S2N client that supports TLS1.3 can resume sessions
@@ -353,7 +454,8 @@ def test_session_resumption_s2n_client_tls13_server_not_tls13(managed_process, c
         insecure=True,
         reconnect=True,
         use_session_ticket=True,
-        protocol=Protocols.TLS13)
+        protocol=Protocols.TLS13,
+    )
 
     server_options = ProviderOptions(
         mode=Provider.ServerMode,
@@ -366,7 +468,8 @@ def test_session_resumption_s2n_client_tls13_server_not_tls13(managed_process, c
         protocol=protocol,
         reconnects_before_exit=num_resumed_connections + num_full_connections,
         key=certificate.key,
-        cert=certificate.cert)
+        cert=certificate.cert,
+    )
 
     # Passing the type of client and server as a parameter will
     # allow us to use a fixture to enumerate all possibilities.
@@ -374,22 +477,26 @@ def test_session_resumption_s2n_client_tls13_server_not_tls13(managed_process, c
     client = managed_process(S2N, client_options, timeout=5)
 
     expected_version = get_expected_s2n_version(protocol, provider)
-    server_accepts_str = str(
-        num_resumed_connections + num_full_connections) + " server accepts that finished"
+    server_accepts_str = (
+        str(num_resumed_connections + num_full_connections)
+        + " server accepts that finished"
+    )
 
     for results in client.get_results():
         results.assert_success()
-        assert results.stdout.count(
-            b'Resumed session') == num_resumed_connections
-        assert to_bytes("Actual protocol version: {}".format(
-            expected_version)) in results.stdout
+        assert results.stdout.count(b"Resumed session") == num_resumed_connections
+        assert (
+            to_bytes("Actual protocol version: {}".format(expected_version))
+            in results.stdout
+        )
 
     for results in server.get_results():
         results.assert_success()
         if provider is S2N:
-            assert results.stdout.count(
-                b'Resumed session') == num_resumed_connections
-            assert to_bytes("Actual protocol version: {}".format(
-                expected_version)) in results.stdout
+            assert results.stdout.count(b"Resumed session") == num_resumed_connections
+            assert (
+                to_bytes("Actual protocol version: {}".format(expected_version))
+                in results.stdout
+            )
         else:
             assert to_bytes(server_accepts_str) in results.stdout
