@@ -78,6 +78,8 @@ S2N_RESULT s2n_check_supported_libcrypto(const char *s2n_libcrypto)
         { .libcrypto = "awslc", .is_openssl = false },
         { .libcrypto = "awslc-fips", .is_openssl = false },
         { .libcrypto = "awslc-fips-2022", .is_openssl = false },
+        { .libcrypto = "awslc-fips-2024", .is_openssl = false },
+        { .libcrypto = "awslc-fips-next", .is_openssl = false },
         { .libcrypto = "boringssl", .is_openssl = false },
         { .libcrypto = "libressl", .is_openssl = false },
         { .libcrypto = "openssl-1.0.2", .is_openssl = true },
@@ -85,7 +87,6 @@ S2N_RESULT s2n_check_supported_libcrypto(const char *s2n_libcrypto)
         { .libcrypto = "openssl-1.1.1", .is_openssl = true },
         { .libcrypto = "openssl-3.0", .is_openssl = true },
         { .libcrypto = "openssl-3.0-fips", .is_openssl = true, .is_opensslfips = true },
-        { .libcrypto = "openssl-3.4", .is_openssl = true },
     };
 
     for (size_t i = 0; i < s2n_array_len(supported_libcrypto); i++) {
@@ -122,7 +123,6 @@ int main()
 
     if (s2n_libcrypto == NULL || s2n_libcrypto[0] == '\0') {
         END_TEST();
-        return 0;
     }
 
     if (strcmp(s2n_libcrypto, "default") == 0) {
@@ -164,7 +164,15 @@ int main()
     {
         if (version != NULL) {
             const char *ssleay_version_text = SSLeay_version(SSLEAY_VERSION);
-            EXPECT_NOT_NULL(strstr(ssleay_version_text, version));
+            if (strstr(ssleay_version_text, version) == NULL) {
+                char fail_msg[256] = { 0 };
+                snprintf(
+                        fail_msg, sizeof(fail_msg),
+                        "Libcrypto version mismatch: expected version '%s' "
+                        "not found in actual libcrypto version string '%s'",
+                        version, ssleay_version_text);
+                FAIL_MSG(fail_msg);
+            }
         }
     };
 
@@ -178,5 +186,4 @@ int main()
     };
 
     END_TEST();
-    return 0;
 }
