@@ -20,8 +20,8 @@
 
 int main(int argc, char **argv)
 {
-    struct s2n_connection *conn;
-    int ctx;
+    struct s2n_connection *conn = NULL;
+    int ctx = 0;
 
     struct s2n_connection *conn_null = NULL;
 
