@@ -245,12 +245,14 @@ int main(int argc, char **argv)
         EXPECT_SUCCESS(s2n_connection_get_selected_digest_algorithm(conn, &output));
         EXPECT_EQUAL(S2N_TLS_HASH_NONE, output);
 
-        s2n_tls_hash_algorithm expected_output[] = {
-            S2N_TLS_HASH_NONE, S2N_TLS_HASH_MD5,
-            S2N_TLS_HASH_SHA1, S2N_TLS_HASH_SHA224,
-            S2N_TLS_HASH_SHA256, S2N_TLS_HASH_SHA384,
-            S2N_TLS_HASH_SHA512, S2N_TLS_HASH_MD5_SHA1,
-            S2N_TLS_HASH_NONE
+        s2n_tls_hash_algorithm expected_output[S2N_HASH_ALGS_COUNT] = {
+            [S2N_HASH_MD5] = S2N_TLS_HASH_MD5,
+            [S2N_HASH_SHA1] = S2N_TLS_HASH_SHA1,
+            [S2N_HASH_SHA224] = S2N_TLS_HASH_SHA224,
+            [S2N_HASH_SHA256] = S2N_TLS_HASH_SHA256,
+            [S2N_HASH_SHA384] = S2N_TLS_HASH_SHA384,
+            [S2N_HASH_SHA512] = S2N_TLS_HASH_SHA512,
+            [S2N_HASH_MD5_SHA1] = S2N_TLS_HASH_MD5_SHA1,
         };
 
         for (size_t i = S2N_TLS_HASH_NONE; i <= UINT16_MAX; i++) {
@@ -258,7 +260,7 @@ int main(int argc, char **argv)
             test_scheme.hash_alg = i;
             conn->handshake_params.client_cert_sig_scheme = &test_scheme;
             conn->handshake_params.server_cert_sig_scheme = &test_scheme;
-            if (i <= S2N_HASH_ALGS_COUNT) {
+            if (i < S2N_HASH_ALGS_COUNT) {
                 EXPECT_SUCCESS(s2n_connection_get_selected_client_cert_digest_algorithm(conn, &output));
                 EXPECT_EQUAL(expected_output[i], output);
 
@@ -300,6 +302,7 @@ int main(int argc, char **argv)
             [S2N_SIGNATURE_ECDSA] = S2N_TLS_SIGNATURE_ECDSA,
             [S2N_SIGNATURE_RSA_PSS_RSAE] = S2N_TLS_SIGNATURE_RSA_PSS_RSAE,
             [S2N_SIGNATURE_RSA_PSS_PSS] = S2N_TLS_SIGNATURE_RSA_PSS_PSS,
+            [S2N_SIGNATURE_MLDSA] = S2N_TLS_SIGNATURE_MLDSA,
         };
 
         for (size_t i = 0; i <= UINT16_MAX; i++) {
@@ -633,7 +636,7 @@ int main(int argc, char **argv)
          * s2n_connection_set_config will fail 
          */
         {
-            struct s2n_security_policy rfc9151_applied_locally = security_policy_rfc9151;
+            struct s2n_security_policy rfc9151_applied_locally = security_policy_20250429;
             rfc9151_applied_locally.certificate_preferences_apply_locally = true;
 
             DEFER_CLEANUP(struct s2n_config *config = s2n_config_new(), s2n_config_ptr_free);
@@ -1007,6 +1010,103 @@ int main(int argc, char **argv)
         };
     };
 
+    /* Test s2n_connection_get_signature_scheme */
+    {
+        /* Safety */
+        {
+            struct s2n_connection conn = { 0 };
+            const char *name = NULL;
+            EXPECT_FAILURE_WITH_ERRNO(s2n_connection_get_signature_scheme(&conn, NULL),
+                    S2N_ERR_NULL);
+            EXPECT_FAILURE_WITH_ERRNO(s2n_connection_get_signature_scheme(NULL, &name),
+                    S2N_ERR_NULL);
+        };
+
+        /* Test: connection without a handshake returns an error */
+        {
+            DEFER_CLEANUP(struct s2n_connection *conn = s2n_connection_new(S2N_CLIENT),
+                    s2n_connection_ptr_free);
+            const char *name = NULL;
+            EXPECT_FAILURE_WITH_ERRNO(s2n_connection_get_signature_scheme(conn, &name),
+                    S2N_ERR_INVALID_STATE);
+        };
+
+        /* Test: all valid signature schemes succeed */
+        {
+            const struct s2n_signature_preferences *all_prefs = &s2n_signature_preferences_all;
+            for (size_t i = 0; i < all_prefs->count; i++) {
+                for (size_t version = 0; version <= S2N_TLS13; version++) {
+                    DEFER_CLEANUP(struct s2n_connection *conn = s2n_connection_new(S2N_CLIENT),
+                            s2n_connection_ptr_free);
+                    conn->handshake.handshake_type = NEGOTIATED;
+                    conn->handshake_params.server_cert_sig_scheme = all_prefs->signature_schemes[i];
+                    conn->actual_protocol_version = version;
+
+                    const char *name = NULL;
+                    EXPECT_SUCCESS(s2n_connection_get_signature_scheme(conn, &name));
+                    EXPECT_NOT_NULL(name);
+                }
+            }
+        };
+
+        /* Test: simple known values */
+        {
+            DEFER_CLEANUP(struct s2n_connection *conn = s2n_connection_new(S2N_CLIENT),
+                    s2n_connection_ptr_free);
+            conn->handshake.handshake_type = NEGOTIATED;
+
+            conn->handshake_params.server_cert_sig_scheme = &s2n_rsa_pss_rsae_sha256;
+            const char *rsa_name = NULL;
+            EXPECT_SUCCESS(s2n_connection_get_signature_scheme(conn, &rsa_name));
+            EXPECT_STRING_EQUAL(rsa_name, "rsa_pss_rsae_sha256");
+
+            conn->handshake_params.server_cert_sig_scheme = &s2n_mldsa65;
+            const char *mldsa_name = NULL;
+            EXPECT_SUCCESS(s2n_connection_get_signature_scheme(conn, &mldsa_name));
+            EXPECT_STRING_EQUAL(mldsa_name, "mldsa65");
+
+            conn->handshake_params.server_cert_sig_scheme = &s2n_ecdsa_sha224;
+            const char *legacy_ecdsa = NULL;
+            EXPECT_SUCCESS(s2n_connection_get_signature_scheme(conn, &legacy_ecdsa));
+            EXPECT_STRING_EQUAL(legacy_ecdsa, "legacy_ecdsa_sha224");
+        };
+
+        /* Test: connection without a chosen signature scheme returns "none" */
+        {
+            DEFER_CLEANUP(struct s2n_connection *conn = s2n_connection_new(S2N_CLIENT),
+                    s2n_connection_ptr_free);
+            conn->handshake.handshake_type = NEGOTIATED;
+            const char *name = NULL;
+            EXPECT_SUCCESS(s2n_connection_get_signature_scheme(conn, &name));
+            EXPECT_STRING_EQUAL(name, "none");
+        };
+
+        /* Test: chooses ECDSA name based on version */
+        {
+            DEFER_CLEANUP(struct s2n_connection *conn = s2n_connection_new(S2N_CLIENT),
+                    s2n_connection_ptr_free);
+            conn->handshake.handshake_type = NEGOTIATED;
+            conn->handshake_params.server_cert_sig_scheme = &s2n_ecdsa_sha256;
+
+            const char *tls13_name = NULL;
+            conn->actual_protocol_version = S2N_TLS13;
+            EXPECT_SUCCESS(s2n_connection_get_signature_scheme(conn, &tls13_name));
+            EXPECT_STRING_EQUAL(tls13_name, s2n_ecdsa_sha256.tls13_name);
+            EXPECT_STRING_EQUAL(tls13_name, "ecdsa_secp256r1_sha256");
+
+            const char *tls12_name = NULL;
+            conn->actual_protocol_version = S2N_TLS12;
+            EXPECT_SUCCESS(s2n_connection_get_signature_scheme(conn, &tls12_name));
+            EXPECT_STRING_EQUAL(tls12_name, s2n_ecdsa_sha256.legacy_name);
+            EXPECT_STRING_EQUAL(tls12_name, "legacy_ecdsa_sha256");
+
+            const char *tls10_name = NULL;
+            conn->actual_protocol_version = S2N_TLS10;
+            EXPECT_SUCCESS(s2n_connection_get_signature_scheme(conn, &tls10_name));
+            EXPECT_STRING_EQUAL(tls10_name, s2n_ecdsa_sha256.legacy_name);
+        };
+    };
+
     EXPECT_SUCCESS(s2n_cert_chain_and_key_free(ecdsa_chain_and_key));
     EXPECT_SUCCESS(s2n_cert_chain_and_key_free(rsa_chain_and_key));
     END_TEST();
