@@ -25,6 +25,12 @@ int main(int argc, char **argv)
 {
     BEGIN_TEST();
 
+    /* Not all hash algorithms are used in the handshake hashes */
+    const bool skip_handshake_hash[S2N_HASH_ALGS_COUNT] = {
+        [S2N_HASH_NONE] = true,
+        [S2N_HASH_SHAKE256_64] = true,
+    };
+
     /* Test s2n_handshake_hashes_new */
     {
         /* Safety */
@@ -42,7 +48,7 @@ int main(int argc, char **argv)
 
             /* Allocates all hashes */
             for (s2n_hash_algorithm alg = 0; alg < S2N_HASH_ALGS_COUNT; alg++) {
-                if (alg == S2N_HASH_NONE) {
+                if (skip_handshake_hash[alg]) {
                     continue;
                 }
 
@@ -75,7 +81,7 @@ int main(int argc, char **argv)
             uint8_t data[100] = { 0 };
 
             for (s2n_hash_algorithm alg = 0; alg < S2N_HASH_ALGS_COUNT; alg++) {
-                if (alg == S2N_HASH_NONE) {
+                if (skip_handshake_hash[alg]) {
                     continue;
                 }
 
