@@ -417,6 +417,11 @@ int main(int argc, char **argv)
     BEGIN_TEST();
     EXPECT_SUCCESS(s2n_disable_tls13_in_test());
 
+    /* FIPS always requires libcrypto hkdf */
+    if (s2n_is_in_fips_mode()) {
+        EXPECT_TRUE(s2n_libcrypto_supports_hkdf());
+    }
+
     EXPECT_SUCCESS(s2n_hmac_new(&hmac));
 
     for (uint8_t i = 0; i < NUM_TESTS; i++) {
