@@ -45,7 +45,7 @@ static int custom_mem_cleanup(void)
 
 static int custom_mem_malloc(void **ptr, uint32_t requested, uint32_t *allocated)
 {
-    int i;
+    int i = 0;
     for (i = 0; i < HISTOGRAM_SIZE; i++) {
         if (histogram_values[i] == 0) {
             histogram_values[i] = requested;
@@ -78,8 +78,8 @@ static int custom_mem_free(void *ptr, uint32_t size)
 void mock_client(struct s2n_test_io_pair *io_pair)
 {
     char buffer[0xffff];
-    struct s2n_connection *conn;
-    struct s2n_config *config;
+    struct s2n_connection *conn = NULL;
+    struct s2n_config *config = NULL;
     s2n_blocked_status blocked;
 
     /* Give the server a chance to listen */
@@ -101,7 +101,7 @@ void mock_client(struct s2n_test_io_pair *io_pair)
 
     uint16_t timeout = 1;
     s2n_connection_set_dynamic_record_threshold(conn, 0x7fff, timeout);
-    int i;
+    int i = 0;
     for (i = 1; i < 0xffff - 100; i += 100) {
         for (int j = 0; j < i; j++) {
             buffer[j] = 33;
@@ -118,7 +118,7 @@ void mock_client(struct s2n_test_io_pair *io_pair)
 
     /* Simulate timeout second conneciton inactivity and tolerate 50 ms error */
     struct timespec sleep_time = { .tv_sec = timeout, .tv_nsec = 50000000 };
-    int r;
+    int r = 0;
     do {
         r = nanosleep(&sleep_time, &sleep_time);
     } while (r != 0);
@@ -148,11 +148,10 @@ void mock_client(struct s2n_test_io_pair *io_pair)
 int main(int argc, char **argv)
 {
     s2n_blocked_status blocked;
-    int status;
-    pid_t pid;
-    char *cert_chain_pem;
-    char *private_key_pem;
-    char *dhparams_pem;
+    int status = 0;
+    char *cert_chain_pem = NULL;
+    char *private_key_pem = NULL;
+    char *dhparams_pem = NULL;
 
     /* We have to set the callback before BEGIN_TEST, because s2n_init() is called
      * there.
@@ -175,7 +174,7 @@ int main(int argc, char **argv)
         EXPECT_SUCCESS(s2n_io_pair_init(&io_pair));
 
         /* Create a child process */
-        pid = fork();
+        pid_t pid = fork();
         if (pid == 0) {
             /* This is the client process, close the server end of the pipe */
             EXPECT_SUCCESS(s2n_io_pair_close_one_end(&io_pair, S2N_SERVER));
