@@ -902,6 +902,12 @@ int main(int argc, char **argv)
         if (s2n_libcrypto_is_awslc()) {
 #if defined(S2N_LIBCRYPTO_SUPPORTS_ENGINE)
             FAIL_MSG("Expected ENGINE feature probe to be disabled with AWS-LC");
+#endif
+        }
+
+        if (s2n_libcrypto_is_openssl_fips()) {
+#if !S2N_LIBCRYPTO_SUPPORTS_PRIVATE_RAND
+            FAIL_MSG("Expected private rand support from openssl3 fips");
 #endif
         }
     };
