@@ -25,21 +25,21 @@
 const s2n_mode modes[] = { S2N_SERVER, S2N_CLIENT };
 
 static uint8_t test_send_key[S2N_TLS_AES_256_GCM_KEY_LEN] = { 0 };
-static int s2n_test_set_send_key(struct s2n_session_key *key, struct s2n_blob *in)
+static S2N_RESULT s2n_test_set_send_key(struct s2n_session_key *key, struct s2n_blob *in)
 {
-    POSIX_ENSURE_REF(key);
-    POSIX_ENSURE_REF(in);
-    POSIX_CHECKED_MEMCPY(test_send_key, in->data, in->size);
-    return S2N_SUCCESS;
+    RESULT_ENSURE_REF(key);
+    RESULT_ENSURE_REF(in);
+    RESULT_CHECKED_MEMCPY(test_send_key, in->data, in->size);
+    return S2N_RESULT_OK;
 }
 
 static uint8_t test_recv_key[S2N_TLS_AES_256_GCM_KEY_LEN] = { 0 };
-static int s2n_test_set_recv_key(struct s2n_session_key *key, struct s2n_blob *in)
+static S2N_RESULT s2n_test_set_recv_key(struct s2n_session_key *key, struct s2n_blob *in)
 {
-    POSIX_ENSURE_REF(key);
-    POSIX_ENSURE_REF(in);
-    POSIX_CHECKED_MEMCPY(test_recv_key, in->data, in->size);
-    return S2N_SUCCESS;
+    RESULT_ENSURE_REF(key);
+    RESULT_ENSURE_REF(in);
+    RESULT_CHECKED_MEMCPY(test_recv_key, in->data, in->size);
+    return S2N_RESULT_OK;
 }
 
 #define EXPECT_IVS_EQUAL(conn, iv, iv_mode)                                               \
@@ -107,12 +107,12 @@ int main(int argc, char **argv)
         /* Derive server handshake traffic keys */
         {
             /**
-             *= https://www.rfc-editor.org/rfc/rfc8448.html#section-3
+             *= https://www.rfc-editor.org/rfc/rfc8448#section-3
              *= type=test
              *#    {client}  derive read traffic keys for handshake data (same as server
              *#        handshake data write traffic keys)
              *
-             *= https://www.rfc-editor.org/rfc/rfc8448.html#section-3
+             *= https://www.rfc-editor.org/rfc/rfc8448#section-3
              *= type=test
              *#    {server}  derive write traffic keys for handshake data:
              *#
@@ -153,12 +153,12 @@ int main(int argc, char **argv)
         /* Derive client handshake traffic keys */
         {
             /**
-             *= https://www.rfc-editor.org/rfc/rfc8448.html#section-3
+             *= https://www.rfc-editor.org/rfc/rfc8448#section-3
              *= type=test
              *#    {client}  derive write traffic keys for handshake data (same as
              *#       server handshake data read traffic keys)
              *
-             *= https://www.rfc-editor.org/rfc/rfc8448.html#section-3
+             *= https://www.rfc-editor.org/rfc/rfc8448#section-3
              *= type=test
              *#    {server}  derive read traffic keys for handshake data:
              *#
@@ -200,12 +200,12 @@ int main(int argc, char **argv)
         /* Derive server application traffic keys */
         {
             /**
-             *= https://www.rfc-editor.org/rfc/rfc8448.html#section-3
+             *= https://www.rfc-editor.org/rfc/rfc8448#section-3
              *= type=test
              *#    {client}  derive read traffic keys for application data (same as
              *#       server application data write traffic keys)
              *
-             *= https://www.rfc-editor.org/rfc/rfc8448.html#section-3
+             *= https://www.rfc-editor.org/rfc/rfc8448#section-3
              *= type=test
              *#    {server}  derive write traffic keys for application data:
              *#
@@ -253,12 +253,12 @@ int main(int argc, char **argv)
         /* Derive client application traffic keys */
         {
             /**
-             *= https://www.rfc-editor.org/rfc/rfc8448.html#section-3
+             *= https://www.rfc-editor.org/rfc/rfc8448#section-3
              *= type=test
              *#    {server}  derive read traffic keys for application data (same as
              *#       client application data write traffic keys)
              *
-             *= https://www.rfc-editor.org/rfc/rfc8448.html#section-3
+             *= https://www.rfc-editor.org/rfc/rfc8448#section-3
              *= type=test
              *#    {client}  derive write traffic keys for application data:
              *#
@@ -309,12 +309,12 @@ int main(int argc, char **argv)
         /* Derive early application traffic keys */
         {
             /**
-             *= https://www.rfc-editor.org/rfc/rfc8448.html#section-4
+             *= https://www.rfc-editor.org/rfc/rfc8448#section-4
              *= type=test
              *#    {server}  derive read traffic keys for early application data (same
              *#       as client early application data write traffic keys)
              *
-             *= https://www.rfc-editor.org/rfc/rfc8448.html#section-4
+             *= https://www.rfc-editor.org/rfc/rfc8448#section-4
              *= type=test
              *#    {client}  derive write traffic keys for early application data:
              *#
