@@ -22,7 +22,7 @@
 #include "tls/s2n_tls13.h"
 
 struct s2n_cipher_preferences {
-    uint8_t count;
+    uint16_t count;
     struct s2n_cipher_suite **suites;
     bool allow_chacha20_boosting;
 };
@@ -50,7 +50,6 @@ extern const struct s2n_cipher_preferences cipher_preferences_20190801;
 extern const struct s2n_cipher_preferences cipher_preferences_20190120;
 extern const struct s2n_cipher_preferences cipher_preferences_20190121;
 extern const struct s2n_cipher_preferences cipher_preferences_20190122;
-extern const struct s2n_cipher_preferences cipher_preferences_rfc9151;
 extern const struct s2n_cipher_preferences cipher_preferences_20210816;
 extern const struct s2n_cipher_preferences cipher_preferences_20210816_gcm;
 extern const struct s2n_cipher_preferences cipher_preferences_20210825;
@@ -63,6 +62,7 @@ extern const struct s2n_cipher_preferences cipher_preferences_20241008;
 extern const struct s2n_cipher_preferences cipher_preferences_20241008_gcm;
 extern const struct s2n_cipher_preferences cipher_preferences_20241009;
 extern const struct s2n_cipher_preferences cipher_preferences_20250211;
+extern const struct s2n_cipher_preferences cipher_preferences_20250429;
 
 extern const struct s2n_cipher_preferences cipher_preferences_default_fips;
 
@@ -94,6 +94,8 @@ extern const struct s2n_cipher_preferences cipher_preferences_cloudfront_upstrea
 extern const struct s2n_cipher_preferences cipher_preferences_cloudfront_upstream_tls10;
 extern const struct s2n_cipher_preferences cipher_preferences_cloudfront_upstream_tls11;
 extern const struct s2n_cipher_preferences cipher_preferences_cloudfront_upstream_tls12;
+extern const struct s2n_cipher_preferences cipher_preferences_cloudfront_upstream_2025_08_08;
+extern const struct s2n_cipher_preferences cipher_preferences_cloudfront_upstream_2025_08_08_tls13;
 /* CloudFront viewer facing */
 extern const struct s2n_cipher_preferences cipher_preferences_cloudfront_ssl_v_3;
 extern const struct s2n_cipher_preferences cipher_preferences_cloudfront_tls_1_0_2014;
@@ -117,6 +119,7 @@ extern const struct s2n_cipher_preferences cipher_preferences_cloudfront_tls_1_2
 extern const struct s2n_cipher_preferences cipher_preferences_aws_crt_sdk_ssl_v3;
 extern const struct s2n_cipher_preferences cipher_preferences_aws_crt_sdk_default;
 extern const struct s2n_cipher_preferences cipher_preferences_aws_crt_sdk_tls_13;
+extern const struct s2n_cipher_preferences cipher_preferences_aws_crt_sdk_2025;
 
 /* AWS KMS Cipher Preferences */
 extern const struct s2n_cipher_preferences cipher_preferences_kms_tls_1_0_2018_10;
