@@ -45,6 +45,7 @@
 #include "tls/s2n_resume.h"
 #include "tls/s2n_security_policies.h"
 #include "tls/s2n_tls.h"
+#include "tls/s2n_tls13_handshake.h"
 #include "tls/s2n_tls_parameters.h"
 #include "utils/s2n_atomic.h"
 #include "utils/s2n_blob.h"
@@ -273,9 +274,11 @@ int s2n_connection_free(struct s2n_connection *conn)
     POSIX_GUARD(s2n_stuffer_free(&conn->handshake.io));
     POSIX_GUARD(s2n_stuffer_free(&conn->post_handshake.in));
     s2n_x509_validator_wipe(&conn->x509_validator);
+    POSIX_GUARD_RESULT(s2n_async_offload_op_wipe(&conn->async_offload_op));
     POSIX_GUARD(s2n_client_hello_free_raw_message(&conn->client_hello));
     POSIX_GUARD(s2n_free(&conn->application_protocols_overridden));
     POSIX_GUARD(s2n_free(&conn->cookie));
+    POSIX_GUARD(s2n_free(&conn->cert_authorities));
     POSIX_GUARD_RESULT(s2n_crypto_parameters_free(&conn->initial));
     POSIX_GUARD_RESULT(s2n_crypto_parameters_free(&conn->secure));
     POSIX_GUARD(s2n_free_object((uint8_t **) &conn, sizeof(struct s2n_connection)));
@@ -453,6 +456,7 @@ int s2n_connection_free_handshake(struct s2n_connection *conn)
     POSIX_GUARD(s2n_free(&conn->our_quic_transport_parameters));
     POSIX_GUARD(s2n_free(&conn->application_protocols_overridden));
     POSIX_GUARD(s2n_free(&conn->cookie));
+    POSIX_GUARD(s2n_free(&conn->cert_authorities));
 
     return 0;
 }
@@ -519,6 +523,7 @@ int s2n_connection_wipe(struct s2n_connection *conn)
     POSIX_GUARD(s2n_stuffer_free(&conn->in));
 
     POSIX_GUARD_RESULT(s2n_psk_parameters_wipe(&conn->psk_params));
+    POSIX_GUARD_RESULT(s2n_async_offload_op_wipe(&conn->async_offload_op));
 
     /* Wipe the I/O-related info and restore the original socket if necessary */
     POSIX_GUARD(s2n_connection_wipe_io(conn));
@@ -531,6 +536,7 @@ int s2n_connection_wipe(struct s2n_connection *conn)
     POSIX_GUARD(s2n_free(&conn->server_early_data_context));
     POSIX_GUARD(s2n_free(&conn->tls13_ticket_fields.session_secret));
     POSIX_GUARD(s2n_free(&conn->cookie));
+    POSIX_GUARD(s2n_free(&conn->cert_authorities));
 
     /* Allocate memory for handling handshakes */
     POSIX_GUARD(s2n_stuffer_resize(&conn->handshake.io, S2N_LARGE_RECORD_LENGTH));
@@ -1004,6 +1010,25 @@ const char *s2n_connection_get_kem_group_name(struct s2n_connection *conn)
     return conn->kex_params.server_kem_group_params.kem_group->name;
 }
 
+int s2n_connection_get_key_exchange_group(struct s2n_connection *conn, const char **group_name)
+{
+    POSIX_ENSURE_REF(conn);
+    POSIX_ENSURE_REF(group_name);
+
+    /* s2n_connection_get_curve returns only the ECDH curve portion of a named group, even if 
+       the negotiated group was a hybrid PQ key exchange also containing a KEM. Therefore,
+       we use the result of s2n_connection_get_kem_group_name if the connection supports PQ. */
+    if (s2n_tls13_pq_hybrid_supported(conn)) {
+        *group_name = s2n_connection_get_kem_group_name(conn);
+    } else {
+        *group_name = s2n_connection_get_curve(conn);
+    }
+
+    POSIX_ENSURE(*group_name != NULL && strcmp(*group_name, "NONE"), S2N_ERR_INVALID_STATE);
+
+    return S2N_SUCCESS;
+}
+
 static S2N_RESULT s2n_connection_get_client_supported_version(struct s2n_connection *conn,
         uint8_t *client_supported_version)
 {
@@ -1077,7 +1102,11 @@ int s2n_connection_get_client_hello_version(struct s2n_connection *conn)
 {
     POSIX_ENSURE_REF(conn);
 
-    return conn->client_hello_version;
+    if (conn->client_hello.sslv2) {
+        return S2N_SSLv2;
+    } else {
+        return MIN(conn->client_hello.legacy_version, S2N_TLS12);
+    }
 }
 
 int s2n_connection_client_cert_used(struct s2n_connection *conn)
@@ -1572,6 +1601,7 @@ static S2N_RESULT s2n_signature_scheme_to_tls_iana(const struct s2n_signature_sc
 {
     RESULT_ENSURE_REF(sig_scheme);
     RESULT_ENSURE_REF(converted_scheme);
+    *converted_scheme = S2N_TLS_HASH_NONE;
 
     switch (sig_scheme->hash_alg) {
         case S2N_HASH_MD5:
@@ -1595,7 +1625,9 @@ static S2N_RESULT s2n_signature_scheme_to_tls_iana(const struct s2n_signature_sc
         case S2N_HASH_MD5_SHA1:
             *converted_scheme = S2N_TLS_HASH_MD5_SHA1;
             break;
-        default:
+        case S2N_HASH_NONE:
+        case S2N_HASH_SHAKE256_64:
+        case S2N_HASH_ALGS_COUNT:
             *converted_scheme = S2N_TLS_HASH_NONE;
             break;
     }
@@ -1631,6 +1663,7 @@ static S2N_RESULT s2n_signature_scheme_to_signature_algorithm(const struct s2n_s
 {
     RESULT_ENSURE_REF(sig_scheme);
     RESULT_ENSURE_REF(converted_scheme);
+    *converted_scheme = S2N_TLS_SIGNATURE_ANONYMOUS;
 
     switch (sig_scheme->sig_alg) {
         case S2N_SIGNATURE_RSA:
@@ -1645,7 +1678,10 @@ static S2N_RESULT s2n_signature_scheme_to_signature_algorithm(const struct s2n_s
         case S2N_SIGNATURE_RSA_PSS_PSS:
             *converted_scheme = S2N_TLS_SIGNATURE_RSA_PSS_PSS;
             break;
-        default:
+        case S2N_SIGNATURE_MLDSA:
+            *converted_scheme = S2N_TLS_SIGNATURE_MLDSA;
+            break;
+        case S2N_SIGNATURE_ANONYMOUS:
             *converted_scheme = S2N_TLS_SIGNATURE_ANONYMOUS;
             break;
     }
@@ -1677,6 +1713,34 @@ int s2n_connection_get_selected_client_cert_signature_algorithm(struct s2n_conne
     return S2N_SUCCESS;
 }
 
+int s2n_connection_get_signature_scheme(struct s2n_connection *conn, const char **scheme_name)
+{
+    POSIX_ENSURE_REF(conn);
+    POSIX_ENSURE_REF(scheme_name);
+    POSIX_ENSURE(IS_NEGOTIATED(conn), S2N_ERR_INVALID_STATE);
+
+    const struct s2n_signature_scheme *scheme = conn->handshake_params.server_cert_sig_scheme;
+    /* The scheme should never be NULL. A "none" placeholder is used if no
+     * scheme has been negotiated.
+     */
+    POSIX_ENSURE_REF(scheme);
+
+    *scheme_name = scheme->name;
+    if (scheme->signature_curve) {
+        /* Some TLS1.2 and TLS1.3 signature schemes share an IANA value,
+         * but are NOT the same. The TLS1.3 version implies a specific curve.
+         */
+        if (conn->actual_protocol_version >= S2N_TLS13) {
+            *scheme_name = scheme->tls13_name;
+        } else {
+            *scheme_name = scheme->legacy_name;
+        }
+    }
+
+    POSIX_ENSURE_REF(*scheme_name);
+    return S2N_SUCCESS;
+}
+
 /*
  * Gets the config set on the connection.
  */
