@@ -51,7 +51,7 @@ static int s2n_random_value_is_hello_retry(struct s2n_connection *conn)
 {
     POSIX_ENSURE_REF(conn);
 
-    POSIX_ENSURE(memcmp(hello_retry_req_random, conn->handshake_params.server_random, S2N_TLS_RANDOM_DATA_LEN) == 0,
+    POSIX_ENSURE(s2n_constant_time_equals(hello_retry_req_random, conn->handshake_params.server_random, S2N_TLS_RANDOM_DATA_LEN),
             S2N_ERR_INVALID_HELLO_RETRY);
 
     return S2N_SUCCESS;
@@ -99,8 +99,8 @@ static int s2n_server_hello_parse(struct s2n_connection *conn)
     POSIX_ENSURE_REF(conn->secure);
 
     struct s2n_stuffer *in = &conn->handshake.io;
-    uint8_t compression_method;
-    uint8_t session_id_len;
+    uint8_t compression_method = 0;
+    uint8_t session_id_len = 0;
     uint8_t protocol_version[S2N_TLS_PROTOCOL_VERSION_LEN];
     uint8_t session_id[S2N_TLS_SESSION_ID_MAX_LEN];
 
@@ -110,14 +110,14 @@ static int s2n_server_hello_parse(struct s2n_connection *conn)
     uint8_t legacy_version = (uint8_t) (protocol_version[0] * 10) + protocol_version[1];
 
     /**
-     *= https://tools.ietf.org/rfc/rfc8446#4.1.3
+     *= https://www.rfc-editor.org/rfc/rfc8446#4.1.3
      *# Upon receiving a message with type server_hello, implementations MUST
      *# first examine the Random value and, if it matches this value, process
      *# it as described in Section 4.1.4).
      **/
     if (s2n_random_value_is_hello_retry(conn) == S2N_SUCCESS) {
         /**
-         *= https://tools.ietf.org/rfc/rfc8446#4.1.4
+         *= https://www.rfc-editor.org/rfc/rfc8446#4.1.4
          *# If a client receives a second
          *# HelloRetryRequest in the same connection (i.e., where the ClientHello
          *# was itself in response to a HelloRetryRequest), it MUST abort the
@@ -126,7 +126,7 @@ static int s2n_server_hello_parse(struct s2n_connection *conn)
         POSIX_ENSURE(!s2n_is_hello_retry_handshake(conn), S2N_ERR_INVALID_HELLO_RETRY);
 
         /**
-         *= https://tools.ietf.org/rfc/rfc8446#4.1.4
+         *= https://www.rfc-editor.org/rfc/rfc8446#4.1.4
          *# Upon receipt of a HelloRetryRequest, the client MUST check the
          *# legacy_version
          **/
@@ -145,11 +145,11 @@ static int s2n_server_hello_parse(struct s2n_connection *conn)
     POSIX_GUARD(s2n_stuffer_read_uint8(in, &compression_method));
 
     /**
-     *= https://tools.ietf.org/rfc/rfc8446#4.1.3
+     *= https://www.rfc-editor.org/rfc/rfc8446#4.1.3
      *# legacy_compression_method:  A single byte which MUST have the
      *# value 0.
      *
-     *= https://tools.ietf.org/rfc/rfc8446#4.1.4
+     *= https://www.rfc-editor.org/rfc/rfc8446#4.1.4
      *# Upon receipt of a HelloRetryRequest, the client MUST check the
      *# legacy_version, legacy_session_id_echo, cipher_suite, and
      *# legacy_compression_method
@@ -157,7 +157,7 @@ static int s2n_server_hello_parse(struct s2n_connection *conn)
     S2N_ERROR_IF(compression_method != S2N_TLS_COMPRESSION_METHOD_NULL, S2N_ERR_BAD_MESSAGE);
 
     bool session_ids_match = session_id_len != 0 && session_id_len == conn->session_id_len
-            && memcmp(session_id, conn->session_id, session_id_len) == 0;
+            && s2n_constant_time_equals(session_id, conn->session_id, session_id_len);
     if (!session_ids_match) {
         conn->ems_negotiated = false;
     }
@@ -165,7 +165,7 @@ static int s2n_server_hello_parse(struct s2n_connection *conn)
     POSIX_GUARD(s2n_server_extensions_recv(conn, in));
 
     /**
-    *= https://tools.ietf.org/rfc/rfc8446#4.1.4
+    *= https://www.rfc-editor.org/rfc/rfc8446#4.1.4
     *# The server's extensions MUST contain "supported_versions".
     **/
     if (s2n_is_hello_retry_message(conn)) {
@@ -185,7 +185,7 @@ static int s2n_server_hello_parse(struct s2n_connection *conn)
          *# it sent in the ClientHello MUST abort the handshake with an
          *# "illegal_parameter" alert.
          *
-         *= https://tools.ietf.org/rfc/rfc8446#4.1.4
+         *= https://www.rfc-editor.org/rfc/rfc8446#4.1.4
          *# Upon receipt of a HelloRetryRequest, the client MUST check the
          *# legacy_version, legacy_session_id_echo
          **/
@@ -206,13 +206,13 @@ static int s2n_server_hello_parse(struct s2n_connection *conn)
         POSIX_ENSURE(!s2n_is_hello_retry_handshake(conn), S2N_ERR_BAD_MESSAGE);
 
         /*
-         *= https://tools.ietf.org/rfc/rfc8446#appendix-D.3
+         *= https://www.rfc-editor.org/rfc/rfc8446#appendix-D.3
          *# A client that attempts to send 0-RTT data MUST fail a connection if
          *# it receives a ServerHello with TLS 1.2 or older.
          */
         POSIX_ENSURE(conn->early_data_state != S2N_EARLY_DATA_REQUESTED, S2N_ERR_PROTOCOL_VERSION_UNSUPPORTED);
 
-        const struct s2n_security_policy *security_policy;
+        const struct s2n_security_policy *security_policy = NULL;
         POSIX_GUARD(s2n_connection_get_security_policy(conn, &security_policy));
 
         if (conn->server_protocol_version < security_policy->minimum_protocol_version
@@ -224,7 +224,7 @@ static int s2n_server_hello_parse(struct s2n_connection *conn)
         conn->actual_protocol_version = MIN(conn->server_protocol_version, conn->client_protocol_version);
 
         /*
-         *= https://tools.ietf.org/rfc/rfc5077#section-3.4
+         *= https://www.rfc-editor.org/rfc/rfc5077#section-3.4
          *# If the server accepts the ticket
          *# and the Session ID is not empty, then it MUST respond with the same
          *# Session ID present in the ClientHello.  This allows the client to
@@ -234,7 +234,7 @@ static int s2n_server_hello_parse(struct s2n_connection *conn)
         if (session_ids_match) {
             /* check if the resumed session state is valid */
             POSIX_ENSURE(conn->resume_protocol_version == conn->actual_protocol_version, S2N_ERR_BAD_MESSAGE);
-            POSIX_ENSURE(memcmp(conn->secure->cipher_suite->iana_value, cipher_suite_wire, S2N_TLS_CIPHER_SUITE_LEN) == 0,
+            POSIX_ENSURE(s2n_constant_time_equals(conn->secure->cipher_suite->iana_value, cipher_suite_wire, S2N_TLS_CIPHER_SUITE_LEN),
                     S2N_ERR_BAD_MESSAGE);
 
             /* Session is resumed */
