@@ -15,6 +15,7 @@
 
 #pragma once
 
+#include "tls/s2n_tls13_secrets.h"
 #include "utils/s2n_result.h"
 
 struct s2n_key_material;
@@ -23,3 +24,4 @@ S2N_RESULT s2n_tls13_key_schedule_generate_key_material(struct s2n_connection *c
 
 S2N_RESULT s2n_tls13_key_schedule_update(struct s2n_connection *conn);
 S2N_RESULT s2n_tls13_key_schedule_reset(struct s2n_connection *conn);
+S2N_RESULT s2n_tls13_key_schedule_set_key(struct s2n_connection *conn, s2n_extract_secret_type_t secret_type, s2n_mode mode);
