@@ -70,6 +70,8 @@
  */
 #define TLS_PQ_KEM_GROUP_ID_SECP256R1_MLKEM_768     0x11EB
 #define TLS_PQ_KEM_GROUP_ID_X25519_MLKEM_768        0x11EC
+#define TLS_PQ_KEM_GROUP_ID_SECP384R1_MLKEM_1024    0x11ED
+#define TLS_PQ_KEM_GROUP_ID_MLKEM_1024              0x0202
 #define TLS_PQ_KEM_GROUP_ID_X25519_KYBER_512_R3     0x2F39
 #define TLS_PQ_KEM_GROUP_ID_SECP256R1_KYBER_512_R3  0x2F3A
 #define TLS_PQ_KEM_GROUP_ID_SECP384R1_KYBER_768_R3  0x2F3C
@@ -158,6 +160,11 @@
 #define TLS_SIGNATURE_SCHEME_RSA_PSS_PSS_SHA384  0x080A
 #define TLS_SIGNATURE_SCHEME_RSA_PSS_PSS_SHA512  0x080B
 
+/* ML-DSA: post-quantum signature schemes */
+#define TLS_SIGNATURE_SCHEME_MLDSA44 0x0904
+#define TLS_SIGNATURE_SCHEME_MLDSA65 0x0905
+#define TLS_SIGNATURE_SCHEME_MLDSA87 0x0906
+
 #define TLS_SIGNATURE_SCHEME_LEN          2
 #define TLS_SIGNATURE_SCHEME_LIST_MAX_LEN 128
 
