@@ -276,6 +276,17 @@ static S2N_RESULT s2n_ensure_uniqueness(void)
     return S2N_RESULT_OK;
 }
 
+static S2N_RESULT s2n_get_libcrypto_private_random_data(struct s2n_blob *out_blob)
+{
+    RESULT_GUARD_PTR(out_blob);
+#if S2N_LIBCRYPTO_SUPPORTS_PRIVATE_RAND
+    RESULT_GUARD_OSSL(RAND_priv_bytes(out_blob->data, out_blob->size), S2N_ERR_DRBG);
+#else
+    RESULT_GUARD_OSSL(RAND_bytes(out_blob->data, out_blob->size), S2N_ERR_DRBG);
+#endif
+    return S2N_RESULT_OK;
+}
+
 static S2N_RESULT s2n_get_libcrypto_random_data(struct s2n_blob *out_blob)
 {
     RESULT_GUARD_PTR(out_blob);
@@ -308,33 +319,23 @@ static S2N_RESULT s2n_get_custom_random_data(struct s2n_blob *out_blob, struct s
     return S2N_RESULT_OK;
 }
 
-static S2N_RESULT s2n_get_random_data(struct s2n_blob *blob, struct s2n_drbg *drbg_state)
-{
-    /* By default, s2n-tls uses a custom random implementation to generate random data for the TLS
-     * handshake. When operating in FIPS mode, the FIPS-validated libcrypto implementation is used
-     * instead.
-     */
-    if (s2n_is_in_fips_mode()) {
-        RESULT_GUARD(s2n_get_libcrypto_random_data(blob));
-        return S2N_RESULT_OK;
-    }
-
-    RESULT_GUARD(s2n_get_custom_random_data(blob, drbg_state));
-
-    return S2N_RESULT_OK;
-}
-
 S2N_RESULT s2n_get_public_random_data(struct s2n_blob *blob)
 {
-    RESULT_GUARD(s2n_get_random_data(blob, &s2n_per_thread_rand_state.public_drbg));
-
+    if (s2n_is_in_fips_mode()) {
+        RESULT_GUARD(s2n_get_libcrypto_random_data(blob));
+    } else {
+        RESULT_GUARD(s2n_get_custom_random_data(blob, &s2n_per_thread_rand_state.public_drbg));
+    }
     return S2N_RESULT_OK;
 }
 
 S2N_RESULT s2n_get_private_random_data(struct s2n_blob *blob)
 {
-    RESULT_GUARD(s2n_get_random_data(blob, &s2n_per_thread_rand_state.private_drbg));
-
+    if (s2n_is_in_fips_mode()) {
+        RESULT_GUARD(s2n_get_libcrypto_private_random_data(blob));
+    } else {
+        RESULT_GUARD(s2n_get_custom_random_data(blob, &s2n_per_thread_rand_state.private_drbg));
+    }
     return S2N_RESULT_OK;
 }
 
