@@ -3,5 +3,5 @@ import utils
 
 utils.chdir_project_root()
 
-utils.run('python3 scripts/format-c.py')
+utils.run('python3 format-check.py -i')
 utils.run('python3 scripts/format-python.py')
