@@ -12,6 +12,8 @@ extern const char *s_capsule_name_hash;
 extern const char *s_capsule_name_hmac;
 /** Name string for rsa capsule. */
 extern const char *s_capsule_name_rsa;
+/** Name string for rsa capsule. */
+extern const char *s_capsule_name_ed25519;
 
 PyObject *aws_py_sha1_new(PyObject *self, PyObject *args);
 PyObject *aws_py_sha256_new(PyObject *self, PyObject *args);
@@ -40,4 +42,17 @@ PyObject *aws_py_rsa_decrypt(PyObject *self, PyObject *args);
 PyObject *aws_py_rsa_sign(PyObject *self, PyObject *args);
 PyObject *aws_py_rsa_verify(PyObject *self, PyObject *args);
 
+PyObject *aws_py_ed25519_new_generate(PyObject *self, PyObject *args);
+PyObject *aws_py_ed25519_export_public_key(PyObject *self, PyObject *args);
+PyObject *aws_py_ed25519_export_private_key(PyObject *self, PyObject *args);
+
+PyObject *aws_py_ec_new_generate(PyObject *self, PyObject *args);
+PyObject *aws_py_ec_key_from_der_data(PyObject *self, PyObject *args);
+PyObject *aws_py_ec_export_key(PyObject *self, PyObject *args);
+PyObject *aws_py_ec_sign(PyObject *self, PyObject *args);
+PyObject *aws_py_ec_verify(PyObject *self, PyObject *args);
+PyObject *aws_py_ec_encode_signature(PyObject *self, PyObject *args);
+PyObject *aws_py_ec_decode_signature(PyObject *self, PyObject *args);
+PyObject *aws_py_ec_get_public_coords(PyObject *self, PyObject *args);
+
 #endif /* AWS_CRT_PYTHON_CRYPTO_H */
