@@ -5,6 +5,7 @@
 #include "module.h"
 
 #include "auth.h"
+#include "cbor.h"
 #include "checksums.h"
 #include "common.h"
 #include "crypto.h"
@@ -14,6 +15,7 @@
 #include "mqtt5_client.h"
 #include "mqtt_client.h"
 #include "mqtt_client_connection.h"
+#include "mqtt_request_response.h"
 #include "s3.h"
 #include "websocket.h"
 
@@ -37,13 +39,6 @@ static bool s_logger_init = false;
 PyObject *aws_py_init_logging(PyObject *self, PyObject *args) {
     (void)self;
 
-    if (s_logger_init) {
-        aws_logger_set(NULL);
-        aws_logger_clean_up(&s_logger);
-    }
-
-    s_logger_init = true;
-
     /* NOTE: We are NOT using aws_py_get_allocator() for logging.
      * This avoid deadlock during aws_mem_tracer_dump() */
     struct aws_allocator *allocator = aws_default_allocator();
@@ -75,8 +70,39 @@ PyObject *aws_py_init_logging(PyObject *self, PyObject *args) {
         log_options.filename = file_path;
     }
 
-    aws_logger_init_standard(&s_logger, allocator, &log_options);
-    aws_logger_set(&s_logger);
+    /* It is not safe to clean up a running logger */
+    if (s_logger_init) {
+        aws_raise_error(AWS_ERROR_INVALID_STATE);
+        return PyErr_AwsLastError();
+    }
+
+    if (aws_logger_init_standard(&s_logger, allocator, &log_options) == AWS_OP_SUCCESS) {
+        aws_logger_set(&s_logger);
+        s_logger_init = true;
+    } else {
+        return PyErr_AwsLastError();
+    }
+
+    Py_RETURN_NONE;
+}
+
+PyObject *aws_py_set_log_level(PyObject *self, PyObject *args) {
+    (void)self;
+
+    if (!s_logger_init) {
+        aws_raise_error(AWS_ERROR_INVALID_STATE);
+        return PyErr_AwsLastError();
+    }
+
+    int log_level = 0;
+    if (!PyArg_ParseTuple(args, "b", &log_level)) {
+        PyErr_SetNone(PyExc_ValueError);
+        return NULL;
+    }
+
+    if (aws_logger_set_log_level(&s_logger, log_level) != AWS_OP_SUCCESS) {
+        return PyErr_AwsLastError();
+    }
 
     Py_RETURN_NONE;
 }
@@ -116,17 +142,32 @@ PyObject *PyUnicode_FromAwsString(const struct aws_string *aws_str) {
     return PyUnicode_FromStringAndSize(aws_string_c_str(aws_str), aws_str->len);
 }
 
+PyObject *PyBytes_FromAwsByteCursor(const struct aws_byte_cursor *cursor) {
+    if (cursor->len > PY_SSIZE_T_MAX) {
+        PyErr_SetString(PyExc_OverflowError, "Cursor exceeds PY_SSIZE_T_MAX");
+        return NULL;
+    }
+    return PyBytes_FromStringAndSize((const char *)cursor->ptr, (Py_ssize_t)cursor->len);
+}
+
 uint32_t PyObject_GetAttrAsUint32(PyObject *o, const char *class_name, const char *attr_name) {
     uint32_t result = UINT32_MAX;
 
-    PyObject *attr = PyObject_GetAttrString(o, attr_name);
+    PyObject *attr = PyObject_GetAttrString(o, attr_name); /* new reference */
     if (!attr) {
         PyErr_Format(PyExc_AttributeError, "'%s.%s' attribute not found", class_name, attr_name);
         return result;
     }
 
+    if (attr == Py_None) {
+        PyErr_Format(PyExc_AttributeError, "'%s.%s' required integral attribute is None", class_name, attr_name);
+        goto done;
+    }
+
     PyObject_GetAsOptionalUint32(attr, class_name, attr_name, &result);
 
+done:
+
     Py_DECREF(attr);
     return result;
 }
@@ -134,14 +175,21 @@ uint32_t PyObject_GetAttrAsUint32(PyObject *o, const char *class_name, const cha
 uint16_t PyObject_GetAttrAsUint16(PyObject *o, const char *class_name, const char *attr_name) {
     uint16_t result = UINT16_MAX;
 
-    PyObject *attr = PyObject_GetAttrString(o, attr_name);
+    PyObject *attr = PyObject_GetAttrString(o, attr_name); /* new reference */
     if (!attr) {
         PyErr_Format(PyExc_AttributeError, "'%s.%s' attribute not found", class_name, attr_name);
         return result;
     }
 
+    if (attr == Py_None) {
+        PyErr_Format(PyExc_AttributeError, "'%s.%s' required integral attribute is None", class_name, attr_name);
+        goto done;
+    }
+
     PyObject_GetAsOptionalUint16(attr, class_name, attr_name, &result);
 
+done:
+
     Py_DECREF(attr);
     return result;
 }
@@ -149,14 +197,21 @@ uint16_t PyObject_GetAttrAsUint16(PyObject *o, const char *class_name, const cha
 uint8_t PyObject_GetAttrAsUint8(PyObject *o, const char *class_name, const char *attr_name) {
     uint8_t result = UINT8_MAX;
 
-    PyObject *attr = PyObject_GetAttrString(o, attr_name);
+    PyObject *attr = PyObject_GetAttrString(o, attr_name); /* new reference */
     if (!attr) {
         PyErr_Format(PyExc_AttributeError, "'%s.%s' attribute not found", class_name, attr_name);
         return result;
     }
 
+    if (attr == Py_None) {
+        PyErr_Format(PyExc_AttributeError, "'%s.%s' required integral attribute is None", class_name, attr_name);
+        goto done;
+    }
+
     PyObject_GetAsOptionalUint8(attr, class_name, attr_name, &result);
 
+done:
+
     Py_DECREF(attr);
     return result;
 }
@@ -164,12 +219,17 @@ uint8_t PyObject_GetAttrAsUint8(PyObject *o, const char *class_name, const char
 bool PyObject_GetAttrAsBool(PyObject *o, const char *class_name, const char *attr_name) {
     bool result = false;
 
-    PyObject *attr = PyObject_GetAttrString(o, attr_name);
+    PyObject *attr = PyObject_GetAttrString(o, attr_name); /* new reference */
     if (!attr) {
         PyErr_Format(PyExc_AttributeError, "'%s.%s' attribute not found", class_name, attr_name);
         return result;
     }
 
+    if (attr == Py_None) {
+        PyErr_Format(PyExc_AttributeError, "'%s.%s' required boolean attribute is None", class_name, attr_name);
+        goto done;
+    }
+
     int val = PyObject_IsTrue(attr);
     if (val == -1) {
         PyErr_Format(PyExc_TypeError, "Cannot convert %s.%s to bool", class_name, attr_name);
@@ -185,14 +245,22 @@ done:
 int PyObject_GetAttrAsIntEnum(PyObject *o, const char *class_name, const char *attr_name) {
     int result = -1;
 
-    PyObject *attr = PyObject_GetAttrString(o, attr_name);
+    PyObject *attr = PyObject_GetAttrString(o, attr_name); /* new reference */
     if (!attr) {
         PyErr_Format(PyExc_AttributeError, "'%s.%s' attribute not found", class_name, attr_name);
         return result;
     }
 
+    if (attr == Py_None) {
+        PyErr_Format(
+            PyExc_AttributeError, "'%s.%s' required integral enumeration attribute is None", class_name, attr_name);
+        goto done;
+    }
+
     PyObject_GetAsOptionalIntEnum(attr, class_name, attr_name, &result);
 
+done:
+
     Py_DECREF(attr);
     return result;
 }
@@ -452,7 +520,8 @@ int aws_py_translate_py_error(void) {
     }
 
     /* Print standard traceback to sys.stderr and clear the error indicator. */
-    PyErr_Print();
+    /* Handles the exception in C, do not set the last vars for python. */
+    PyErr_PrintEx(0 /*set_sys_last_vars*/);
     fprintf(stderr, "Treating Python exception as error %d(%s)\n", aws_error_code, aws_error_name(aws_error_code));
 
     return aws_error_code;
@@ -713,6 +782,7 @@ static PyMethodDef s_module_methods[] = {
     AWS_PY_METHOD_DEF(tls_connection_options_set_alpn_list, METH_VARARGS),
     AWS_PY_METHOD_DEF(tls_connection_options_set_server_name, METH_VARARGS),
     AWS_PY_METHOD_DEF(init_logging, METH_VARARGS),
+    AWS_PY_METHOD_DEF(set_log_level, METH_VARARGS),
     AWS_PY_METHOD_DEF(input_stream_new, METH_VARARGS),
     AWS_PY_METHOD_DEF(pkcs11_lib_new, METH_VARARGS),
 
@@ -742,6 +812,13 @@ static PyMethodDef s_module_methods[] = {
     AWS_PY_METHOD_DEF(mqtt5_client_get_stats, METH_VARARGS),
     AWS_PY_METHOD_DEF(mqtt5_ws_handshake_transform_complete, METH_VARARGS),
 
+    /* MQTT Request Response Client */
+    AWS_PY_METHOD_DEF(mqtt_request_response_client_new_from_5, METH_VARARGS),
+    AWS_PY_METHOD_DEF(mqtt_request_response_client_new_from_311, METH_VARARGS),
+    AWS_PY_METHOD_DEF(mqtt_request_response_client_make_request, METH_VARARGS),
+    AWS_PY_METHOD_DEF(mqtt_request_response_client_create_stream, METH_VARARGS),
+    AWS_PY_METHOD_DEF(mqtt_streaming_operation_open, METH_VARARGS),
+
     /* Cryptographic primitives */
     AWS_PY_METHOD_DEF(md5_new, METH_NOARGS),
     AWS_PY_METHOD_DEF(sha256_new, METH_NOARGS),
@@ -762,6 +839,21 @@ static PyMethodDef s_module_methods[] = {
     AWS_PY_METHOD_DEF(rsa_sign, METH_VARARGS),
     AWS_PY_METHOD_DEF(rsa_verify, METH_VARARGS),
 
+    /* ED25519 crypto primitives */
+    AWS_PY_METHOD_DEF(ed25519_new_generate, METH_NOARGS),
+    AWS_PY_METHOD_DEF(ed25519_export_public_key, METH_VARARGS),
+    AWS_PY_METHOD_DEF(ed25519_export_private_key, METH_VARARGS),
+
+    /* EC crypto primitives */
+    AWS_PY_METHOD_DEF(ec_new_generate, METH_VARARGS),
+    AWS_PY_METHOD_DEF(ec_key_from_der_data, METH_VARARGS),
+    AWS_PY_METHOD_DEF(ec_export_key, METH_VARARGS),
+    AWS_PY_METHOD_DEF(ec_sign, METH_VARARGS),
+    AWS_PY_METHOD_DEF(ec_verify, METH_VARARGS),
+    AWS_PY_METHOD_DEF(ec_encode_signature, METH_VARARGS),
+    AWS_PY_METHOD_DEF(ec_decode_signature, METH_VARARGS),
+    AWS_PY_METHOD_DEF(ec_get_public_coords, METH_VARARGS),
+
     /* Checksum primitives */
     AWS_PY_METHOD_DEF(checksums_crc32, METH_VARARGS),
     AWS_PY_METHOD_DEF(checksums_crc32c, METH_VARARGS),
@@ -773,6 +865,7 @@ static PyMethodDef s_module_methods[] = {
     AWS_PY_METHOD_DEF(http_client_connection_new, METH_VARARGS),
     AWS_PY_METHOD_DEF(http_client_stream_new, METH_VARARGS),
     AWS_PY_METHOD_DEF(http_client_stream_activate, METH_VARARGS),
+    AWS_PY_METHOD_DEF(http2_client_stream_write_data, METH_VARARGS),
     AWS_PY_METHOD_DEF(http_message_new_request, METH_VARARGS),
     AWS_PY_METHOD_DEF(http_message_get_request_method, METH_VARARGS),
     AWS_PY_METHOD_DEF(http_message_set_request_method, METH_VARARGS),
@@ -850,6 +943,41 @@ static PyMethodDef s_module_methods[] = {
     AWS_PY_METHOD_DEF(websocket_increment_read_window, METH_VARARGS),
     AWS_PY_METHOD_DEF(websocket_create_handshake_request, METH_VARARGS),
 
+    /* CBOR Encode */
+    AWS_PY_METHOD_DEF(cbor_encoder_new, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_encoder_get_encoded_data, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_encoder_write_uint, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_encoder_write_negint, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_encoder_write_float, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_encoder_write_bytes, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_encoder_write_text, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_encoder_write_array_start, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_encoder_write_map_start, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_encoder_write_tag, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_encoder_write_bool, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_encoder_write_simple_types, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_encoder_write_py_list, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_encoder_write_py_dict, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_encoder_write_data_item, METH_VARARGS),
+
+    /* CBOR Decode */
+    AWS_PY_METHOD_DEF(cbor_decoder_new, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_decoder_peek_type, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_decoder_get_remaining_bytes_len, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_decoder_consume_next_element, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_decoder_consume_next_data_item, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_decoder_pop_next_unsigned_int, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_decoder_pop_next_negative_int, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_decoder_pop_next_float, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_decoder_pop_next_boolean, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_decoder_pop_next_bytes, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_decoder_pop_next_text, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_decoder_pop_next_array_start, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_decoder_pop_next_map_start, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_decoder_pop_next_tag, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_decoder_pop_next_py_list, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_decoder_pop_next_py_dict, METH_VARARGS),
+    AWS_PY_METHOD_DEF(cbor_decoder_pop_next_data_item, METH_VARARGS),
     {NULL, NULL, 0, NULL},
 };
 
