@@ -10,9 +10,8 @@ os.environ['AWS_CRT_CRASH_HANDLER'] = '1'   # noqa
 
 from awscrt import NativeResource
 from awscrt._test import check_for_leaks
-from awscrt.io import init_logging, LogLevel
+import time
 import unittest
-import sys
 
 TIMEOUT = 30.0
 
@@ -57,3 +56,23 @@ class NativeResourceTest(unittest.TestCase):
         except Exception:
             NativeResourceTest._previous_test_failed = True
             raise
+
+
+MAX_RETRIES = 5
+
+
+def _is_retryable_exception(e):
+    exception_text = str(e)
+    return "AWS_IO_TLS_NEGOTIATION_TIMEOUT" in exception_text or "AWS_IO_SOCKET_TIMEOUT" in exception_text
+
+
+def test_retry_wrapper(test_function):
+    for i in range(MAX_RETRIES):
+        try:
+            test_function()
+            return
+        except Exception as e:
+            if _is_retryable_exception(e) and i + 1 < MAX_RETRIES:
+                time.sleep(1)
+            else:
+                raise
