@@ -3,8 +3,7 @@
 
 from concurrent.futures import Future
 from awscrt import mqtt5, io, http, exceptions
-from test import NativeResourceTest
-from threading import Lock
+from test import test_retry_wrapper, NativeResourceTest
 import os
 import unittest
 import uuid
@@ -105,6 +104,9 @@ class Mqtt5TestCallbacks():
             self.future_disconnection.set_result(lifecycle_disconnect_data)
 
 
+MAX_RETRIES = 5
+
+
 class Mqtt5ClientTest(NativeResourceTest):
 
     def _create_client(
@@ -195,7 +197,7 @@ class Mqtt5ClientTest(NativeResourceTest):
     #             DIRECT CONNECT TEST CASES
     # ==============================================================
 
-    def test_direct_connect_minimum(self):
+    def _test_direct_connect_minimum(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_DIRECT_MQTT_HOST")
         input_port = int(_get_env_variable("AWS_TEST_MQTT5_DIRECT_MQTT_PORT"))
 
@@ -210,7 +212,10 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
-    def test_direct_connect_basic_auth(self):
+    def test_direct_connect_minimum(self):
+        test_retry_wrapper(self._test_direct_connect_minimum)
+
+    def _test_direct_connect_basic_auth(self):
         input_username = _get_env_variable("AWS_TEST_MQTT5_BASIC_AUTH_USERNAME")
         input_password = _get_env_variable("AWS_TEST_MQTT5_BASIC_AUTH_PASSWORD")
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_DIRECT_MQTT_BASIC_AUTH_HOST")
@@ -233,7 +238,10 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
-    def test_direct_connect_tls(self):
+    def test_direct_connect_basic_auth(self):
+        test_retry_wrapper(self._test_direct_connect_basic_auth)
+
+    def _test_direct_connect_tls(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_DIRECT_MQTT_TLS_HOST")
         input_port = int(_get_env_variable("AWS_TEST_MQTT5_DIRECT_MQTT_TLS_PORT"))
 
@@ -252,7 +260,10 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
-    def test_direct_connect_mutual_tls(self):
+    def test_direct_connect_tls(self):
+        test_retry_wrapper(self._test_direct_connect_tls)
+
+    def _test_direct_connect_mutual_tls(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_HOST")
         input_cert = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_CERT")
         input_key = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_KEY")
@@ -274,7 +285,10 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
-    def test_direct_connect_http_proxy_tls(self):
+    def test_direct_connect_mutual_tls(self):
+        test_retry_wrapper(self._test_direct_connect_mutual_tls)
+
+    def _test_direct_connect_http_proxy_tls(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_DIRECT_MQTT_TLS_HOST")
         input_port = int(_get_env_variable("AWS_TEST_MQTT5_DIRECT_MQTT_TLS_PORT"))
         input_proxy_host = _get_env_variable("AWS_TEST_MQTT5_PROXY_HOST")
@@ -303,7 +317,10 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
-    def test_direct_connect_maximum(self):
+    def test_direct_connect_http_proxy_tls(self):
+        test_retry_wrapper(self._test_direct_connect_http_proxy_tls)
+
+    def _test_direct_connect_maximum(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_DIRECT_MQTT_HOST")
         input_port = int(_get_env_variable("AWS_TEST_MQTT5_DIRECT_MQTT_PORT"))
 
@@ -358,11 +375,14 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
+    def test_direct_connect_maximum(self):
+        test_retry_wrapper(self._test_direct_connect_maximum)
+
     # ==============================================================
     #             WEBSOCKET CONNECT TEST CASES
     # ==============================================================
 
-    def test_websocket_connect_minimum(self):
+    def _test_websocket_connect_minimum(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_WS_MQTT_HOST")
         input_port = int(_get_env_variable("AWS_TEST_MQTT5_WS_MQTT_PORT"))
 
@@ -379,7 +399,10 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
-    def test_websocket_connect_basic_auth(self):
+    def test_websocket_connect_minimum(self):
+        test_retry_wrapper(self._test_websocket_connect_minimum)
+
+    def _test_websocket_connect_basic_auth(self):
         input_username = _get_env_variable("AWS_TEST_MQTT5_BASIC_AUTH_USERNAME")
         input_password = _get_env_variable("AWS_TEST_MQTT5_BASIC_AUTH_PASSWORD")
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_WS_MQTT_BASIC_AUTH_HOST")
@@ -404,7 +427,10 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
-    def test_websocket_connect_tls(self):
+    def test_websocket_connect_basic_auth(self):
+        test_retry_wrapper(self._test_websocket_connect_basic_auth)
+
+    def _test_websocket_connect_tls(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_WS_MQTT_TLS_HOST")
         input_port = int(_get_env_variable("AWS_TEST_MQTT5_WS_MQTT_TLS_PORT"))
 
@@ -424,9 +450,12 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
-    # TODO test_websocket_connect_sigv4 against IoT Core
+    def test_websocket_connect_tls(self):
+        test_retry_wrapper(self._test_websocket_connect_tls)
 
-    def test_websocket_connect_http_proxy_tls(self):
+    #  test_websocket_connect_sigv4 against IoT Core : tested in the SDK
+
+    def _test_websocket_connect_http_proxy_tls(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_WS_MQTT_TLS_HOST")
         input_port = int(_get_env_variable("AWS_TEST_MQTT5_WS_MQTT_TLS_PORT"))
         input_proxy_host = _get_env_variable("AWS_TEST_MQTT5_PROXY_HOST")
@@ -457,7 +486,10 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
-    def test_websocket_connect_maximum(self):
+    def test_websocket_connect_http_proxy_tls(self):
+        test_retry_wrapper(self._test_websocket_connect_http_proxy_tls)
+
+    def _test_websocket_connect_maximum(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_WS_MQTT_HOST")
         input_port = int(_get_env_variable("AWS_TEST_MQTT5_WS_MQTT_PORT"))
 
@@ -512,6 +544,9 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
+    def test_websocket_connect_maximum(self):
+        test_retry_wrapper(self._test_websocket_connect_maximum)
+
     # ==============================================================
     #             NEGATIVE CONNECT TEST CASES
     # ==============================================================
@@ -590,7 +625,7 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
-    # TODO test_websocket_handshake_failure
+    # test_websocket_handshake_failure : tested in the SDK
 
     def test_double_client_id_failure(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_DIRECT_MQTT_HOST")
@@ -612,6 +647,8 @@ class Mqtt5ClientTest(NativeResourceTest):
         client1.start()
         callbacks.future_connection_success.result(TIMEOUT)
 
+        time.sleep(5)
+
         client2.start()
 
         callbacks.future_disconnection.result(TIMEOUT)
@@ -760,7 +797,7 @@ class Mqtt5ClientTest(NativeResourceTest):
         with self.assertRaises(OverflowError) as cm:
             self._create_client(client_options=client_options)
 
-    def test_negative_disconnect_packet_properties(self):
+    def _test_negative_disconnect_packet_properties(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_HOST")
         input_cert = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_CERT")
         input_key = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_KEY")
@@ -787,7 +824,10 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
-    def test_negative_publish_packet_properties(self):
+    def test_negative_disconnect_packet_properties(self):
+        test_retry_wrapper(self._test_negative_disconnect_packet_properties)
+
+    def _test_negative_publish_packet_properties(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_HOST")
         input_cert = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_CERT")
         input_key = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_KEY")
@@ -813,7 +853,10 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
-    def test_negative_subscribe_packet_properties(self):
+    def test_negative_publish_packet_properties(self):
+        test_retry_wrapper(self._test_negative_publish_packet_properties)
+
+    def _test_negative_subscribe_packet_properties(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_HOST")
         input_cert = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_CERT")
         input_key = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_KEY")
@@ -842,11 +885,14 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
+    def test_negative_subscribe_packet_properties(self):
+        test_retry_wrapper(self._test_negative_subscribe_packet_properties)
+
     # ==============================================================
     #             NEGOTIATED SETTINGS TEST CASES
     # ==============================================================
 
-    def test_negotiated_settings_minimal_settings(self):
+    def _test_negotiated_settings_minimal_settings(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_DIRECT_MQTT_HOST")
         input_port = int(_get_env_variable("AWS_TEST_MQTT5_DIRECT_MQTT_PORT"))
 
@@ -869,7 +915,10 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
-    def test_negotiated_settings_maximum_settings(self):
+    def test_negotiated_settings_minimal_settings(self):
+        test_retry_wrapper(self._test_negotiated_settings_minimal_settings)
+
+    def _test_negotiated_settings_maximum_settings(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_DIRECT_MQTT_HOST")
         input_port = int(_get_env_variable("AWS_TEST_MQTT5_DIRECT_MQTT_PORT"))
 
@@ -910,7 +959,10 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
-    def test_negotiated_settings_server_limit(self):
+    def test_negotiated_settings_maximum_settings(self):
+        test_retry_wrapper(self._test_negotiated_settings_maximum_settings)
+
+    def _test_negotiated_settings_server_limit(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_DIRECT_MQTT_HOST")
         input_port = int(_get_env_variable("AWS_TEST_MQTT5_DIRECT_MQTT_PORT"))
 
@@ -943,11 +995,14 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
+    def test_negotiated_settings_server_limit(self):
+        test_retry_wrapper(self._test_negotiated_settings_server_limit)
+
     # ==============================================================
     #             OPERATION TEST CASES
     # ==============================================================
 
-    def test_operation_sub_unsub(self):
+    def _test_operation_sub_unsub(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_HOST")
         input_cert = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_CERT")
         input_key = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_KEY")
@@ -1009,162 +1064,10 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
-    total_callbacks = 0
-    all_packets_received = Future()
-    mutex = Lock()
-    received_subscriptions = [0] * 10
+    def test_operation_sub_unsub(self):
+        test_retry_wrapper(self._test_operation_sub_unsub)
 
-    def subscriber1_callback(self, publish_received_data: mqtt5.PublishReceivedData):
-        self.mutex.acquire()
-        var = publish_received_data.publish_packet.payload
-        self.received_subscriptions[int(var)] = 1
-        self.total_callbacks = self.total_callbacks + 1
-        if self.total_callbacks == 10:
-            self.all_packets_received.set_result(None)
-        self.mutex.release()
-
-    def subscriber2_callback(self, publish_received_data: mqtt5.PublishReceivedData):
-        self.mutex.acquire()
-        var = publish_received_data.publish_packet.payload
-        self.received_subscriptions[int(var)] = 1
-        self.total_callbacks = self.total_callbacks + 1
-        if self.total_callbacks == 10:
-            self.all_packets_received.set_result(None)
-        self.mutex.release()
-
-    def test_operation_shared_subscription(self):
-        input_host_name = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_HOST")
-        input_cert = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_CERT")
-        input_key = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_KEY")
-
-        client_id_subscriber1 = create_client_id()
-        client_id_subscriber2 = create_client_id()
-        client_id_publisher = create_client_id()
-
-        testTopic = "test/MQTT5_Binding_Python_" + client_id_publisher
-        sharedTopicfilter = "$share/crttest/test/MQTT5_Binding_Python_" + client_id_publisher
-
-        tls_ctx_options = io.TlsContextOptions.create_client_with_mtls_from_path(
-            input_cert,
-            input_key
-        )
-
-        # subscriber 1
-        connect_subscriber1_options = mqtt5.ConnectPacket(client_id=client_id_subscriber1)
-        subscriber1_generic_callback = Mqtt5TestCallbacks()
-        subscriber1_options = mqtt5.ClientOptions(
-            host_name=input_host_name,
-            port=8883,
-            tls_ctx=io.ClientTlsContext(tls_ctx_options),
-            connect_options=connect_subscriber1_options,
-            on_publish_callback_fn=self.subscriber1_callback,
-            on_lifecycle_event_stopped_fn=subscriber1_generic_callback.on_lifecycle_stopped,
-            on_lifecycle_event_attempting_connect_fn=subscriber1_generic_callback.on_lifecycle_attempting_connect,
-            on_lifecycle_event_connection_success_fn=subscriber1_generic_callback.on_lifecycle_connection_success,
-            on_lifecycle_event_connection_failure_fn=subscriber1_generic_callback.on_lifecycle_connection_failure
-        )
-        subscriber1_client = mqtt5.Client(client_options=subscriber1_options)
-
-        # subscriber 2
-        connect_subscriber2_options = mqtt5.ConnectPacket(client_id=client_id_subscriber2)
-        subscriber2_generic_callback = Mqtt5TestCallbacks()
-        subscriber2_options = mqtt5.ClientOptions(
-            host_name=input_host_name,
-            port=8883,
-            tls_ctx=io.ClientTlsContext(tls_ctx_options),
-            connect_options=connect_subscriber2_options,
-            on_publish_callback_fn=self.subscriber2_callback,
-            on_lifecycle_event_stopped_fn=subscriber2_generic_callback.on_lifecycle_stopped,
-            on_lifecycle_event_attempting_connect_fn=subscriber2_generic_callback.on_lifecycle_attempting_connect,
-            on_lifecycle_event_connection_success_fn=subscriber2_generic_callback.on_lifecycle_connection_success,
-            on_lifecycle_event_connection_failure_fn=subscriber2_generic_callback.on_lifecycle_connection_failure
-        )
-        subscriber2_client = mqtt5.Client(client_options=subscriber2_options)
-
-        # publisher
-        connect_publisher_options = mqtt5.ConnectPacket(client_id=client_id_publisher)
-        publisher_generic_callback = Mqtt5TestCallbacks()
-
-        publisher_options = mqtt5.ClientOptions(
-            host_name=input_host_name,
-            port=8883,
-            tls_ctx=io.ClientTlsContext(tls_ctx_options),
-            connect_options=connect_publisher_options,
-            on_lifecycle_event_stopped_fn=publisher_generic_callback.on_lifecycle_stopped,
-            on_lifecycle_event_attempting_connect_fn=publisher_generic_callback.on_lifecycle_attempting_connect,
-            on_lifecycle_event_connection_success_fn=publisher_generic_callback.on_lifecycle_connection_success,
-            on_lifecycle_event_connection_failure_fn=publisher_generic_callback.on_lifecycle_connection_failure
-        )
-        publisher_client = mqtt5.Client(client_options=publisher_options)
-
-        print("Connecting all 3 clients\n")
-        subscriber1_client.start()
-        subscriber1_generic_callback.future_connection_success.result(TIMEOUT)
-
-        subscriber2_client.start()
-        subscriber2_generic_callback.future_connection_success.result(TIMEOUT)
-
-        publisher_client.start()
-        publisher_generic_callback.future_connection_success.result(TIMEOUT)
-        print("All clients connected\n")
-
-        # Subscriber 1
-        subscriptions = []
-        subscriptions.append(mqtt5.Subscription(topic_filter=sharedTopicfilter, qos=mqtt5.QoS.AT_LEAST_ONCE))
-        subscribe_packet = mqtt5.SubscribePacket(
-            subscriptions=subscriptions)
-        subscribe_future = subscriber1_client.subscribe(subscribe_packet=subscribe_packet)
-        suback_packet1 = subscribe_future.result(TIMEOUT)
-        self.assertIsInstance(suback_packet1, mqtt5.SubackPacket)
-
-        # Subscriber 2
-        subscriptions2 = []
-        subscriptions2.append(mqtt5.Subscription(topic_filter=sharedTopicfilter, qos=mqtt5.QoS.AT_LEAST_ONCE))
-        subscribe_packet2 = mqtt5.SubscribePacket(
-            subscriptions=subscriptions2)
-        subscribe_future2 = subscriber2_client.subscribe(subscribe_packet=subscribe_packet2)
-        suback_packet2 = subscribe_future2.result(TIMEOUT)
-        self.assertIsInstance(suback_packet2, mqtt5.SubackPacket)
-
-        publishes = 10
-        for x in range(0, publishes):
-            packet = mqtt5.PublishPacket(
-                payload=f"{x}",
-                qos=mqtt5.QoS.AT_LEAST_ONCE,
-                topic=testTopic
-            )
-            publish_future = publisher_client.publish(packet)
-            publish_future.result(TIMEOUT)
-
-        self.all_packets_received.result(TIMEOUT)
-
-        topic_filters = []
-        topic_filters.append(testTopic)
-        unsubscribe_packet = mqtt5.UnsubscribePacket(topic_filters=testTopic)
-
-        unsubscribe_future = subscriber1_client.unsubscribe(unsubscribe_packet)
-        unsuback_packet = unsubscribe_future.result(TIMEOUT)
-        self.assertIsInstance(unsuback_packet, mqtt5.UnsubackPacket)
-
-        unsubscribe_future = subscriber2_client.unsubscribe(unsubscribe_packet)
-        unsuback_packet = unsubscribe_future.result(TIMEOUT)
-        self.assertIsInstance(unsuback_packet, mqtt5.UnsubackPacket)
-
-        self.assertEqual(self.total_callbacks, 10)
-
-        for e in self.received_subscriptions:
-            self.assertEqual(e, 1)
-
-        subscriber1_client.stop()
-        subscriber1_generic_callback.future_stopped.result(TIMEOUT)
-
-        subscriber2_client.stop()
-        subscriber2_generic_callback.future_stopped.result(TIMEOUT)
-
-        publisher_client.stop()
-        publisher_generic_callback.future_stopped.result(TIMEOUT)
-
-    def test_operation_will(self):
+    def _test_operation_will(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_HOST")
         input_cert = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_CERT")
         input_key = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_KEY")
@@ -1227,8 +1130,11 @@ class Mqtt5ClientTest(NativeResourceTest):
         client2.stop()
         callbacks2.future_stopped.result(TIMEOUT)
 
-    def do_will_correlation_data_test(self, outbound_correlation_data_bytes, outbound_correlation_data,
-                                      expected_correlation_data_bytes, expected_correlation_data):
+    def test_operation_will(self):
+        test_retry_wrapper(self._test_operation_will)
+
+    def _do_will_correlation_data_test(self, outbound_correlation_data_bytes, outbound_correlation_data,
+                                       expected_correlation_data_bytes, expected_correlation_data):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_HOST")
         input_cert = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_CERT")
         input_key = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_KEY")
@@ -1297,29 +1203,44 @@ class Mqtt5ClientTest(NativeResourceTest):
         client2.stop()
         callbacks2.future_stopped.result(TIMEOUT)
 
-    def test_will_correlation_data_bytes_binary(self):
+    def _test_will_correlation_data_bytes_binary(self):
         correlation_data = bytearray(os.urandom(64))
-        self.do_will_correlation_data_test(correlation_data, None, correlation_data, None)
+        self._do_will_correlation_data_test(correlation_data, None, correlation_data, None)
+
+    def test_will_correlation_data_bytes_binary(self):
+        test_retry_wrapper(self._test_will_correlation_data_bytes_binary)
+
+    def _test_will_correlation_data_bytes_string(self):
+        correlation_data = "CorrelationData"
+        correlation_data_as_bytes = correlation_data.encode('utf-8')
+        self._do_will_correlation_data_test(correlation_data, None, correlation_data_as_bytes, correlation_data)
 
     def test_will_correlation_data_bytes_string(self):
-        correlation_data = "CorrelationData"
-        correlation_data_as_bytes = correlation_data.encode('utf-8')
-        self.do_correlation_data_test(correlation_data, None, correlation_data_as_bytes, correlation_data)
+        test_retry_wrapper(self._test_will_correlation_data_bytes_string)
+
+    def _test_will_correlation_data_binary(self):
+        correlation_data = bytearray(os.urandom(64))
+        self._do_will_correlation_data_test(None, correlation_data, correlation_data, None)
 
     def test_will_correlation_data_binary(self):
-        correlation_data = bytearray(os.urandom(64))
-        self.do_correlation_data_test(None, correlation_data, correlation_data, None)
+        test_retry_wrapper(self._test_will_correlation_data_binary)
 
-    def test_will_correlation_data_string(self):
+    def _test_will_correlation_data_string(self):
         correlation_data = "CorrelationData"
         correlation_data_as_bytes = correlation_data.encode('utf-8')
-        self.do_correlation_data_test(None, correlation_data, correlation_data_as_bytes, correlation_data)
+        self._do_will_correlation_data_test(None, correlation_data, correlation_data_as_bytes, correlation_data)
+
+    def test_will_correlation_data_string(self):
+        test_retry_wrapper(self._test_will_correlation_data_string)
+
+    def _test_will_correlation_data_bytes_binary_precedence(self):
+        correlation_data = bytearray(os.urandom(64))
+        self._do_will_correlation_data_test(correlation_data, "Ignored", correlation_data, None)
 
     def test_will_correlation_data_bytes_binary_precedence(self):
-        correlation_data = bytearray(os.urandom(64))
-        self.do_correlation_data_test(correlation_data, "Ignored", correlation_data, None)
+        test_retry_wrapper(self._test_will_correlation_data_bytes_binary_precedence)
 
-    def test_operation_binary_publish(self):
+    def _test_operation_binary_publish(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_HOST")
         input_cert = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_CERT")
         input_key = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_KEY")
@@ -1381,8 +1302,11 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
-    def do_correlation_data_test(self, outbound_correlation_data_bytes, outbound_correlation_data,
-                                 expected_correlation_data_bytes, expected_correlation_data):
+    def test_operation_binary_publish(self):
+        test_retry_wrapper(self._test_operation_binary_publish)
+
+    def _do_correlation_data_test(self, outbound_correlation_data_bytes, outbound_correlation_data,
+                                  expected_correlation_data_bytes, expected_correlation_data):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_HOST")
         input_cert = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_CERT")
         input_key = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_KEY")
@@ -1434,33 +1358,48 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
-    def test_operation_publish_correlation_data_bytes_binary(self):
+    def _test_operation_publish_correlation_data_bytes_binary(self):
         correlation_data = bytearray(os.urandom(64))
-        self.do_correlation_data_test(correlation_data, None, correlation_data, None)
+        self._do_correlation_data_test(correlation_data, None, correlation_data, None)
+
+    def test_operation_publish_correlation_data_bytes_binary(self):
+        test_retry_wrapper(self._test_operation_publish_correlation_data_bytes_binary)
+
+    def _test_operation_publish_correlation_data_bytes_string(self):
+        correlation_data = "CorrelationData"
+        correlation_data_as_bytes = correlation_data.encode('utf-8')
+        self._do_correlation_data_test(correlation_data, None, correlation_data_as_bytes, correlation_data)
 
     def test_operation_publish_correlation_data_bytes_string(self):
-        correlation_data = "CorrelationData"
-        correlation_data_as_bytes = correlation_data.encode('utf-8')
-        self.do_correlation_data_test(correlation_data, None, correlation_data_as_bytes, correlation_data)
+        test_retry_wrapper(self._test_operation_publish_correlation_data_bytes_string)
+
+    def _test_operation_publish_correlation_data_binary(self):
+        correlation_data = bytearray(os.urandom(64))
+        self._do_correlation_data_test(None, correlation_data, correlation_data, None)
 
     def test_operation_publish_correlation_data_binary(self):
-        correlation_data = bytearray(os.urandom(64))
-        self.do_correlation_data_test(None, correlation_data, correlation_data, None)
+        test_retry_wrapper(self._test_operation_publish_correlation_data_binary)
 
-    def test_operation_publish_correlation_data_string(self):
+    def _test_operation_publish_correlation_data_string(self):
         correlation_data = "CorrelationData"
         correlation_data_as_bytes = correlation_data.encode('utf-8')
-        self.do_correlation_data_test(None, correlation_data, correlation_data_as_bytes, correlation_data)
+        self._do_correlation_data_test(None, correlation_data, correlation_data_as_bytes, correlation_data)
+
+    def test_operation_publish_correlation_data_string(self):
+        test_retry_wrapper(self._test_operation_publish_correlation_data_string)
+
+    def _test_operation_publish_correlation_data_bytes_binary_precedence(self):
+        correlation_data = bytearray(os.urandom(64))
+        self._do_correlation_data_test(correlation_data, "Ignored", correlation_data, None)
 
     def test_operation_publish_correlation_data_bytes_binary_precedence(self):
-        correlation_data = bytearray(os.urandom(64))
-        self.do_correlation_data_test(correlation_data, "Ignored", correlation_data, None)
+        test_retry_wrapper(self._test_operation_publish_correlation_data_bytes_binary_precedence)
 
     # ==============================================================
     #             OPERATION ERROR TEST CASES
     # ==============================================================
 
-    def test_operation_error_null_publish(self):
+    def _test_operation_error_null_publish(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_DIRECT_MQTT_HOST")
         input_port = int(_get_env_variable("AWS_TEST_MQTT5_DIRECT_MQTT_PORT"))
 
@@ -1479,7 +1418,10 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
-    def test_operation_error_null_subscribe(self):
+    def test_operation_error_null_publish(self):
+        test_retry_wrapper(self._test_operation_error_null_publish)
+
+    def _test_operation_error_null_subscribe(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_DIRECT_MQTT_HOST")
         input_port = int(_get_env_variable("AWS_TEST_MQTT5_DIRECT_MQTT_PORT"))
 
@@ -1498,7 +1440,10 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
-    def test_operation_error_null_unsubscribe(self):
+    def test_operation_error_null_subscribe(self):
+        test_retry_wrapper(self._test_operation_error_null_subscribe)
+
+    def _test_operation_error_null_unsubscribe(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_DIRECT_MQTT_HOST")
         input_port = int(_get_env_variable("AWS_TEST_MQTT5_DIRECT_MQTT_PORT"))
 
@@ -1517,7 +1462,10 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
-    def test_operation_rejoin_always(self):
+    def test_operation_error_null_unsubscribe(self):
+        test_retry_wrapper(self._test_operation_error_null_unsubscribe)
+
+    def _test_operation_rejoin_always(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_HOST")
         input_cert = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_CERT")
         input_key = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_KEY")
@@ -1553,11 +1501,14 @@ class Mqtt5ClientTest(NativeResourceTest):
         client2.stop()
         callbacks2.future_stopped.result(TIMEOUT)
 
+    def test_operation_rejoin_always(self):
+        test_retry_wrapper(self._test_operation_rejoin_always)
+
     # ==============================================================
     #             QOS1 TEST CASES
     # ==============================================================
 
-    def test_qos1_happy_path(self):
+    def _test_qos1_happy_path(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_HOST")
         input_cert = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_CERT")
         input_key = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_KEY")
@@ -1622,11 +1573,14 @@ class Mqtt5ClientTest(NativeResourceTest):
         client2.stop()
         callbacks2.future_stopped.result(TIMEOUT)
 
+    def test_qos1_happy_path(self):
+        test_retry_wrapper(self._test_qos1_happy_path)
+
     # ==============================================================
     #             RETAIN TEST CASES
     # ==============================================================
 
-    def test_retain_set_and_clear(self):
+    def _test_retain_set_and_clear(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_HOST")
         input_cert = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_CERT")
         input_key = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_KEY")
@@ -1711,11 +1665,14 @@ class Mqtt5ClientTest(NativeResourceTest):
         client3.stop()
         callbacks3.future_stopped.result(TIMEOUT)
 
+    def test_retain_set_and_clear(self):
+        test_retry_wrapper(self._test_retain_set_and_clear)
+
     # ==============================================================
     #             INTERRUPTION TEST CASES
     # ==============================================================
 
-    def test_interruption_sub(self):
+    def _test_interruption_sub(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_HOST")
         input_cert = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_CERT")
         input_key = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_KEY")
@@ -1750,7 +1707,10 @@ class Mqtt5ClientTest(NativeResourceTest):
 
         callbacks.future_stopped.result(TIMEOUT)
 
-    def test_interruption_unsub(self):
+    def test_interruption_sub(self):
+        test_retry_wrapper(self._test_interruption_sub)
+
+    def _test_interruption_unsub(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_HOST")
         input_cert = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_CERT")
         input_key = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_KEY")
@@ -1784,7 +1744,10 @@ class Mqtt5ClientTest(NativeResourceTest):
 
         callbacks.future_stopped.result(TIMEOUT)
 
-    def test_interruption_qos1_publish(self):
+    def test_interruption_unsub(self):
+        test_retry_wrapper(self._test_interruption_unsub)
+
+    def _test_interruption_qos1_publish(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_HOST")
         input_cert = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_CERT")
         input_key = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_KEY")
@@ -1821,11 +1784,14 @@ class Mqtt5ClientTest(NativeResourceTest):
 
         callbacks.future_stopped.result(TIMEOUT)
 
+    def test_interruption_qos1_publish(self):
+        test_retry_wrapper(self._test_interruption_qos1_publish)
+
     # ==============================================================
     #             MISC TEST CASES
     # ==============================================================
 
-    def test_operation_statistics_uc1(self):
+    def _test_operation_statistics_uc1(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_HOST")
         input_cert = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_CERT")
         input_key = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_KEY")
@@ -1876,6 +1842,9 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
+    def test_operation_statistics_uc1(self):
+        test_retry_wrapper(self._test_operation_statistics_uc1)
+
 
 if __name__ == 'main':
     unittest.main()
